/***************************************************************************
                          calias.cpp  -  One alias
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : t sep 10 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "calias.h"

#include "csendcmd.h"
#include "cgroup.h"

#include <kconfig.h>

cAlias::cAlias (cSendCmd *_sendcmd)
{
  sendcmd = _sendcmd;
  
  text = "";

  //default compare type is substring
  type = substring;
  //do not send original command by default
  sendoriginal = false;
  //whole words expansion
  wholewords = true;
  //include prefix/suffix
  includeprefixsuffix = true;
}

cAlias::~cAlias ()
{
  //nothing here
}

cSaveableField *cAlias::newInstance ()
{
  return new cAlias (sendcmd);
}

QString cAlias::performAction ()
{
  if (sendoriginal)
    sendcmd->addCommand (lasttext);
  QStringList::iterator it;
  commandstoexec.clear ();
  for (it = newtext.begin(); it != newtext.end(); ++it)
  {
    QString nt = *it;
    expandPseudoVariables (nt);
    QString newcommand;
    if (includeprefixsuffix)
      newcommand = prefix + nt + suffix;
    else
      newcommand = nt;
    commandstoexec.push_back (newcommand);
  }
  return QString::null;
}

bool cAlias::shouldContinue (bool match)
{
  //if our group is not activated, then we should always
  //continue (as if we hadn't existed)
  if ((!nogroup) && (ourgroup != NULL) && (!ourgroup->active()))
    return true;   

  //otherwise we should continue only if there was no match
  return (!match);
}

bool cAlias::match (const QString &what)
{
  //no matching if the group is not activated
  if ((!nogroup) && (ourgroup != NULL) && (!ourgroup->active()))
    return false;

  bool b = cSaveableField::match (what);
  if (b)
  {
    //no match if it was a part of a word and wholewords is on!
    if (wholewords)
    {
      if ((prefix.length () > 0) && (!(prefix[prefix.length()-1].isSpace ())))
        b = false;
      if ((suffix.length () > 0) && (!(suffix[0].isSpace ())))
        b = false;
    }
  }
  return b;
}

void cAlias::load (KConfig *config, const QString &group)
{
  config->setGroup (group);
  setText (config->readEntry ("Text", ""));

  //ntxt: for compatibility with KMuddy <= 0.6pre1
  QString ntxt = config->readEntry ("Replacement text", QString::null);

  newtext.clear();
  int repcount = config->readNumEntry ("Replacement count", -1);
  if (repcount == -1)   //use old-style replacement
    newtext.push_back (ntxt);
  else    //new-style replacement
    for (int i = 1; i <= repcount; i++)
    {
      QString repline = config->readEntry ("Replacement line " +
            QString::number (i), "");
      newtext.push_back (repline); 
    }

  setType (config->readNumEntry ("Type", substring));
  includeprefixsuffix = config->readBoolEntry ("Include prefix suffix", true);
  sendoriginal = config->readBoolEntry ("Send original", false);
  wholewords = config->readBoolEntry ("Whole words", true);
  _group = config->readNumEntry ("Group ID", 0);
}

void cAlias::save (KConfig *config, const QString &group)
{
  config->setGroup (group);
  config->writeEntry ("Text", text);
  //remove obsolete entry
  config->deleteEntry ("Replacement text");

  int repcount = newtext.count();
  config->writeEntry ("Replacement count", repcount);
  QStringList::iterator it = newtext.begin();
  for (int i = 1; i <= repcount; i++)
  {
    config->writeEntry ("Replacement line " + QString::number (i), *it);
    it++;
  }
  
  config->writeEntry ("Type", type);
  config->writeEntry ("Send original", sendoriginal);
  config->writeEntry ("Include prefix suffix", includeprefixsuffix);
  config->writeEntry ("Whole words", wholewords);
  config->writeEntry ("Group ID", _group);
}
