/***************************************************************************
                          cansiparser.cpp  -  ANSI parser
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : P Jun 21 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define CANSIPARSER_CPP

#include "cansiparser.h"

#include <ctype.h>

#include "ctelnet.h"

cANSIParser::cANSIParser()
{
  buffer = "";
  useansi = true;
}

cANSIParser::~cANSIParser()
{
  //nothing for now... But maybe I'll need this later (?)
}

void cANSIParser::parseText (const QString &data)
{
  //buffer may contain unfinished ANSI sequences, so we add new input to it...
  buffer += data;
  //this will contain ansi commands
  QString ansicmd = "";
  //this will contain plain text
  QString text = "";
  bool inANSI = false;
  int len = buffer.length ();
  int processed = -1;   //variable processed has the same role as one in cTelnet
  for (int i = 0; i < len; i++)
  {
    if (inANSI)
    {
      char ch = buffer[i].latin1();
      ansicmd += ch;
      //ANSI commands end with a letter...
      if (((ch >= 'a') && (ch <= 'z')) || ((ch >= 'A') && (ch <= 'Z')))
      {
        inANSI = false;
        processed = i;
        if (ch == 'm')
           //ANSI command that changed color; other commands are
           //not interesting (for us at least ;-))...
        {
          //the text that preceeded this command should be displayed in old
          //colors - make it happen!
          emit plainText (text);
          text = "";  //old text is no longer needed

          //the ANSI command is now in ansicmd...
          //we parse it, getting all integers and signaling them (each
          //integer in this cmd is a code for ANSI color)
          int color = 0;
          int innumber = false;
          int alen = ansicmd.length ();
          int codeCount = 0;

          for (int j = 0; j < alen; j++)
          {
            char nn = ansicmd[j].latin1();
            if (isdigit (nn)) //a digit is here
            {
              if (!innumber)
              {
                innumber = true;
                color = 0;
              }
              color = color * 10 + (nn - 48); //ASCII code for '0' is 48, ...
            }
            else  //something else is here
            {
              if (innumber)
              {
                //only emit the signal if we use ANSI colors
                if (useansi)
                  emit newColor (color);
                color = 0;
                codeCount++;
                innumber = false;
              }
            }
          }
          if (codeCount == 0)
            // ESC[m should be treated as ESC[0m
            if (useansi)
              emit newColor (0);
          
          //the text ends in a 'm', so that we don't have to worry about numbers
          //ending at the end of the string...
        }
        emit (ansiSequence (ansicmd));
        ansicmd = "";
      }
    }
    else
    {
      if (buffer[i].latin1() == 27)   //new ANSI command begins...
      {
        inANSI = true;
        //start new ANSI command
        ansicmd = buffer[i];
      }
      else
      {
        text += buffer[i];    //this is normal text, no ANSI...
        processed = i;
      }
    }
  }
  emit plainText (text);
  text = "";  //old text is no longer needed
  buffer.remove (0, processed + 1);  //remove processed data from the buffer
}

void cANSIParser::flush ()
{
  buffer = "";
}
