/***************************************************************************
                          cansiparser.h  -  ANSI parser
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : P Jun 21 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*
Current status of cANSIParser (Nov 1, 2002):
 - all required functionality has been implemented
 - new features may still be added
 - code has been tested and it works
*/

#ifndef CANSIPARSER_H
#define CANSIPARSER_H

#include <qobject.h>

#include "csessionbase.h"

//some #defines - ANSI colors
//constants are #defined here, but they're used in KMUDConsole widget
#define ANSI_NORMAL 0    //this resets everything to normal (incl. text/bk. color)
#define ANSI_BOLD 1      //shows text in brighter color
#define ANSI_UNDERLINE 4 //ignored
#define ANSI_BLINK 5     //the same as ANSI_BOLD
#define ANSI_ITALICS 7   //ignored
#define ANSI_INVISIBLE 8 //the same as ANSI_TEXT_BLACK + ANSI_BK_BLACK

#define ANSI_TEXT_BLACK 30
#define ANSI_TEXT_RED 31
#define ANSI_TEXT_GREEN 32
#define ANSI_TEXT_YELLOW 33
#define ANSI_TEXT_BLUE 34
#define ANSI_TEXT_MAGENTA 35
#define ANSI_TEXT_CYAN 36
#define ANSI_TEXT_WHITE 37

#define ANSI_BK_BLACK 40
#define ANSI_BK_RED 41
#define ANSI_BK_GREEN 42
#define ANSI_BK_YELLOW 43
#define ANSI_BK_BLUE 44
#define ANSI_BK_MAGENTA 45
#define ANSI_BK_CYAN 46
#define ANSI_BK_WHITE 47


/**
ANSI parser - slot is connected to cTelnet, output goes to displaying widget
and also to some other classes that handle triggers, automapper etc.
  *@author Tomas Mecir
  */

class cANSIParser : public QObject, public cSessionBase  {
  Q_OBJECT
public: 
  cANSIParser();
  ~cANSIParser();

  void setUseAnsi (bool val) { useansi = val; };
signals:
  void newColor (int color);
  void plainText (const QString &text);
  /** ansiSequence is here for those who want to include ANSI sequences in session transcript */
  void ansiSequence (const QString &text);
public slots:
  void parseText (const QString &data);
  void flush ();
protected:
  QString buffer;
  bool useansi;
};

#endif
