/***************************************************************************
                          cconsole.cpp  -  main displaying widget
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : So Jun 22 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define CCONSOLE_CPP

//Better don't try to find function definitions manually, this is large.

#include "cconnection.h"
#include "cconsole.h"
#include "csession.h"
#include "cprompt.h"
#include "cstatus.h"
#include "ctextprocessor.h"
#include "cconnprefs.h"
#include "kmuddy.h"

#include <qclipboard.h>
#include <qcursor.h>
#include <qfile.h>
#include <qfontmetrics.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qtextstream.h>
#include <qtimer.h>
#include <kapplication.h>
#include <kglobalsettings.h>
#include <klocale.h>

#include <ctype.h>

#define BLINK_INTERVAL 1000

extern KMuddy *kmuddy;

int cConsole::initHistory = 1000;

//don't know why min() macro is not in standard headers...
#define min(a,b) ((a)  < (b) ? (a) : (b))

inline void swap (int &a, int &b)
{
  int tmp = a;
  a = b;
  b = tmp;
}

// -------------- cHistoryBuffer ------------

//I'm disabling this code, because I keep getting all sorts of strange
//crashes here, and I have no idea why :(
/*

template<class T>
inline cHistoryBuffer<T>::cHistoryBuffer(const int bufferMaxSize)
{
   if (bufferMaxSize > 0)
   {
      buffer = new T[bufferMaxSize];
   } // endif specified buffer size
   else
   {
      buffer = NULL;
   } // endelse no buffer size

   bufSize   = bufferMaxSize;
   start     = 0;
   numItems  = 0;

} // cHistoryBuffer<T>::cHistoryBuffer

template<class T>
inline cHistoryBuffer<T>::~cHistoryBuffer()
{
   delete [] buffer;
} // cHistoryBuffer<T>::cHistoryBuffer

template<class T>
inline T &cHistoryBuffer<T>::operator[](const int offset)
{
   return buffer[(start+offset)%bufSize];

} // cHistoryBuffer<T>::operator[]

template<class T>
inline void cHistoryBuffer<T>::add(const T &item)
{
   int index = (start+numItems)%bufSize;

   buffer[index] = item;

   if (numItems < bufSize)
   {
      numItems++;
   } // endif buffer not filled yet
   else
   {
      start++;
      start %= bufSize;
   } // endelse buffer full

} // cHistoryBuffer<T>::add

template<class T>
inline void cHistoryBuffer<T>::resize(const int bufferMaxSize)
{
   T   *newBuffer;
   int  newBufSize = bufferMaxSize;
   int  copyOffset;

   if (bufferMaxSize <= 0)
   {
      newBuffer  = NULL;
      start      = 0;
      numItems   = 0;
      newBufSize = 0;
   } // endif zero buffer
   else
   {
      newBufSize = bufferMaxSize;
      newBuffer  = new T[newBufSize];

      if (newBufSize < bufSize)
      {
         copyOffset = bufSize-newBufSize;
      } // endif smaller buffer
      else
      {
         copyOffset = 0;
      } // endelse bigger buffer

      if (numItems > newBufSize)
      {
         numItems = newBufSize;
      } // endif too many items for smaller buffer

      if (numItems > 0)
      {
         for (int index = 0;index < newBufSize;index++)
         {
            newBuffer[index] = (*this)[copyOffset++];
         } // endfor more info left to copy

      } // endif existing contents to copy

   } // endelse specified buffer size

   delete [] buffer;
   buffer  = newBuffer;
   bufSize = newBufSize;

} // cHistoryBuffer<T>::resize

template<class T>
inline void cHistoryBuffer<T>::flush()
{
   start      = 0;
   numItems   = 0;

} // cHistoryBuffer<T>::flush
*/

// ----------------- cConsole ---------------


cConsole::cConsole (bool auxiliary, QWidget *parent, const char *name ) :
      QGridView (parent, name, WStaticContents | WRepaintNoErase)
{
  aux = auxiliary;
  aconvisible = false;
  
  echocolor = CL_YELLOW | CL_BRIGHT;
  systemcolor = CL_CYAN | CL_BRIGHT;
  scriptoutputcolor = CL_WHITE;
  usedrows = 0;
  currow = -1;
  currowpos = -1;
  
  currows = curcols = 0;
  charCount = charWidth = 0;
  
  if (!aux)
    historySize = initHistory;  //set history size; won't be modified if
    //user changes that setting in global prefs, that'll only affect new
    //connections
  else
    historySize = 100;        //aux. console does not need that big history
  QGridView::setNumCols (1);

  wordWrapping = true;
  indentValue = 0;
  
  //prepare the cache
  for (int i = 0; i < MAXCACHE; i++)
  {
    cacheEntry[i] = 0;
    cacheBlinkOn[i] = 0;
    cacheBlinkOff[i] = 0;
  }

  //blinking
  blinkPhase = true;
  blinking = false;
  blinkTimer = 0;
  
  //some memory is allocated here:
  typedef unsigned char *uchar;
  typedef QChar *uqchar;
  cell = new uqchar[historySize];
  tcolor = new uchar[historySize];
  bcolor = new uchar[historySize];
/*
  cell.resize(historySize);
  tcolor.resize(historySize);
  bcolor.resize(historySize);
*/
  for (int i = 0; i < historySize; i++)
  {
    cell[i] = 0;
    tcolor[i] = 0;
    bcolor[i] = 0;
  }
  selected = false;
  canselect = false;

  //default color scheme:
  mycolor[CL_BLACK] = black;
  mycolor[CL_RED] = darkRed;
  mycolor[CL_GREEN] = darkGreen;
  mycolor[CL_YELLOW] = darkYellow;
  mycolor[CL_BLUE] = darkBlue;
  mycolor[CL_MAGENTA] = darkMagenta;
  mycolor[CL_CYAN] = darkCyan;
  mycolor[CL_WHITE] = lightGray;

  mycolor[CL_BRIGHT | CL_BLACK] = darkGray;
  mycolor[CL_BRIGHT | CL_RED] = red;
  mycolor[CL_BRIGHT | CL_GREEN] = green;
  mycolor[CL_BRIGHT | CL_YELLOW] = yellow;
  mycolor[CL_BRIGHT | CL_BLUE] = blue;
  mycolor[CL_BRIGHT | CL_MAGENTA] = magenta;
  mycolor[CL_BRIGHT | CL_CYAN] = cyan;
  mycolor[CL_BRIGHT | CL_WHITE] = white;

  //background color
  int defbkcolor = CL_BLACK;
  viewport()->setPaletteBackgroundColor (mycolor[defbkcolor]);
  setPaletteBackgroundColor (mycolor[defbkcolor]);
  setBackgroundMode (Qt::FixedColor);

  //size policy
  QSizePolicy qsp (QSizePolicy::Expanding, QSizePolicy::Expanding);
  setSizePolicy (qsp);
  
  //enable vertical scrollbar, disable the other one
  setHScrollBarMode (AlwaysOff);
  if (!aux)
    setVScrollBarMode (AlwaysOn);
  else
    setVScrollBarMode (AlwaysOff);

  //create our auxiliary console if needed
  if (!aux)
  {
    aconsole = new cConsole (true, this);
    aconsole->hide ();
    aconvisible = false;
    aconsize = 25;  //aconsole covers 25% by default
    
    //and connect() slider so that aconsole is shown/hidden as needed
    connect (verticalScrollBar (), SIGNAL (sliderMoved (int)), this, SLOT (sliderChanged (int)));
    connect (verticalScrollBar (), SIGNAL (valueChanged (int)), this, SLOT (sliderChanged (int)));
  }
  else
    aconsole = 0;

  //set default system fixed font
  setFont (KGlobalSettings::fixedFont ()); //default system fixed font

  //set the I-beam cursor
  viewport()->setCursor (QCursor(IbeamCursor));

  //fix output
  fixupOutput ();
}

cConsole::~cConsole()
{
  if (blinkTimer != 0)
  {
    blinkTimer->stop();
    delete blinkTimer;
  }
  emptyCache ();
  if (!aux)
  {
    delete aconsole;
  }
  for (int i = 0; i < historySize; i++)
  {
    delete[] bcolor[i];
    delete[] tcolor[i];
    delete[] cell[i];
    bcolor[i] = 0;
    tcolor[i] = 0;
    cell[i] = 0;
  }
  delete[] bcolor;
  delete[] tcolor;
  delete[] cell;
}

cConsole *cConsole::auxConsole ()
{
  if (aux)
    return 0;
  else
    return aconsole;
}

QColor cConsole::color (int index)
{
  if ((index >= 0) && (index <= 15))
    return mycolor[index];
  else
    return white;    //returns white for invalid parameter values
}

void cConsole::setColor (QColor color, int index)
{
  if ((index >= 0) && (index <= 15))
  {
    mycolor[index] = color;
    //repaint console to change colors
    emptyCache ();

    int defbkcolor = sess->textproc()->defaultBkColor();
    setPaletteBackgroundColor (mycolor[defbkcolor]);
    viewport()->setPaletteBackgroundColor (mycolor[defbkcolor]);
    repaintContents (false);
    if (!aux)
      aconsole->setColor (color, index);
  }
}

void cConsole::setFont (QFont f)
{
  myfont = f;
  QFontMetrics fm (myfont);
  descent = fm.descent ();
  charWidth = fm.width ("m");
  setCellHeight (fm.height () + 2);

  ///fix output window
  fixupOutput ();
  
  //and repaint everything
  repaintContents (false);
  if (!aux)
    aconsole->setFont (f);

  //changing font often causes view to move - move to the very bottom
  if (!aux)
    verticalScrollBar()->setValue (verticalScrollBar()->maxValue());
}

void cConsole::setDefaultBkColor (int index)
{
  setPaletteBackgroundColor (mycolor[index]);
  viewport()->setPaletteBackgroundColor (mycolor[index]);
  emptyCache ();
  repaintContents (false);
  if (!aux)
    aconsole->setDefaultBkColor (index);
}

void cConsole::setEchoColor (int index)
{
  echocolor = index;
  //no need to do this for aconsole
}

void cConsole::setPromptLabel (bool value)
{
  promptlabel = value;
  if(value == true)
  {
    if(sess != NULL)
    {
      sess->prompt()->show();
      sess->status()->clearPartialLine();
    }
  }
  else
  {
    if(sess != NULL)
      sess->prompt()->hide();
  }
}

void cConsole::setSystemColor (int index)
{
  systemcolor = index;
  //no need to do this for aconsole
}

void cConsole::setScriptOutputColor (int index)
{
  scriptoutputcolor = index;
  //no need to do this for aconsole
}

void cConsole::setEnableEcho (bool value)
{
  cmdecho = value;
  //no need to do this for aconsole
}

void cConsole::setEnableMessages (bool value)
{
  messages = value;
  //no need to do this for aconsole
}

void cConsole::setWordWrapping (bool val)
{
  wordWrapping = val;
  if (!aux)
    aconsole->setWordWrapping (val);
}

void cConsole::setIndentation (int val)
{
  //indentation must not exceed 10 characters
  if ((val >= 0) && (val <= 10))
    indentValue = val;
  if (!aux)
    aconsole->setIndentation (val);
}

void cConsole::setEnableBlinking (bool value)
{
  //look if this changes blinking status
  if (blinking == value)
    return;
  
  //if so, update blinking timer status
  blinking = value;
  if (blinking)
  {
    if (blinkTimer == 0)
    {
      blinkTimer = new QTimer (this);
      connect (blinkTimer, SIGNAL (timeout()), this, SLOT (blinkTimerTimeout ()));
    }
    //blinking will occur once per second
    blinkTimer->start (BLINK_INTERVAL);
    //we start with characters displayed
    blinkPhase = true;
  }
  else
  {
    if (blinkTimer != 0)
    {
      blinkPhase = true;
      blinkTimer->stop ();
      //if status goes from enabled to disabled, we may need to repaint - do it!
      updateContents ();
    }
  }  

  if (!aux)
    aconsole->setEnableBlinking (value);
}

void cConsole::clear ()
{
  emptyCache ();
  for (int i = 0; i < historySize; i++)
  {
    delete[] cell[i];
    delete[] tcolor[i];
    delete[] bcolor[i];
    cell[i] = 0;
    tcolor[i] = 0;
    bcolor[i] = 0;
  }
  int oldHistorySize = historySize;
  
  //update history size...
  if (!aux)
    historySize = initHistory;
  else
    historySize = 100;        //aux. console does not need that big history

  aconsize = 25;  //aconsole covers 25% by default

  usedrows = 0;
  currow = -1;
  currowpos = -1;
  
  setNumRows (0);

  if (oldHistorySize != historySize)
  {
    //reallocate memory
    delete[] cell;
    delete[] tcolor;
    delete[] bcolor;

    typedef unsigned char *uchar;
    typedef QChar *uqchar;
    cell = new uqchar[historySize];
    tcolor = new uchar[historySize];
    bcolor = new uchar[historySize];

    for (int i = 0; i < historySize; i++)
    {
      cell[i] = 0;
      tcolor[i] = 0;
      bcolor[i] = 0;
    }
  }
   
  /*
  cell.resize (historySize);
  tcolor.resize (historySize);
  bcolor.resize (historySize);

  cell.flush();
  tcolor.flush();
  bcolor.flush();
  */
  
  repaintContents ();
  if (!aux)
    aconsole->clear ();

  //fix output
  fixupOutput ();

  //this processEvents() MUST be here! - otherwise aconsole gets shown on
  //second+ connection... (problem with scrollbar's maxvalue)
  (KApplication::kApplication ())->processEvents ();
  if (!aux)
    sliderChanged (0);  //this one is needed as well
}

void cConsole::addLine (const QString &text, const QString &tc,
    const QString &bc)
{
  addNewText (text, tc, bc, true);
}

void cConsole::addText (const QString &text, const QString &tc,
    const QString &bc)
{
  cConnPrefs *cp = sess->connection()->getConnPrefs(); 
  if(cp && cp->consolePrompt())
    addNewText (text, tc, bc, false);
}

void cConsole::addNewText (const QString &text, const QString &tc,
    const QString &bc, bool endTheLine)
{
  //disable repainting
  setUpdatesEnabled (false);

  pendingLineMove = 0;
  
  //first of all, if we're writing the very first string
  if (currow == -1)
    endLine ();    //prepare first line
    
  int start = -1, end = -1;
  int len = text.length();

  //we'll go line after line (visible line, not received line), trying to
  //find wrap-position, then we copy that part of the string to another
  //string and display it
  while (start < len)
  {
    //delete current line from the redraw cache
    for (int j = MAXCACHE - 1; j >= 0; j--)
      if ((cacheBlinkOn[j] != 0) && (cacheEntry[j] == currow))
        deleteCacheEntry (j);

    //second+ line and not at the end -> starting at indentValue
    if ((start > -1) && (end + 1 < len))
      currowpos = indentValue;

    //compute new start/end values
    start = end + 1;
    if (start < len)  //only if we haven't reached end of string
    {
      end = start + (charCount - currowpos) - 1;
      if (end >= len)   //reached end of string now
        end = len - 1;
      else              //otherwise it's okay, unless we need to wordwrap
      {
        if (wordWrapping)
          if (text[end + 1] != ' ') //okay if next char is a space
          {
            int _e = end;
            while (_e >= start)
              if (text[_e] == ' ')  //wrap here
              {
                end = _e;
                break;
              }
              else
                _e--;
            if (_e < start)   //no suitable wrap-position found
              if (currowpos > indentValue)
                  //something is already on the line, probably a prompt
                end = start - 1;  //we'll try luck next time
              //else we write as much as possible (it's one single word)
          }
      }
      //Okay. start/end determined, let's write!
      for (int i = start; i <= end; i++)
      {
        cell[currow][currowpos] = text[i];
        tcolor[currow][currowpos] = (unsigned char) tc[i].latin1();
        bcolor[currow][currowpos] = (unsigned char) bc[i].latin1();
        currowpos++;
      }

      if (end == len - 1) //last part of string
      {
        if (endTheLine)   //we should end the line -> do it!
          endLine ();
        else
          if (currowpos >= charCount - 1)
              //otherwise end the line if we've reached its end
            endLine ();
      }
      else
        endLine ();   //goin' to new line
    }
  }

  //end the line if the text is empty and we're asked to do so
  // (it hasn't been done yet in such case)
  if (endTheLine && (len == 0))
    endLine ();
  
  //increase number of rows if needed; it's not in endLine for speed's sake
  if (numRows() < currow + 1)
    setNumRows (currow + 1);

  //ensure cell visibility if needed; it's not in endLine for speed's sake
  if (aux || (!aconvisible))
    ensureCellVisible (currow, 0);
  else
    if ((currow == historySize - 1) && pendingLineMove)
      //contents have moved pendingLineMove rows up
      setContentsPos (0, contentsY()-(pendingLineMove*cellHeight()));

  //enable repainting here
  setUpdatesEnabled (true); 

  //request repaint here
  updateContents ();

  if (!aux)
  {
    aconsole->addNewText (text, tc, bc, endTheLine);
    kmuddy->displayedSomeText (sess);
  }
}

void cConsole::handleBufferShift()
{
  //move selection if needed
  if (selected)
  {
    selrow1--;
    selrow2--;

    if (selrow1 < 0) selrow1 = 0;
    if (selrow2 < 0) selrow2 = 0;
  }

  //no cache for aconsole
  if (!aux)
  {
    for (int i = 0; i < MAXCACHE; i++)
    {
      if (cacheBlinkOn[i] != 0)
      {
        cacheEntry[i]--;
        if (cacheEntry[i] < 0)
          deleteCacheEntry (i);
      }
    }
  }

  //shift buffer
  delete[] cell[0];
  delete[] tcolor[0];
  delete[] bcolor[0];

  for (int i = 1; i < historySize; i++)
  {
    //this was done automatically by cHistoryBuffer
    cell[i - 1] = cell[i];
    tcolor[i - 1] = tcolor[i];
    bcolor[i - 1] = bcolor[i];
  }
}

void cConsole::endLine ()
{
  currowpos = 0;
  pendingLineMove++;
  if (currow < historySize - 1)
    currow++;
  else
    handleBufferShift ();

  //cell.add(), ... was here when using cHistoryBuffer
  cell[currow] = new QChar[charCount];
  tcolor[currow] = new unsigned char[charCount];
  bcolor[currow] = new unsigned char[charCount];

  //fill new line with spaces
  for (int i = 0; i < charCount; i++)
  {
    cell[currow][i] = QChar (' ');
    if(sess != NULL)
    {
      tcolor[currow][i] = sess->textproc()->defaultTextColor();
      bcolor[currow][i] = sess->textproc()->defaultBkColor();
    }
  }

  usedrows = currow + 1;
}

void cConsole::forceBeginOfLine ()
{
  if (currowpos > 0)
    endLine ();
}

void cConsole::addCommand (const QString &text)
//addText plus some colouring...
{
  //IMPORTANT: the signal emitted here is also captured by session transcript
  //so, if you disable cmd echo here, it won't work with session
  //transcript!!!

  if (sess->connection()->connectionClosed())
  {
    int sessNum = kmuddy->getSessionNumber(sess);
    if (sessNum != -1)
      kmuddy->removeSession (sessNum, false);
  }
  else
  if (cmdecho)  //only if command echo is enabled
  {
    QString tcols, bcols;
    QChar t = QChar (echocolor);
    QChar b = QChar (sess->textproc()->defaultBkColor());
    int len = text.length();
    for (int i = 0; i < len; i++)
    {
      tcols += t;
      bcols += b;
    }
    addLine (text, tcols, bcols);
    emit commandDisplayed (text);
  }
}

void cConsole::systemMessage (const QString &text)
//addText plus some colouring...
{
  if (messages) //only if messages are enabled
  {
    QString tcols, bcols;
    QChar t = QChar (systemcolor);
    QChar b = QChar (sess->textproc()->defaultBkColor());
    int len = text.length();
    for (int i = 0; i < len; i++)
    {
      tcols += t;
      bcols += b;
    }
    forceBeginOfLine();
    addLine (text, tcols, bcols);
  }
}

void cConsole::addScriptOutput (const QString &text)
//addText plus some colouring...
{
  QString tcols, bcols;
  QChar t = QChar (scriptoutputcolor);
  QChar b = QChar (sess->textproc()->defaultBkColor());
  int len = text.length();
  for (int i = 0; i < len; i++)
  {
    tcols += t;
    bcols += b;
  }
  forceBeginOfLine();
  addLine (text, tcols, bcols);
}

void cConsole::decisionMessage (const QString &text)
//addText plus some colouring...
{
  QString tcols, bcols;
  QChar t = QChar (systemcolor);
  QChar b = QChar (sess->textproc()->defaultBkColor());
  int len = text.length();
  for (int i = 0; i < len; i++)
  {
    tcols += t;
    bcols += b;
  }
  forceBeginOfLine();
  addLine (text, tcols, bcols);
}

void cConsole::makeDecision ()
{
  QString s;
  s = i18n ("My decision");
  s += ": ";
  QString ss;
  //generate a random number in 0..9 range
  int which = kapp->random() % 10;
  switch (which) {
    case 0: ss = i18n ("No, no, no!"); break;
    case 1: ss = i18n ("I don't agree with it."); break;
    case 2: ss = i18n ("Better not."); break;
    case 3: ss = i18n ("I'd probably suggest to reject."); break;
    case 4: ss = i18n ("Saying no seems a bit better."); break;
    case 5: ss = i18n ("Saying yes seems a bit better."); break;
    case 6: ss = i18n ("I'd probably suggest to accept."); break;
    case 7: ss = i18n ("Sounds good."); break;
    case 8: ss = i18n ("I agree with it."); break;
    case 9: ss = i18n ("Definitely yes!"); break;
  };
  s += ss;
  //now display that decision
  decisionMessage (s);
}

void cConsole::dumpBuffer (bool fromcurrent, FILE *file)
{
  //NOTE: buffer dumping does NOT support ANSI sequences!!!
  //file must be opened for writing!

  //get visible range
  int y = contentsY();
  int row = rowAt (y);
  int start = fromcurrent ? row : 0;

  int cols = numCols();
  for (int i = start; i <= currow; i++)
  {
    //proceed line by line
    QString line = QString::null;
    for (int j = 0; j < cols; j++)
      line += cell[i][j];
    //remove trailing spaces from that string (but not leading ones)
    int column = cols - 1;
    while (column >= 0)
    {
      if (!(line[column].isSpace()))
        break;
      column--;
    }
    line = line.left (column + 1);
    fputs (line, file);
    fputc ('\n', file);
  }
}

void cConsole::getAllColors (int *_echo, int *_system)
//this is used by cTranscript class to determine correct colors for command output
{
  *_echo = echocolor;
  *_system = systemcolor;
}

void cConsole::paintCell (QPainter *p, int row, int)
{
  if (row < usedrows)     //only the first "usedrows" rows contain data
  {
    QColor paintColor, fillColor;
    QFont paintFont = myfont;
    
    int cw = QGridView::cellWidth ();
    int ch = cellHeight ();

    QPixmap *pix = 0, *pix2 = 0;

    //if this line is selected (partially selected), do NOT use cache!
    bool forceNoCache = false;
    int ss1, ss2;
    ss1 = selrow1;
    ss2 = selrow2;
    if (ss1 > ss2)
      swap (ss1, ss2);
    if (selected && (ss1 <= row) && (ss2 >= row))
      forceNoCache = true;
    if (aux)  //no cache for auxiliary console
      forceNoCache = true;
      
    //try to find the pixmap in the cache first...
    bool foundInCache = false;
    if (!forceNoCache)
      for (int i = 0; i < MAXCACHE; i++)
        if ((cacheBlinkOn[i] != NULL) && (cacheEntry[i] == row))
        {
          foundInCache = true;
          if (blinking)
          {
            pix = cacheBlinkOn[i];
            pix2 = cacheBlinkOff[i];
          }
          else
            pix = cacheBlinkOn[i];
          break;
        }
            
    //look if there are any blinking chars
    bool hasblink = hasBlink (row);
    //if search fails, we have to draw that line :(
    //if we'll be putting it into cache and there are blinking chars in the
    //line, then we'll construct both blinkOn and blinkOff line
    if (!foundInCache)
    {
      //We create both entries even if blinking is disabled, because user
      //can enable it at any time. This can have impact on performance -
      //maybe I'll change this behavior?
      
      pix = new QPixmap;
      pix->resize (cw, ch);
      QPainter *p2 = new QPainter (pix);
      p2->setBackgroundMode (OpaqueMode);

      QPainter *p2b = 0;
      if (hasblink)
      {
        //pix2 and p2b represent pixmap with hidden blinking letters
        pix2 = new QPixmap;
        pix2->resize (cw, ch);
        p2b = new QPainter (pix2);
        p2b->setBackgroundMode (OpaqueMode);
      }
      
      //first create an array of colors
      char *bcolors = new char[charCount];
      char *fcolors = new char[charCount];
      for (int i = 0; i < charCount; i++)
      {
        bool isselected = false;  //is this cell selected?
        if (selected)
        {
          int mypos = row * charCount + i;
          int selstart = selrow1 * charCount + selrowpos1;
          int selend = selrow2 * charCount + selrowpos2;
          if (selstart > selend)
            swap (selstart, selend);
          if ((selstart <= mypos) && (mypos < selend))
            isselected = true;
        }
        //tcolor without attributes
        int tcol = tcolor[row][i] % 16;
        int bcol = bcolor[row][i] % 16;
        bcolors[i] = isselected?tcol:bcol;
        fcolors[i] = isselected?bcol:tcol;
        
      }

      //draw the background
      int selstart = 0;
      int selcolor = bcolors[0];
      for (int i = 1; i < charCount; i++)
        if (bcolors[i] != selcolor) //do some painting!
        {
          fillColor = mycolor[selcolor];
          p2->fillRect (charWidth * selstart, 0, charWidth * i, cellHeight(), fillColor);
          if (hasblink)
            p2b->fillRect (charWidth * selstart, 0, charWidth * i, cellHeight(), fillColor);
          selstart = i;
          selcolor = bcolors[i];
        }
      fillColor = mycolor[selcolor];
      p2->fillRect (charWidth * selstart, 0, charWidth * charCount, cellHeight(), fillColor);
      if (hasblink)
        p2b->fillRect (charWidth * selstart, 0, charWidth * charCount, cellHeight(), fillColor);
      
      //then the text
      p2->setFont (paintFont);
      if (hasblink)
        p2b->setFont (paintFont);

      int curcolor = fcolors[0];
      int curbcolor = bcolors[0];
      bool charUnderlined = (tcolor[row][0] & 16) ? true : false;
      bool charItalics = (tcolor[row][0] & 32) ? true : false;
      bool charBlinks = (tcolor[row][0] & 64) ? true : false;
      bool charStrikeOut = (tcolor[row][0] & 128) ? true : false;
      int startat = 0;
      QString text = "";
      for (int i = 0; i < charCount; i++)
      {
        bool curCharUnderlined = (tcolor[row][i] & 16) ? true : false;
        bool curCharItalics = (tcolor[row][i] & 32) ? true : false;
        bool curCharBlinks = (tcolor[row][i] & 64) ? true : false;
        bool curCharStrikeOut = (tcolor[row][i] & 128) ? true : false;
        if ((fcolors[i] != curcolor) || (bcolors[i] != curbcolor) ||
            (charBlinks != curCharBlinks) ||
            (charUnderlined != curCharUnderlined) ||
            (charItalics != curCharItalics) ||
            (charStrikeOut != curCharStrikeOut))
        {
          paintColor = mycolor[curcolor];
          fillColor = mycolor[curbcolor];
          //font attributes
          paintFont.setUnderline (charUnderlined ? true : myfont.underline());
          paintFont.setItalic (charItalics ? true : myfont.italic());
          paintFont.setStrikeOut (charStrikeOut ? true : myfont.strikeOut());
          p2->setPen (paintColor);
          p2->setBackgroundColor (fillColor);
          p2->setFont (paintFont);
          p2->drawText (charWidth * startat, cellHeight() - descent - 1, text);
          if (hasblink)
          //only write these characters if they don't blink
            if (!charBlinks)
            {
              p2b->setPen (paintColor);
              p2b->setBackgroundColor (fillColor);
              p2b->setFont (paintFont);
              p2b->drawText (charWidth * startat, cellHeight() - descent - 1, text);
            }
          curcolor = fcolors[i];
          curbcolor = bcolors[i];
          charUnderlined = curCharUnderlined;
          charItalics = curCharItalics;
          charBlinks = curCharBlinks;
          charStrikeOut = curCharStrikeOut;
          startat = i;
          text = cell[row][i];
        }
        else
          text += cell[row][i];
      }
      paintColor = mycolor[curcolor];
      fillColor = mycolor[curbcolor];
      //font attributes
      paintFont.setUnderline (charUnderlined ? true : myfont.underline());
      paintFont.setItalic (charItalics ? true : myfont.italic());
      paintFont.setStrikeOut (charStrikeOut ? true : myfont.strikeOut());
      p2->setPen (paintColor);
      p2->setBackgroundColor (fillColor);
      p2->setFont (paintFont);
      p2->drawText (charWidth * startat, cellHeight() - descent - 1, text);
      if (hasblink)
      //only write these characters if they don't blink
        if (!charBlinks)
        {
          p2b->setPen (paintColor);
          p2b->setBackgroundColor (fillColor);
          p2b->setFont (paintFont);
          p2b->drawText (charWidth * startat, cellHeight() - descent - 1, text);
        }

      delete fcolors;
      delete bcolors;

      p2->end ();
      delete p2;
      if (hasblink)
      {
        p2b->end();
        delete p2b;
      }

      //Good. Now add this pixmap to the cache (if allowed).
      if (!forceNoCache)
      {
        int idx = -1;
        for (int i = 0; i < MAXCACHE; i++)
          if (cacheBlinkOn[i] == 0)
          {
            idx = i;
            break;
          }
        if (idx == -1)   //cache is FULL - create space!
        {
          deleteCacheEntry (0);
          for (int i = 0; i < MAXCACHE - 1; i++)
          {
            cacheBlinkOn[i] = cacheBlinkOn[i+1];
            cacheBlinkOff[i] = cacheBlinkOff[i+1];
            cacheEntry[i] = cacheEntry[i+1];
          }
          //we'll add a new entry at the free position
          idx = MAXCACHE - 1;
        }  
        cacheEntry[idx] = row;
        cacheBlinkOn[idx] = pix;
        if (hasblink)
          cacheBlinkOff[idx] = pix2;
        else
          cacheBlinkOff[idx] = pix;
      }
    }
    
    if (blinking && hasblink)
      p->drawPixmap (QPoint (0, 0), blinkPhase ? (*pix) : (*pix2),
            QRect (0, 0, cw, ch));
    else
      p->drawPixmap (QPoint (0, 0), *pix, QRect (0, 0, cw, ch));

    //if pix is not in the cache, then we have to delete it!
    //the same for pix2
    if (forceNoCache)
    {
      delete pix;
      if (hasblink)
        delete pix2;
    }
  }
  //else we do nothing...
  else
    return;
}

void cConsole::fixupOutput ()
{
  //cache is no longer valid
  emptyCache ();

  //compute new number of columns
  int newcols = viewport()->width() / charWidth;
  int spaceforrows = viewport()->height() / cellHeight ();
  QGridView::setCellWidth (newcols * charWidth);

  //columns are handled first...
  if (newcols != charCount)
  {
    for (int i = 0; i < usedrows; i++)
    {
      QChar *oldcell = cell[i];
      unsigned char *oldtcolor = tcolor[i];
      unsigned char *oldbcolor = bcolor[i];
      cell[i] = new QChar[newcols];
      tcolor[i] = new unsigned char[newcols];
      bcolor[i] = new unsigned char[newcols];
      int cp = min (newcols, charCount);
      if (oldcell == NULL)    //should never happen, but...
        cp = 0;
      for (int j = 0; j < cp; j++)
      {
        cell[i][j] = oldcell[j];
        tcolor[i][j] = oldtcolor[j];
        bcolor[i][j] = oldbcolor[j];
      }
      for (int j = cp; j < newcols; j++)
      {
        cell[i][j] = ' ';
        if(sess != NULL)
        {
          tcolor[i][j] = sess->textproc()->defaultTextColor();
          bcolor[i][j] = sess->textproc()->defaultBkColor();
        }
      }
      delete[] oldcell;
      delete[] oldtcolor;
      delete[] oldbcolor;
    }
    charCount = newcols;
  }

  //rows don't need much manipulation...
  setNumRows (usedrows);

  //set auxiliary console geometry, if needed
  if (!aux)
  {
    //geometry is relative to parent widget - that's the console object
    aconsole->setGeometry (0, height() - height() * aconsize / 100,
    width() - verticalScrollBar()->width(), height() * aconsize / 100);
          //height()*(100-aconsize)/100 can NOT be used in param 2,
          //as it may cause one-pixel shift, because integer division
          //is not accurate (A*3/4 + A/4 is not always A)
    //update scrollbar's properties
  }

  //inform others about the changes; telnet with NAWS should be connected here...
  if (!aux)
  {
    currows = spaceforrows;
    curcols = newcols;
    emit dimensionsChanged (newcols, spaceforrows);
  }

  //update the widget
  viewport()->update ();
  updateContents ();
}

void cConsole::resizeEvent (QResizeEvent *e)
{
  //let the parent process it first
  QGridView::resizeEvent (e);

  //fixup output window to reflect changes
  fixupOutput ();
}

void cConsole::updateRowRange (int r1, int r2)
{
  int rr1 = r1;
  int rr2 = r2;
  if (rr1 > rr2)
    swap (rr1, rr2);
  for (int i = rr1; i <= rr2; i++)
    updateCell (i, 0);
}

void cConsole::mousePressEvent (QMouseEvent *e)
{
  if (e->button() == LeftButton)
  {
    //clear old selection first
    if (selected)
    {
      selected = false;
      updateRowRange (selrow1, selrow2);
    }

    if (numRows() == 0)
      return;

    int inrow = rowAt (contentsY() + e->y());
    int incol = e->x() / charWidth;
    if (incol >= charCount)
      incol = charCount;
    if (incol < 0)
      incol = 0;
    if (inrow < 0)
      inrow = 0;
    if (inrow >= numRows())
      inrow = numRows() - 1;
    //do not crash if widget is still empty
    if ((inrow < 0) || (incol < 0))
      return;
    previousr = selrow2 = selrow1 = inrow;
    previousc = selrowpos2 = selrowpos1 = incol;
    canselect = true;
    selected = false;
  }
  else
    if (e->button() == MidButton)
    {
      if (aux)
        //auxiliary console won't process this, it will leave it for main
        //console (it's a parent of aconsole)
        e->ignore();
      else
      {
        //middle button scrolls to bottom, leaving split-screen mode
        verticalScrollBar()->setValue (verticalScrollBar()->maxValue());
        e->accept();
      }
    }
}

void cConsole::mouseReleaseEvent (QMouseEvent *e)
{
  if (e->button() == LeftButton)
    canselect = false;
}

void cConsole::mouseMoveEvent (QMouseEvent *e)
//event only occurs while some mouse button is pressed
{
  if (canselect)
  {
    int inrow = rowAt (contentsY() + e->y());
    int incol = e->x() / charWidth;
    if (incol >= charCount)
      incol = charCount;
    if (incol < 0)
      incol = 0;
    if (inrow < 0)
      inrow = 0;
    if (inrow >= numRows())
      inrow = numRows() - 1;
    //do not crash if widget is still empty
    if ((inrow < 0) || (incol < 0))
      return;
    previousr = selrow2;
    previousc = selrowpos2;
    selrow2 = inrow;
    selrowpos2 = incol;
    if ((inrow != previousr) || (incol != previousc))
      //moved mouse to another cell while pressing button
    {
      selected = true;

      //update all selected cells:
      updateRowRange (previousr, selrow2);

      //put selected text to mouse selection...
      addSelectionToClipboard (false);
    }
  }
}

void cConsole::mouseDoubleClickEvent (QMouseEvent *e)
{
  if (e->button() == LeftButton)
  {
    //clear old selection first - there should be none, as mousePress handler
    //has already been called, but I want to be sure...
    if (selected)
    {
      selected = false;
      updateRowRange (selrow1, selrow2);
      (KApplication::kApplication())->processEvents ();
    }

    int inrow = rowAt (contentsY() + e->y());
    int incol = e->x() / charWidth;
    if (incol >= charCount)
      incol = charCount;
    if (incol < 0)
      incol = 0;
    if (inrow < 0)
      inrow = 0;
    if (inrow >= numRows())
      inrow = numRows() - 1;
    //do not crash if widget is still empty
    if ((inrow < 0) || (incol < 0))
      return;  
    previousr = selrow2 = selrow1 = inrow;
    canselect = false;
    //find the whole word, if we've clicked in one... More than one row
    //can never be selected.
    QChar ch = cell[inrow][incol];
    if (ch.isLetterOrNumber() || (ch == '_'))
    {
      int index;
      //find the beginning
      index = incol;
      while (ch.isLetterOrNumber() || (ch == '_'))
      {
        if (index == 0)
          break;
        ch = cell[inrow][--index];
      }
      if (!(ch.isLetterOrNumber() || (ch == '_')))
        index++;
      selrowpos1 = (index >= 0) ? index : 0;  //set beginning of selection
      //and the end of the text
      index = incol;
      ch = cell[inrow][incol];
      while (ch.isLetterOrNumber() || (ch == '_'))
      {
        if (index == charCount - 1)
          break;
        ch = cell[inrow][++index];
      }
      if (!(ch.isLetterOrNumber() || (ch == '_')))
        index--;
      selrowpos2 = (index < charCount) ? index+1 : charCount;
          //set end of selection - it's one character further!

      selected = true;

      //update all selected cells:
      updateRowRange (selrow1, selrow2);

      //put selected text to mouse selection...
      addSelectionToClipboard (false);
    }
    else
    {
      previousc = selrowpos2 = selrowpos1 = incol;
      selected = false;
    }
  }
}

void cConsole::wheelEvent (QWheelEvent *e)
{
  /*
  Our parent processes whell event correctly, but there's one problem.
  If this is an auxiliary console, it shouldn't be scrollable, but the
  default wheelEvent implementation doesn't know it and scrolls.

  My solution is to process whelel event for the standard console by
  default handler and to ignore it by aconsole. Then it gets to the parent
  widget (main console), which processes it. It means that even if you
  use the wheel in aconsole region, main console gets scrolled, which is
  exactly what we want.
  */
  if (!aux)
    QGridView::wheelEvent (e);
  else
    e->ignore ();
}

void cConsole::addSelectionToClipboard (bool toclipboard)
{
  if (selected)
  {
    QString text = "";
    int r1 = selrow1;
    int r2 = selrow2;
    int c1 = selrowpos1;
    int c2 = selrowpos2;
    int colsperrow = charCount;
    int start = r1 * colsperrow + c1;
    int end = r2 * colsperrow + c2;
    if (start > end)
    {
      swap (r1, r2);
      swap (c1, c2);
    }
    for (int i = r1; i <= r2; i++)
    {
      QString line = "";
      for (int j = ((i == r1) ? c1 : 0);
             j < ((i == r2) ? c2 : charCount); j++)
        line += cell[i][j];
      //ok; now remove trailing spaces and add a \n, unless it's the last line
      int newlen = line.length();
      while ((newlen > 0) && (line[newlen - 1].latin1() == ' '))
        newlen--;
      line.truncate (newlen);
      if (i < r2)
        line += '\n';
      //and add this line to the resulting text
      text += line;
    }
    //now I have the whole text - put it to the clipboard!
    if (text.length() > 0)
    {
      QClipboard *clip = (KApplication::kApplication())->clipboard ();
      bool isselenabled = clip->selectionModeEnabled ();
      clip->setSelectionMode (!toclipboard);
      clip->setText (text);
      clip->setSelectionMode (isselenabled);
    }
  }
}

//the following functions are not implemented with {add/subtract}{Line/Page},
//because pageStep cannot be redefined in QGridView (no idea why). So I
//have to do this manually...

void cConsole::lineUp ()
{
  QScrollBar *s = verticalScrollBar ();
  s->setValue (s->value() - cellHeight ());
}

void cConsole::lineDown ()
{
  QScrollBar *s = verticalScrollBar ();
  s->setValue (s->value() + cellHeight ());
}

void cConsole::pageUp ()
{
  QScrollBar *s = verticalScrollBar ();
  s->setValue (s->value() - (height() - height() / 4) * 9/10);
}

void cConsole::pageDown ()
{
  QScrollBar *s = verticalScrollBar ();
  s->setValue (s->value() + (height() - height() / 4) * 9/10);
}

void cConsole::aconUp ()
{
  //85% is max size
  if (aconsize <= 80)
    aconsize += 5;
  else
    return;
  
  //set auxiliary console geometry, if needed
  if (!aux)
  {
    //geometry is relative to parent widget - that's the console object
    aconsole->setGeometry (0, height() - height() * aconsize / 100,
    width() - verticalScrollBar()->width(), height() * aconsize / 100);
          //height()*(100-aconsize)/100 can NOT be used in param 2,
          //as it may cause one-pixel shift, because integer division
          //is not accurate (A*3/4 + A/4 is not always A)
    //no need to move contents here, because it'll be done automatically
    // (we're always at the very bottom)
  }
}

void cConsole::aconDown ()
{
  int oldaconsize = aconsize;
  //5% is min size
  if (aconsize >= 10)
    aconsize -= 5;
  else
    return;
  
  //set auxiliary console geometry, if needed
  if (!aux)
  {
    int newy = height() - height() * aconsize / 100;
    int oldy = height() - height() * oldaconsize / 100;

    //geometry is relative to parent widget - that's the console object
    aconsole->setGeometry (0, newy,
    width() - verticalScrollBar()->width(), height() * aconsize / 100);
          //height()*(100-aconsize)/100 can NOT be used in param 2,
          //as it may cause one-pixel shift, because integer division
          //is not accurate (A*3/4 + A/4 is not always A)

    //we also need to move contents

    aconsole->scrollBy (0, newy - oldy);
  }
}

void cConsole::sliderChanged (int value)
{
  int maxval = verticalScrollBar()->maxValue ();
  if ((value == maxval) && (aconvisible))
  {
    aconsole->hide ();
    aconvisible = false;
    repaintContents (false);
  }
  if ((value != maxval) && (!aconvisible))
  {
    aconsole->show ();
    aconvisible = true;
  }
}

void cConsole::deleteCacheEntry (int which)
{
  if ((which < 0) || (which >= MAXCACHE))
    return;
  if (aux)   //no cache for aconsole
    return;
  if (cacheBlinkOn[which] != 0)
  {
    cacheEntry[which] = 0;
    //no blinking here
    if ((cacheBlinkOn[which] == cacheBlinkOff[which]) || (cacheBlinkOff[which] == 0))
    {
      delete cacheBlinkOn[which];
      cacheBlinkOff[which] = 0;
      cacheBlinkOn[which] = 0;
    }
    else
    {
      delete cacheBlinkOff[which];
      delete cacheBlinkOn[which];
      cacheBlinkOff[which] = 0;
      cacheBlinkOn[which] = 0;
    }
  }
}

void cConsole::emptyCache ()
{
  if (aux) //no cache for aconsole
    return;
  for (int i = 0; i < MAXCACHE; i++)
    deleteCacheEntry (i);
}

bool cConsole::hasBlink (int row)
{
  if ((row < 0) || (row >= usedrows))
    return false;
  if (tcolor[row] == 0)  //that row does not exist yet
    return false;
  bool hb = false;
  for (int i = 0; i < charCount; i++)
    if (tcolor[row][i] >= 64)
    {
      hb = true;
      break;
    }
  return hb;
}

void cConsole::forceEmitSize ()
{
  emit dimensionsChanged (curcols, currows);
}

void cConsole::blinkTimerTimeout ()
{
  //switch state
  blinkPhase = blinkPhase ? false : true;
  
  //get visible range
  int y1 = contentsY();
  int y2 = y1 + contentsHeight() - 1;
  int row1 = rowAt (y1);
  int row2 = rowAt (y2);

  //for each row
  for (int i = row1; i < row2; i++)
  {
    //look if it contains blinking characters
    if (hasBlink (i))
    //if it does, redraw it
      updateCell (i, 0);
  }
}

void cConsole::updatePrompt(const QString &text)
{
  if(promptlabel == true)
    {
	if(sess != NULL)
	    sess->prompt()->updatePrompt(text);
    }
}
