/***************************************************************************
                          cglobalsettings.h  -  global application settings
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : St Aug 14 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CGLOBALSETTINGS_H
#define CGLOBALSETTINGS_H

/*
Current status of cGlobalSettings class (Nov 29, 2002):
 - requested functionality implemented
 - new features may still be added
 - code has been tested and it works
 - TODO: the mechanism that is used to inform other instances that the
   settings have changed is not very good, as it could confuse other KDE
   apps that communicate with the same message ID; something better
   should be implemented!
*/

//this message ID will be used to communicate with other instances;
//it's a random number, it has no real meaning :)
#define KIPC_MESSAGE_ID 1158978

#include <config.h>

#include <qcolor.h>
#include <qfont.h>
#include <qobject.h>
#include <qstringlist.h>

class dlgAppSettings;
class KActionCollection;
/**
Class that stores and manages global application configuration.
  *@author Tomas Mecir
  */

class cGlobalSettings : public QObject  {
  Q_OBJECT
public:
  cGlobalSettings();
  ~cGlobalSettings();
  /** Applies settings. Called when settings change and on application startup. */
  void apply (bool informOthers = true);
  /** get object with accelerator keys - kmuddy class will fill it */
  KActionCollection *getACol ();
public slots:
  void showSettingsDialog ();
protected slots:
  void getSettingsFromDialog ();
  void KIPCMessage (int id, int param);
protected:
  /** load settings from kmuddyrc */
  void load ();
  /** save settings there */
  void save (bool informOthers = true);
  /** put settings to the dialog (called by showSettingsDialog()) */
  void putSettingsToDialog ();
  
  /** the dialog itself */
  dlgAppSettings *sdlg;

  /** next IPC message will be ignored - used to prevent reaction on our
    own message */
  bool ignoreNextMessage;
  
//settings data start here...

  //Window
  bool alwaystabbar, auxinput;
  bool globalnotify, localnotify, alwaysnotify;
  QString autoconnect;
  
  //Output area
  int bkcolorindex, fgcolorindex;
  int echoindex;
  int systemindex;
  int scriptoutputindex;
  bool allowblinking;
  bool cmdecho, messages;
  bool lpmudstyle;
  bool wordwrapping;
  int indentvalue;
  int hissize;
  
  //Fonts
  QFont cfont, ifont, mfont;
  
  //Input line
  bool keeptext;
  bool selectkepttext;
  bool arrowshistory;
  bool useac;       //ac = auto-completion
  int typeofac;
  bool tnpaste;
  bool trimspaces;
  int inputbkcolorindex, inputcolorindex;
      
  //Colors
  QColor color[16];

  //Characters
  QString separstr, walkstr, scriptstr, multistr;
  bool allowemptywalkstr;
   //BS = backslashes, AT = aliases/triggers
  bool expandBS, BSinAT;
  
#ifdef HAVE_SOUND
  //MUD Sound Prococol settings
  QStringList sounddirs;
  bool allowmsp, allowdload;
#endif
  
  //Shortcuts
  KActionCollection *acol;
};


#endif
