/***************************************************************************
                          cgrouplist.cpp  -  description
                             -------------------
    begin                : St dec 5 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cgrouplist.h"
#include "cgroup.h"

#include <kconfig.h>
#include <klocale.h>

cGroupList::cGroupList (QString filename, cSession *_sess, cSaveableList *assignedList) :
    cSaveableList (_sess)
{
  list = assignedList;
  
  //sess is assigned in cSaveableList's constructor
  prototype = new cGroup;
  objName = "Group";
  config = new KConfig (filename);
  load ();

  //there must be a BASE group
  cGroup *base = getGroup (0);
  //if there is none, create it!!!
  if (base == 0)
  {
    //we cannot use the addGroup function, because we need to add the BASE
    //group to the beginning of our list
    cGroup *group = new cGroup;
    group->setActive (true);
    group->setNumber (0);
    group->setText (i18n ("BASE"));
    addToBegin (group);
  }
   
  updateAssignedList ();
}

cGroupList::~cGroupList ()
{
  save ();
  config->sync ();
}

void cGroupList::updateAssignedList ()
{
  //update pointers to cGroup; maybe it could have been done more
  //effectively, but grouplist will only contain a few items, so that
  //it doesn't matter...
  for (list->reset(); *list; (*list)++)  //for every item in the list
    (**list)->updateGroupPointer (this);   //update its pointer
}

void cGroupList::activateGroup (int id, bool active)
{
  cGroup *group = getGroup (id);
  if (group != 0)
    group->setActive (active);
}

#define CHECKSTEP 100

int cGroupList::addGroup (const QString &name)
{
  //find first free ID first

  //To speed things up, we create an array with 100 flags and check for
  //100 free IDs in one run. This means that in most cases we'll find
  //a free ID in one cycle (few people use more than 100 groups)
  bool exists[CHECKSTEP];
     //to be more flexible, the number can easily be changed
  int base = 0;
  int freeID = -1;
  while (freeID == -1)  //while we have no free ID
  {
    for (int i = 0; i < CHECKSTEP; i++)
      exists[i] = false;
    for (reset(); *this; (*this)++)
    {
      cGroup *group = (cGroup *) **this;
      int gn = group->number();
      gn -= base;
      //if group number fits in the 100-member array for this run
      if ((gn >= 0) && (gn < CHECKSTEP))
        exists[gn] = true;      
    }
    //now the exists array contains information about free/occupied items
    //in ID range <base ; base+99>
    //look if there's some free ID...
    for (int i = 0; i < CHECKSTEP; i++)
      if (!exists[i]) //this ID is FREE!!!
      {
        freeID = base + i;
        break;
      }
    if (freeID == -1) //no free ID found
      base += CHECKSTEP;
  }
  
  //OK, now we have the ID, so we add a group with required parameters
  cGroup *group = new cGroup;
  group->setActive (true);
  group->setNumber (freeID);
  group->setText (name);
  addToEnd (group);

  //done! return that ID...
  return freeID;
}

bool cGroupList::removeGroup (int id)
{
  if (id == 0)   //BASE group can NOT be removed
    return false;
  cGroup *group = getGroup (id);
  //returned group has been set as current in our list
  if (group != 0)
  {
    removeCurrent();

    //if some items were in removed group, this will move them to the BASE group
    //no other changes will (should) be made
    updateAssignedList ();
    return true;
  }
  return false; //no such group
}

bool cGroupList::renameGroup (int id, const QString &newname)
{
  cGroup *group = getGroup (id);
  //returned group has been set as current in our list
  if (group != 0)
  {
    group->setText (newname);
    return true;
  }
  return false; //no such group
}

cGroup *cGroupList::getGroup (int id)
{
  for (reset(); *this; (*this)++)
  {
    cGroup *item = (cGroup *) cur;
    if (item->number() == id)  //this is what we're looking for!
      return item;
  }
  //if we're here, we've found nothing :(
  return 0;
}
