/***************************************************************************
                          cinputline.h  -  input line widgets
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : So Jun 29 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*
Current status of cInputLine class (Jul 5, 2002):
 - all required functionality has been implemented
 - new features may still be added (should not be needed...)
 - it works properly
*/

#ifndef CINPUTLINE_H
#define CINPUTLINE_H

#include <qwidget.h>
#include <klineedit.h>

#include "csessionbase.h"

#define CMDHISTORYSIZE 100

/**Input line - that's where you type your commands...
  *@author Tomas Mecir
  */

class cInputLine : public KLineEdit, public cSessionBase  {
   Q_OBJECT
public: 
  cInputLine (QWidget *parent=0, const char *name=0);
  ~cInputLine();
  /** initialize the input line - must be separated
  from the constructor, because it uses cSession, which is not
  available in constructor */
  void initialize ();
  
  void keepText (bool value);
  void selectKeptText (bool value);
  void setArrowsHistory (bool value);
  void setAC (bool useac);
/** set auto-completion type. Types are defined in KGlobalSettings:
  CompletionNone=1      //No completion is used.
  CompletionAuto=2      //Text is automatically filled in whenever possible.
  CompletionMan=3       //Same as automatic except shortest match is used for completion
  CompletionShell=4     //Complete text much in the same way as a typical *nix shell would.
  CompletionPopup=5     //Lists all possible matches in a popup list-box to choose from.
  CompletionPopupAuto=6 //Lists all possible matches in a popup list-box to choose from, and automatically * fill the result whenever possible.
    Types 4 and 6 don't work. I have no idea why...
*/
  void setACType (int typeofac);
  void setTelnetPaste (bool tnp);
protected slots:
  void handleEnter ();
  void menuItemHandler (int index);
protected:
  /** keypress event - handles history Up/Down browsing*/
  void keyPressEvent (QKeyEvent *e);
  /** we got focus - restore selection (needed because selection is lost
    when switching between connections) */
  void focusInEvent (QFocusEvent *e);
  /** we lost focus - store selection (needed because selection is lost
    when switching between connections)*/
  void focusOutEvent (QFocusEvent *e);
  /** reimplemented handler for pasting */
  virtual void paste ();
  /** reimplemented to work with my popup menu */
  virtual QPopupMenu *createPopupMenu ();
  QString getHistory (bool next);
  QString menuitem[CMDHISTORYSIZE];
  int menuitems;
  int lastid;
  bool keeptext;
  bool selectkepttext;
  bool arrowshistory;
  int curactype;
  int historypos;
  /** telnet-style paste */
  bool tnpaste;
  /** If we browse history with cursor keys, we certainly don't want to
  lose command that we're typing, do we? So we store it here. */
  QString origcommand;

  /** selection start/length, stored/restored in focusIN/focusOut events */
  int ss, sl;
};

#endif
