/***************************************************************************
                          cprofiles.h  -  list of profiles
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Ct Jul 25 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CPROFILES_H
#define CPROFILES_H

#include <qstringlist.h>

#include "cconnectioninfo.h"

/*
Current status of cProfiles class:
 - requested features implemented
 - new features may still be added
 - code seems to work properly
 - TODO: there should be some mechanism to determine if a connection with
   profile X is currently opened (some functions do not work properly then)
*/

/**
This class handles a list of profiles.

Instance only exists when really needed, so that we don't run into problems
when more than one instance of this application is running...

It includes a function that returns a cConnectionInfo object for a profile.

Function returning cConnPrefs is not provided, as it's really simple
to construct such object (by passing profile name to cConnPrefs' constructor).

*@author Tomas Mecir
*/

class cProfiles {
public: 
  cProfiles();
  ~cProfiles();
  void load ();
  void save ();   
  /** returns a list of profiles */
  const QStringList & getprofiles ();
  /** adds a new profile */
  bool addProfile (QString & name, cConnectionInfo *cci);
  /** !!! BEWARE !!! deleteProfile and modifyProfile will NOT work correctly
    if connection with that profile is currently opened !!! (because closing
    that connection will cause it to save its preferences, rewriting all
    modifications done here) */
  bool deleteProfile (QString & name);
  bool modifyProfile (QString & name, cConnectionInfo *cci);
  /** duplicates a profile */
  bool duplicateProfile (const QString &name, const QString &newname);
  /** returns whether a profile with this name already exists */
  bool exists (QString & name);
  /** this CREATES A NEW OBJECT OF TYPE cConnectionInfo and returns a pointer
    it. CALLER MUST DESTROY THIS OBJECT when not needed. */
  cConnectionInfo * getProfileConnInfo (QString & name);

  bool importProfile (const QString &name, const QString &profileFile);
  bool exportProfile (const QString &name, const QString &profileFile);
  const QString &lastError () { return error; };
protected:
  QStringList profiles;
  QString error;
};

#endif
