/***************************************************************************
                          crunningscript.cpp  -  running script
                             -------------------
    begin                : Ne dec 8 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "crunningscript.h"

#include "cscript.h"
#include "cunixsocket.h"

#include <kprocess.h>

#include <stdlib.h>

cRunningScript::cRunningScript (cScript *s)
{
  script = s;
  process = 0;
  suspended      = false;
  scriptDying    = false;
  dontSignal     = false;
  sendInProgress = false; // No data being sent yet
  stdinBuffer = "";       // Ensure buffer is empty
  unixsocket = 0;
  launchAfter = 0;
}

cRunningScript::~cRunningScript ()
{
  dontSignal = true;
  delete process;
  if (unixsocket) delete unixsocket;
  unixsocket  = 0;
  scriptDying = true; // Prevent further data being sent to this script
  
  if (sendInProgress && (!noflowcontrol))
  {
    //some text was about to be sent, but it won't be sent anymore...
    //we behave as if it was sent, to prevent deadlock in cRunningList
    // (it would endlessly wait for the signal, and no new text would
    // be sent to any other script)
    //
    // scriptDying flag set to prevent the signal causing
    // further data to be sent to this script.
    emit textSent ();
  }
  
}

void cRunningScript::launch (int fcState)
{
  // ALEX CHANGE HERE
  // Register the script as started, even if it's actually delayed by flow
  // control.  This prevents multiple instances of a single instance script.
  script->scriptIsStarting ();
  // END ALEX CHANGE
  
  if (noflowcontrol)
    doLaunch ();
  else
  {
    //we'll launch when flow control reaches the current point
    launchAfter = fcState;
    if (fcState == 0)
      doLaunch ();
  }
      
}

void cRunningScript::doLaunch ()
{
  line = QString::null;
  suspended = false;
  launchAfter = 0;
  
  //most stuff is already ready, only processExited remains...
  connect (process, SIGNAL (processExited (KProcess *)), this, SLOT (exited (KProcess *)));

  //start the process!
  // ALEX CHANGE HERE
/*  if (process->start (KProcess::NotifyOnExit, (KProcess::Communication) comm))
    script->scriptIsStarting ();
  else
    emit scriptFailed (this);*/
  if (!process->start (KProcess::NotifyOnExit, (KProcess::Communication) comm))
  {
    script->scriptIsTerminating ();
    emit scriptFailed (this);
  }
  // END ALEX CHANGE
  //we no longer need this environment variable
  unsetenv ("KMUDDY_SOCKET");
}

void cRunningScript::exited (KProcess *)
{
  if (unixsocket) delete unixsocket;
  unixsocket  = 0;
  scriptDying = true; // Prevent further data being sent to this script
  script->scriptIsTerminating ();
  //if some text was being sent
  if (sendInProgress)
  {
    //we behave as if the text was delivered successfully, to avoid
    //deadlock
    sendInProgress = false;
    if (!noflowcontrol)
      emit textSent ();
  }
  if (!dontSignal)
  {
    if (process->normalExit())
      emit scriptFinished (this, process->exitStatus());
    else
      emit scriptKilled (this);
  }
}

//improvements here by Alex Bache
void cRunningScript::sendCommandToScript (const QString &command, char type)
{
  actuallySent = false;
  
  if (process == 0)
    return;

  //are we waiting for flow control?
  if (launchAfter)
  {
    launchAfter--;
    if (launchAfter == 0)
      //we're in sync - start the script!
      doLaunch ();
    //do not send current line... if we have started the script
    //now, next line will be the first one that it will get
    return;
  }

  if (!inputenabled)
    return;   //no input => no sending!
  if ((type == USERCOMMAND) && (!sendusercommands))
      //type = user command, user commands are not sent
    return;
/*  no, we'll send prompt even if there's no adv.comm
  if ((type == PROMPT) && (!useadvcomm))
      //prompt only sent if using adv.communication
    return;
*/
  
  //don't send anything if the script is suspended
   //(otherwise the script would react on it immediately after it's resumed)
  if (suspended)
    return;

  // don't send anything if the script is in the process of shutting down
  if (scriptDying)
    return;
  
  //maybe we have a limiting text, test against it if needed
  if (script->getText().length() > 0)
    if (script->match (command) != onlyifmatch) //do nothing if text matches
        //but we request no match, or vice versa
      return;
  
  QString txt = command;
  //advanced communication?
  if (useadvcomm)
  {
    QString beg = QChar (type);
    beg += QChar (' ');
    txt = beg + txt;
  }
  
  //send it if nothing else in progress
  
  if (sendInProgress)
  {
    stdinBuffer.append(txt);
  } // endif must buffer for later
  else
  {
    stdinSending   = txt;
    sendInProgress = true;
    actuallySent   = true;
    process->writeStdin (stdinSending, stdinSending.length());
  } // endelse able to send immediately
  if (!noflowcontrol)
    emit textAccepted ();
}

//function written by Alex Bache
// stdinReady() called when script has processed the data on its STDIN and is ready for more
void cRunningScript::stdinReady()
{
  //only if we're sending something
  if (sendInProgress)
  {
    if (stdinBuffer.length() > 0)
    {
      //this shouldn't happen with the new input flow control, but just in case
      stdinSending = stdinBuffer;
      stdinBuffer  = "";
      process->writeStdin (stdinSending, stdinSending.length());
    } // endif more data to send to script's STDIN
    else
    {
      sendInProgress = false;
    } // endelse no more data to send at this time

    // inform cRunningList only when ALL buffered text has been sent
    if ((!sendInProgress) && (!noflowcontrol))
      emit textSent ();
  }
} // cRunningScript::stdinReady

void cRunningScript::processScriptStdOutput (KProcess *p, char *buffer, int buflen)
{
  processScriptOutput( p, buffer, buflen, sendstdout );
}

void cRunningScript::processScriptStdError (KProcess *p, char *buffer, int buflen)
{
  processScriptOutput( p, buffer, buflen, sendstderr );
}

void cRunningScript::processScriptOutput (KProcess *, char *buffer, int buflen, bool sendoutput)
{
  // cScript::prepareToLaunch connects to processScriptStdOutput
  // and processScriptStdError. They call this and tell if
  // the output needs to be sent to the MUD.
  for (int i = 0; i < buflen; i++)
  {
    if (buffer[i] != '\n')
      line += (QChar) buffer[i];
    else
    {
      //these will be connected by cRunningList
      //note: preprocessor scripts only emit sendText, not displayText
      if (sendoutput)
        emit sendText (prefix + line + suffix);
      else
        emit displayText (prefix + line + suffix);
      line = QString::null;
    }
  }
}

const QString cRunningScript::name ()
{
  return script->getName();
}


void cRunningScript::suspend ()
{
  if (process == 0)
    return;
  //nothing if the script waits for FC
  if (launchAfter) return;
  bool ret = process->kill (SIGSTOP);
  if (ret)  //success
  {
    suspended = true;
    stdinBuffer = stdinSending = QString::null;
    //if some text was being sent
    if (sendInProgress)
    {
      //we behave as if the text was delivered successfully, to avoid
      //deadlock
      sendInProgress = false;
      if (!noflowcontrol)
        emit textSent ();
    }
  }
}

void cRunningScript::resume ()
{
  if (process == 0)
    return;
  //nothing if the script waits for FC
  if (launchAfter) return;
  bool ret = process->kill (SIGCONT);
  if (ret)  //success
    suspended = false;
}

bool cRunningScript::isSuspended ()
{
  if (process == 0)
    return false;
  //nothing if the script waits for FC
  if (launchAfter) return true;
  return suspended;
}

bool cRunningScript::isRunning ()
{
  if (process == 0)
    return false;
  //nothing if the script waits for FC
  if (launchAfter) return false;
  return !suspended;
}

void cRunningScript::terminate ()
{
  //resume process if it's stopped
  if (process == 0)
    return;
    
  scriptDying = true; // Prevent further data being sent to this script
  process->kill (SIGCONT);
  //now try to terminate...
  if (isRunning ())
    process->kill ();
  //if some text was being sent
  if (sendInProgress)
  {
    //we behave as if the text was delivered successfully, to avoid
    //deadlock
    sendInProgress = false;
    if (!noflowcontrol)
      emit textSent ();
  }
}

void cRunningScript::kill ()
{
  //resume process if it's stopped
  if (process == 0)
    return;

  scriptDying = true; // Prevent further data being sent to this script
  process->kill (SIGCONT);
  //now go and KILL!!!
  if (isRunning ())
    process->kill (SIGKILL);
  //if some text was being sent
  if (sendInProgress)
  {
    //we behave as if the text was delivered successfully, to avoid
    //deadlock
    sendInProgress = false;
    if (!noflowcontrol)
      emit textSent ();
  }
}

void cRunningScript::establishSocket (cSession *sess)
{
  //we create a new socket
  unixsocket = new cUnixSocket (sess, this);
  
  //then we get its name
  QString sname = unixsocket->getName ();
  
  //and store it in environment for the script to pick up
  setenv ("KMUDDY_SOCKET", sname.latin1(), 1);
}
