/***************************************************************************
                          cscriptlist.cpp  -  list of scripts
                             -------------------
    begin                : Pi dec 13 2002
    copyright            : (C) 2002-2003 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cscriptlist.h"

#include "cconnection.h"
#include "cconnprefs.h"
#include "cconsole.h"
#include "crunninglist.h"
#include "crunningscript.h"
#include "csession.h"
#include "cscript.h"

#include <kconfig.h>
#include <klocale.h>

cScriptList::cScriptList (QString filename, cSession *_sess)
      : cSaveableList (_sess)
{
  //sess is assigned in cSaveableList's constructor
  prototype = new cScript (sess);
  objName = "Script";
  config = new KConfig (filename);
  load ();
}

cScriptList::~cScriptList ()
{
  save ();
  config->sync ();
}

bool cScriptList::nameExists (const QString &name)
{
  for (reset (); *this; (*this)++)
    if (((cScript *) **this)->getName() == name)
      //this is the right script
      return true;
  return false;
}

bool cScriptList::runCurrent (const QString &paramlist)
{
  return runScript ((cScript *) cur, paramlist);
}

bool cScriptList::runScript (QString name, const QString &paramlist)
{
  for (reset (); *this; (*this)++)
    if (((cScript *) **this)->getName() == name)
      //is this the right script?
    {
      return runScript ((cScript *) **this, paramlist);
    }
  sess->console()->systemMessage(i18n ("Such script does not exist!"));
  return false;
}

bool cScriptList::runScript (cScript *script, const QString &paramlist)
{
  if (script == 0) return false;
  cRunningScript *rs = script->prepareToLaunch(paramlist);
  if (rs != 0)
  {
    cRunningList *rl = sess->connection()->getConnPrefs()->runningList();
    //we know that it's safe to run a preprocessor script now
    if (script->isPreprocessor())
      rl->addPreprocessorScript (rs);
    else
      rl->addScript (rs);
    rs->launch(rl->fcState());
    return true;
  }
  return false;
}
