/***************************************************************************
                  csendcmd.cpp  -  class for sending commands to the MUD
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : t Jun 18 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define CSENDCMD_CPP

#include "csendcmd.h"

#include "csession.h"
#include "cpluginmanager.h"
#include "ctextprocessor.h"

cSendCmd::cSendCmd ()
{
  reset ();
}

cSendCmd::~cSendCmd()
{
  clearCommandQueue ();
}

void cSendCmd::reset ()
{
  sentcmds = 0;
  clearCommandQueue ();
}

void cSendCmd::addCommand (const QString &command)
{
  //plug-ins may want to modify the command :)
  bool dontSend = false;
  QString cmd = command;
  pluginmanager->passCommand (sess, cmd, dontSend);
  if (dontSend) return;
  queue.append (cmd);
}

void cSendCmd::clearCommandQueue ()
{
  queue.clear ();
}

void cSendCmd::oneCommandSender ()
{
  if (queue.empty())
    return;
  QString s = queue.first ();
  QString s2 = s;
  s += QChar (13);  //append a CRLF...
  s += QChar (10);
  emit commandReady (s, true, true);
  queue.pop_front ();

  sentcmds++;
  //do some necessary actions...
  sess->textproc()->sendingCommand();

    //remove() is NOT suitable, as it removes ALL commands that are the
    //same as this one (implicit sharing has its disadvantages...)

    //IMPORTANT: this line HAS TO BE the last one in this method!!!!!!
    // reason is that console, attached to this signal, may call
    // KMuddy::removeSession, thus this instance will no longer exist
    // when we get control again
  emit commandSent (s2);      //signals command WITHOUT the appended CRLF...
}
