/***************************************************************************
                  csendcmd.h  -  class for sending commands to the MUD
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : t Jun 18 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CSENDCMD_H
#define CSENDCMD_H

#include <qobject.h>
#include <qstringlist.h>

#include "csessionbase.h"

/*
Current status of cSendCmd class (Jul 4, 2002):
 - all required functionality has been implemented
 - new features may still be added (not probable)
 - code works
*/

/**
class cSendCmd is used to send commands to the MUD. It includes
a command queue, commands are sent one-by-one in an interval specified by
cTelnet class.
  *@author Tomas Mecir
  */

class cSendCmd : public QObject, public cSessionBase {
   Q_OBJECT
public: 
  /** constructor */
  cSendCmd ();
  /** destructor, nothing special */
  ~cSendCmd ();
  int sentCommands () { return sentcmds; };
signals:
  /** a new command has been sent - displaying widget and logging class
  are connected here */
  void commandSent (const QString &command);
  /** new command is about to be sent */
  void commandReady (const QString &data, bool addTimingRequest = false, bool use8Bit = false);
public slots:
  /** cancels sending of all commands that weren't already sent*/
  void clearCommandQueue ();
  /** adds a new command to the queue */
  void addCommand (const QString &command);
  /** attached to timing signal from cTelnet - sends the oldest available
  command and deletes it from the queue (FIFO queue) */
  void oneCommandSender ();
  /** reset counters and such */
  void reset ();
protected:
  /** FIFO queue of commands that are to be sent */
  QStringList queue;
  int sentcmds;
};

#endif
