/***************************************************************************
                          csession.cpp  -  basis of multi-world support
                             -------------------
    begin                : Pi nov 15 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "csession.h"


#include "ctelnet.h"
#include "csendcmd.h"
#include "cansiparser.h"
#include "cconsole.h"
#include "cinputline.h"
#include "ccmdprocessor.h"
#include "cstatus.h"
#include "cconnection.h"
#include "ctextprocessor.h"
#include "ctranscript.h"
#include "cprompt.h"

#include <kstatusbar.h>
#include <kapp.h>

cSession::cSession (QWidget *parent, const char *name) :
    QVBox (parent, name)
{
  _flashing = false;
  
  //create objects
  _telnet = new cTelnet;
  _telnet->setSession (this);
  _sendcmd = new cSendCmd;
  _sendcmd->setSession (this);
  _ansiparser = new cANSIParser;
  _ansiparser->setSession (this);
  _cmdproc = new cCmdProcessor;
  _cmdproc->setSession (this);
  _textproc = new cTextProcessor;
  _textproc->setSession (this);
  _connection = new cConnection;
  _connection->setSession (this);
  
  _console = new cConsole (false, this);
  _console->setSession (this);
  _console->auxConsole()->setSession (this);
  _auxinputline = new cInputLine (this);
  _auxinputline->setSession (this);
  _auxinputline->initialize ();
  _auxinputline->hide ();
  
  QHBox *promptinput = new QHBox (this);
  _prompt = new cPrompt(promptinput);
  
  _inputline = new cInputLine (promptinput);
  _inputline->setSession (this);
  _inputline->initialize ();
  
  KStatusBar *statusbar = new KStatusBar (this);
  _status = new cStatus (statusbar);
  _status->setSession (this);

  _transcript = new cTranscript;
  _transcript->setSession (this);

  //--- establish connections between objects ---

  //telnet->preprocessor->ansi parser->textproc->console - text displaying
  connect (_telnet, SIGNAL (dataReceived (const QString &)), _textproc, SLOT (preprocess (const QString &)));
  connect (_textproc, SIGNAL (preprocessed (const QString &)), _ansiparser, SLOT (parseText (const QString &)));
  connect (_ansiparser, SIGNAL (newColor (int)), _textproc, SLOT (changeColor (int)));
  connect (_ansiparser, SIGNAL (plainText (const QString &)), _textproc, SLOT (newText (const QString &)));
  connect (_textproc,
    SIGNAL (displayLine (const QString &, const QString &, const QString &)),
    _console,
    SLOT (addLine (const QString &, const QString &, const QString &)));
  connect (_textproc,
    SIGNAL (displayPrompt (const QString &, const QString &, const QString &)),
    _console,
    SLOT (addText (const QString &, const QString &, const QString &)));
  connect (_telnet, SIGNAL (receivedGA ()), _textproc, SLOT (receivedGA ()));
  connect (_telnet, SIGNAL (moreTextHere ()), _textproc, SLOT (moreTextHere ()));
  
  //dimensions...
  connect (_console, SIGNAL (dimensionsChanged (int, int)), _telnet, SLOT (windowSizeChanged (int, int)));

  //command sending
  connect (_telnet, SIGNAL (cmdRequest ()), _sendcmd, SLOT (oneCommandSender ()));
  connect (_sendcmd, SIGNAL (commandReady (const QString &, bool, bool)), _telnet, SLOT (sendData (const QString &, bool, bool)));
  connect (_sendcmd, SIGNAL (commandSent (const QString &)), _console, SLOT (addCommand (const QString &)));

  //command processing
  connect (_inputline, SIGNAL (returnPressed (const QString &)), _cmdproc, SLOT (processInput (const QString &)));
  connect (_auxinputline, SIGNAL (returnPressed (const QString &)), _cmdproc, SLOT (processInput (const QString &)));
  connect (_cmdproc, SIGNAL (newCommand (const QString &)), _sendcmd, SLOT (addCommand(const QString &)));
  
  //session transcript
  connect (_telnet, SIGNAL (disconnected ()), _transcript, SLOT (stopTranscript ()));
  connect (_telnet, SIGNAL (disconnected ()), _transcript, SLOT (stopAdvTranscript ()));
  connect (_textproc,
    SIGNAL (displayLine (const QString &, const QString &, const QString &)),
    _transcript,
    SLOT (addLineToTranscript (const QString &, const QString &, const QString &)));
  connect (_textproc,
    SIGNAL (displayLine (const QString &, const QString &, const QString &)),
    _transcript,
    SLOT (addLineToAdvTranscript (const QString &))); 
  // if line went to output window and is gagged from main window, store the line in transcript anyway
    connect (_textproc,
    SIGNAL (transcriptLine (const QString &, const QString &, const QString &)),
    _transcript,
    SLOT (addLineToTranscript (const QString &, const QString &, const QString &)));
  connect (_textproc,
    SIGNAL (transcriptLine (const QString &, const QString &, const QString &)),
    _transcript,
    SLOT (addLineToAdvTranscript (const QString &))); 
  //prompt is added as a separate line; this is to prevent problems when
  //command displaying is off
  connect (_textproc,
    SIGNAL (displayPrompt (const QString &, const QString &, const QString &)),
    _transcript,
    SLOT (addLineToTranscript (const QString &, const QString &, const QString &)));
  connect (_textproc,
    SIGNAL (displayPrompt (const QString &, const QString &, const QString &)),
    _transcript,
    SLOT (addLineToAdvTranscript (const QString &)));
  connect (_console, SIGNAL (commandDisplayed (const QString &)), _transcript, SLOT (addCommandToTranscript (const QString &)));

  //status-bar
  connect (_console, SIGNAL (dimensionsChanged (int, int)), _status, SLOT (dimensionsChanged (int, int)));
  connect (_telnet, SIGNAL (connected()), _status, SLOT (connected ()));
  connect (_telnet, SIGNAL (disconnected()), _status, SLOT (disconnected ()));
  connect (_telnet, SIGNAL (connected()), _status, SLOT (timerStart ()));
  connect (_telnet, SIGNAL (connected()), _status, SLOT (timerReset ()));
  connect (_telnet, SIGNAL (disconnected()), _status, SLOT (timerStop ()));
  connect (_textproc, SIGNAL (receivedPrompt (const QString &)), _status, SLOT (partialLine (const QString &)));
  connect (_textproc, SIGNAL (showPartialLine (const QString &)), _status, SLOT (partialLine (const QString &)));
  connect (_textproc, SIGNAL (receivedPrompt(const QString &)), _console, SLOT (updatePrompt (const QString &)));

  //clear console, counters and internal buffers on connect
  connect (_telnet, SIGNAL (connected ()), _console, SLOT (clear ()));
  connect (_telnet, SIGNAL (connected ()), _ansiparser, SLOT (flush ()));
  connect (_telnet, SIGNAL (connected ()), _textproc, SLOT (flush ()));
  connect (_telnet, SIGNAL (connected ()), _sendcmd, SLOT (reset ()));

  KApplication::kApplication()->processEvents ();

}

cSession::~cSession ()
{
  //transcript must be deleted before the console!
  delete _transcript;
  _transcript = NULL;

  delete _status;  //this also destroys the statusbar
  _status = NULL;
  delete _auxinputline;
  _auxinputline = NULL;
  delete _inputline;
  _inputline = NULL;
  delete _console;
  _console = NULL;
  delete _prompt;
  _prompt = NULL;

  delete _connection;
  _connection = NULL;
  delete _textproc;
  _textproc = NULL;
  delete _cmdproc;
  _cmdproc = NULL;
  delete _ansiparser;
  _ansiparser = NULL;
  delete _sendcmd;
  _sendcmd = NULL;
  delete _telnet;
  _telnet = NULL;
  
}

void cSession::setAuxInput (bool val)
{
  val ? _auxinputline->show () : _auxinputline->hide ();
}
