/***************************************************************************
                          csoundplayer.cpp  -  sound player
                             -------------------
    begin                : So apr 19 2003
    copyright            : (C) 2003 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "csoundplayer.h"

#ifdef HAVE_SOUND

#include <qtimer.h>

#ifdef HAVE_ARTS
#include <arts/kartsdispatcher.h>
#include <arts/kartsserver.h>
#include <arts/kplayobjectfactory.h>
#include <arts/kplayobject.h>
#endif

cSoundPlayer::cSoundPlayer (bool isWAVE)
{
  isWave = isWAVE;
  nosound = false;

#ifdef HAVE_ARTS
  //no idea what's this dispatcher good for, but documentation says that it
  //should be done like this...
  dispatcher = new KArtsDispatcher;
  server = new KArtsServer;
  factory = new KDE::PlayObjectFactory (server->server());
  playObject = 0;
#endif
#ifdef HAVE_SDL
  Mix_AllocateChannels (1);
  wave = 0;
  music = 0;
#endif
  
  timer = new QTimer;
  connect (timer, SIGNAL (timeout()), this, SLOT (timeout ()));
  startedPlaying = false;
}

cSoundPlayer::~cSoundPlayer()
{
  timer->stop ();
  delete timer;

  stop ();
#ifdef HAVE_ARTS
  delete factory;
  delete server;
  delete dispatcher;
#endif  //HAVE_ARTS
}

bool cSoundPlayer::isPlaying ()
{
  if (nosound) return false;

#ifdef HAVE_ARTS
  if (playObject && (playObject->state() == Arts::posPlaying))
    return true;
#endif
#ifdef HAVE_SDL
  if (isWave)
    return wave && Mix_Playing (0);
  else
    return music && Mix_PlayingMusic ();
#endif
  return false;
}

void cSoundPlayer::setPriority (int val)
{
  newPriority = val;
}

void cSoundPlayer::setRepeatsCount (int val)
{
  newRepeatCount = val;
}

void cSoundPlayer::setFileName (const QString &name)
{
  newFName = name;
}

void cSoundPlayer::setVolume (int val)
{
  newVolume = val;
}

void cSoundPlayer::play ()
{
  if (nosound) return;

  //this stop() MUST be called even if we're not playing, because the
  //old playobject (if any) may still be in memory
  stop ();    //will also free old data

  //apply new parameters
  fName = newFName;
  repeatCount = newRepeatCount;
  priority = newPriority;
  volume = newVolume;
  
  //set up timer
  timer->stop ();
  timer->start (1000);
  startedPlaying = false;
  
#ifdef HAVE_ARTS
  //create new KPlayObject
  playObject = factory->createPlayObject (fName, true);
  connect (playObject, SIGNAL (playObjectCreated ()), this, SLOT (playObjectCreated ()));
#endif
#ifdef HAVE_SDL
  if (isWave)
  {
    wave = Mix_LoadWAV (fName);
    if (!wave)
      return;
  }
  else
  {
    music = Mix_LoadMUS (fName);
    if (!music)
      return;
  }
#endif

  //volume
#ifdef HAVE_ARTS
  //not yet implemented for aRts
#endif
#ifdef HAVE_SDL
  int vol = (int) (volume * MIX_MAX_VOLUME / (float) 100);
  if (isWave)
    Mix_Volume (0, vol);
  else
    Mix_VolumeMusic (vol);
#endif
  
#ifdef HAVE_ARTS
  //PLAY !!!
  playObject->play ();
#endif
#ifdef HAVE_SDL
  startedPlaying = true;
  if (isWave)
  {
    if (Mix_PlayChannel (0, wave, 0) < 0)
    {
      stop ();
      startedPlaying = false;
      return;
    }
  }
  else
  {
    if (Mix_PlayMusic (music, 1) < 0)
    {
      stop ();
      startedPlaying = false;
      return;
    }
  }
#endif
}

void cSoundPlayer::stop ()
{
  if (nosound) return;

#ifdef HAVE_ARTS
  if (!playObject)
    return;
  playObject->halt ();
  delete playObject;
  playObject = 0;
#endif
#ifdef HAVE_SDL
  startedPlaying = false;
  if (isWave)
  {
    if (isPlaying ())
      Mix_HaltChannel (0);
    if (wave)
      Mix_FreeChunk (wave);
    wave = 0;
  }
  else
  {
    if (isPlaying ())
      Mix_HaltMusic ();
    if (music)
      Mix_FreeMusic (music);
    music = 0;
  }
#endif
}

void cSoundPlayer::forceUpdateParams ()
{
  repeatCount = newRepeatCount;
  priority = newPriority;
  volume = newVolume;
}

void cSoundPlayer::disableSound ()
{
  nosound = true;
}

#endif  //HAVE_SOUND

//slots must always have some implementation - MOC depends on that

void cSoundPlayer::playObjectCreated ()
//looks like playObjectCreated signal is never emitted; no idea why :(
{
//this is only used with aRts... SDL does not need it.
#ifdef HAVE_SOUND
  startedPlaying = true;
#endif
}

void cSoundPlayer::timeout ()
{
#ifdef HAVE_SOUND
#ifdef HAVE_ARTS
  if (!playObject)
    return;

  //we need to set this variable here, because createdPlayObject is not
  //called when it should be; no idea why
  if (isPlaying ())
    startedPlaying = true;
  
  if (startedPlaying && (!isPlaying())) //finished playing!
  {
    startedPlaying = false;
    if (repeatCount != -1)  //-1 means infinite playing
      repeatCount--;  //decrease repeat count
    if (repeatCount != 0)
      playObject->play ();
    else
      stop ();
  }  
#endif
#ifdef HAVE_SDL
  if (startedPlaying && (!isPlaying())) //finished playing!
  {
    startedPlaying = false;
    if (repeatCount != -1)  //-1 means infinite playing
      repeatCount--;  //decrease repeat count
    if (repeatCount != 0)
    {
      startedPlaying = true;
      if (isWave)
      {
        if (Mix_PlayChannel (0, wave, 0) < 0)
          stop ();
      }
      else
        if (Mix_PlayMusic (music, 1) < 0)
          stop ();
    }
    else
      stop ();
  }
#endif
#endif //HAVE_SOUND
}


