/***************************************************************************
                          csoundplayer.h  -  sound player
                             -------------------
    begin                : So apr 19 2003
    copyright            : (C) 2003 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CSOUNDPLAYER_H
#define CSOUNDPLAYER_H

#include <config.h>

#include <qobject.h>

#ifdef HAVE_SOUND

class QTimer;

#ifdef HAVE_ARTS

namespace KDE {
  class PlayObjectFactory;
  class PlayObject;
};

class KArtsDispatcher;
class KArtsServer;

#endif  //HAVE_ARTS

#ifdef HAVE_SDL
#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>
#endif  //HAVE_SDL

#endif //HAVE_SOUND

/**
Plays sound via aRts (using KPlayObject) or SDL_mixer.
  *@author Tomas Mecir
  */

class cSoundPlayer : public QObject  {
   Q_OBJECT
#ifdef HAVE_SOUND   //header must exist - MOC depends on that
public: 
  cSoundPlayer (bool isWAVE = true);
  ~cSoundPlayer ();
  /** are we currently playing? */
  bool isPlaying ();
  /** priority of currently played sound */
  int curPriority () { return priority; };
  /** set new priority */
  void setPriority (int val);
  /** remaining # of repeats, INCLUDING current one */
  int remainingRepeats () { return repeatCount; };
  /** set repeats count; will NOT affect playing sound */
  void setRepeatsCount (int val);
  /** played file name */
  const QString &fileName () { return fName; };
  /** set new file name */
  void setFileName (const QString &name);
  /** current volume */
  int curVolume () { return volume; };
  /** set new volume */
  void setVolume (int val);
  /** play sound, stopping previous one, if any */
  void play ();
  /** stop playing */
  void stop ();
  /** will force updating of some parameters for currently played sound;
  this includes repeat count, priority and volume */
  void forceUpdateParams ();
  void disableSound ();
#endif  //HAVE_SOUND
//slots must be defined - MOC depends on that
protected slots:
	void playObjectCreated ();
  void timeout ();
#ifdef HAVE_SOUND
protected:
  QTimer *timer;

  bool isWave;
  bool nosound;
  
#ifdef HAVE_ARTS
  KArtsDispatcher *dispatcher;
  KArtsServer *server;
  KDE::PlayObjectFactory *factory;
  KDE::PlayObject *playObject;
#endif
#ifdef HAVE_SDL
  Mix_Chunk *wave;
  Mix_Music *music;
#endif

  bool startedPlaying;
  
  QString fName;
  int repeatCount, priority, volume;
  QString newFName;
  int newRepeatCount, newPriority, newVolume;
#endif  //HAVE_SOUND
};

#endif
