/***************************************************************************
                          cstatus.cpp -  manages status bar
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Ne Jul 7 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cstatus.h"

#include <qdatetime.h>
#include <klocale.h>

#include "cconsole.h"
#include "csession.h"
#include "cconnection.h"
#include "cconnprefs.h"

#define ID_DIMENSION 1
#define ID_TIMER 2
#define ID_CURTIME 3
#define ID_CONNECTED 4
#define ID_PARTIAL 10

cStatus::cStatus(KStatusBar *statusbar)
{
  sb = statusbar;

  sb->insertItem (" " + i18n ("Off-line") + " ", ID_CONNECTED, 0, true);
  sb->insertItem (" ??x?? ", ID_DIMENSION, 0, true);
  sb->insertItem (" 0:00:00 ", ID_TIMER, 0, true);
  sb->insertItem (" 00:00 ", ID_CURTIME, 0, true);
  sb->insertItem ("", ID_PARTIAL);
  timerShown = true;

  //dimension will be signaled soon - don't bother with it now...

  timing = false;
  timer = new QTimer;
  connect (timer, SIGNAL (timeout ()), this, SLOT (timerTick ()));
  timer->start (1000);

  timerTick ();
}

cStatus::~cStatus()
{
  timer->stop ();
  delete timer;

  delete sb;
}

void cStatus::showTimer ()
{
  if (!timerShown)
  {
    timerReset ();
    sb->changeItem (" 0:00:00 ", ID_TIMER);
  }
  timerShown = true;
}

void cStatus::hideTimer ()
{
  if (timerShown)
  {
    timerStop ();
    sb->changeItem ("", ID_TIMER);  //item is invisible when its text is empty
  }
  timerShown = false;
}

void cStatus::showMessage (const QString & message)
{
  sb->message (message, 2000);
}

void cStatus::dimensionsChanged (int x, int y)
{
  char s1[10];
  QString s2;
  sprintf (s1, " %dx%d ", x, y);
  s2 = s1;
  sb->changeItem ("xxxxx", ID_DIMENSION);
  sb->changeItem (s2, ID_DIMENSION);
  sb->repaint ();
}

void cStatus::timerStart ()
{
  timing = true;
}

void cStatus::timerStop ()
{
  timing = false;
}

void cStatus::timerReset ()
{
  conntime = 0;
  if (timerShown)
    sb->changeItem (" 0:00:00 ", ID_TIMER);
}

void cStatus::connected ()
{
  sb->changeItem (" " + i18n ("Connected") + " ", ID_CONNECTED);
  sb->changeItem ("", ID_PARTIAL);
  showMessage (i18n ("Connected."));
}

void cStatus::disconnected ()
{
  sb->changeItem (" " + i18n ("Off-line") + " ", ID_CONNECTED);
  showMessage (i18n ("Disconnected."));
  sb->changeItem ("", ID_PARTIAL);
  sess->console()->systemMessage (i18n ("Connection has been closed."));
}

void cStatus::partialLine (const QString &line)
{
  cConnPrefs *cp = sess->connection()->getConnPrefs ();
  if (cp && cp->statusPrompt())
    sb->changeItem (line, ID_PARTIAL);
}

void cStatus::clearPartialLine ()
{
    sb->changeItem ("", ID_PARTIAL);
}

const QString cStatus::connTimeString ()
{
  char s1[10];
  int h = conntime / 3600;
  int s = conntime % 3600;
  int m = s / 60;
  s = s % 60;
  sprintf (s1, " %d:%02d:%02d ", h, m, s);
  QString s2 = s1;
  return s2;
}

void cStatus::timerTick ()
{
  char s1[10];
  QString s2;
  int h, m;
  if (timing)
  {
    conntime++;

    s2 = connTimeString ();
    sb->changeItem (s2, ID_TIMER);
  }

  if (timerShown)
  {
    QTime t = QTime::currentTime ();
    h = t.hour ();
    m = t.minute ();
    sprintf (s1, " %d:%02d ", h, m);
    s2 = s1;
    sb->changeItem (s2, ID_CURTIME);
  }
}
