/***************************************************************************
                          ctextprocessor.h  -  text processor
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Po oct 14 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CTEXTPROCESSOR_H
#define CTEXTPROCESSOR_H

/*
Current status of cTextProcessor class (Oct 14, 2002):
 - requested functionality implemented
 - new features may still be added
 - code has been tested and it seems to work
*/

#include <qobject.h>

#include "csessionbase.h"

class QTimer;

/**
Text processor - handles trigger execution and such...
  *@author Tomas Mecir
  */

class cTextProcessor : public QObject, public cSessionBase {
   Q_OBJECT
public: 
  cTextProcessor ();
  ~cTextProcessor ();
  int linesReceived () { return lines; };

  /** get default textcolor */
  int defaultTextColor ();
  /** change default textcolor */
  void setDefaultTextColor (int index);
  /** get default background */
  int defaultBkColor ();
  /** change default background */
  void setDefaultBkColor (int index);

  /** recolorize the line - used by color triggers
  params would fit in one integer array, but this is more flexible :P */
  void recolorize (int *newfg, int *newbg);
  /** this line should be gagged - called by gagging triggers */
  void gagLine () { gag = true; };
  /** this line should be gagged in primary window */
  void gagLineInPrimary () { primarygag = true; };
  void setHavePrompt () { haveprompt = true; };
  /** handle some issued when sending a command */
  void sendingCommand ();

  void setOutputWindow(QString winname) { wname = winname; };
public slots:
  /** preprocess this text */
  void preprocess (const QString &text);
  /** process output from preprocessor script */
  void preprocessorOutput (const QString &text);

  /** process this text */
  void newText (const QString &text);
  void changeColor (int color);
  void flush ();
  void receivedGA ();
  /** used to display partial lines in the status bar */
  void moreTextHere ();
signals:
  /** text preprocessed, can continue processing... */
  void preprocessed (const QString &text);
  void displayLine (const QString &text, const QString &tcolor,
      const QString &bcolor);
  void transcriptLine (const QString &text, const QString &tcolor,
      const QString &bcolor);
  void displayPrompt (const QString &text, const QString &tcolor,
      const QString &bcolor);
  void textLine (const QString &text);
  void receivedPrompt (const QString &prompt);
  void showPartialLine (const QString &line);
protected slots:
  /** used by prompt-detection timer */
  void timeout ();
protected:
  void newLine ();

  /** a timer used to auto-detect prompts */
  QTimer *pdtimer;
  int elapsedticks;

  QString line, tcolor, bcolor;
  int lines;

  /** some ANSI flags */
  bool boldactive, blinkactive;
  bool underline, italics, strikeout;
  bool negactive;
  bool invisible;

  /** some colors */
  int curcolor, curbkcolor;
  int defcolor, defbkcolor;

  /** gagging */
  bool gag, primarygag;
  /** prompt detection */
  bool haveprompt, gotprompt;

  bool owin;
  QString wname;
};

#endif
