/***************************************************************************
                          ctimer.cpp  -  description
                             -------------------
    begin                : ?t mj 8 2003
    copyright            : (C) 2003 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ctimer.h"

#include "csession.h"
#include "ccmdprocessor.h"

#include <kconfig.h>

cTimer::cTimer (cSession *s)
{
  sess = s;

  _interval = 60;
  _singleshot = false;
  _active = false;
  _command = QString::null;

  _nextlaunch = -1;
  _shoulddelete = false;
}

cTimer::~cTimer ()
{

}

cSaveableField *cTimer::newInstance ()
{
  return new cTimer (sess);
}

void cTimer::load (KConfig *config, const QString &group)
{
  config->setGroup (group);

  setInterval (config->readNumEntry ("Interval", 60));
  setSingleShot (config->readBoolEntry ("Single shot", false));
  setActive (config->readBoolEntry ("Active", false));
  setCommand (config->readEntry ("Command", QString::null));
}

void cTimer::save (KConfig *config, const QString &group)
{
  config->setGroup (group);

  config->writeEntry ("Interval", _interval);
  config->writeEntry ("Single shot", _singleshot);
  config->writeEntry ("Active", _active);
  config->writeEntry ("Command", _command);
}

QString cTimer::performAction ()
{
  if (!_active)
    return QString::null;

  //process and send the command
  sess->cmdproc()->processInput (_command);

  return QString::null;
}

void cTimer::tick (int tickTime)
{
  //not active - reset next-launch time and return
  if (!_active)
  {
    _nextlaunch = -1;
    return;
  }
  if (_interval < 1)  //no interval set
    return;
  
  if (_nextlaunch == -1) //no next-launch time set
  {
    //update next-launch time and return
    _nextlaunch = tickTime + _interval;
    return;
  }

  //check if we should execute now
  if (_nextlaunch == tickTime)
  {
    performAction ();
    _nextlaunch += _interval;
    //if we're an single-shot timer, we should be deleted
    if (_singleshot)
      _shoulddelete = true;
  }
}

void cTimer::resetTickCount ()
{
  _nextlaunch = -1;
}

void cTimer::setInterval (int what)
{
  if (what >= 1)  //only positive intervals are allowed
    _interval = what;
}
