/***************************************************************************
                          ctriggerlist.cpp  -  trigger list
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Ne oct 13 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ctriggerlist.h"

#include "ccmdprocessor.h"
#include "cconnection.h"
#include "cconnprefs.h"
#include "csession.h"
#include "ctrigger.h"
#include "kmuddy.h"

extern KMuddy *kmuddy;

#include <kconfig.h>
#include <kmessagebox.h>
#include <klocale.h>

//used in loop detection
#define MAXTRIGGERS 50

cTriggerList::cTriggerList (QString filename, cSession *_sess)
  : cSaveableList (_sess)
{
  prototype = new cTrigger (_sess);
  objName = "Trigger";
  config = new KConfig (filename);
  load ();
  count = 0;
  time.start ();
}

cTriggerList::~cTriggerList ()
{
  save ();
  config->sync ();
}

void cTriggerList::processCommands (const QStringList &commands)
{
  if (commands.isEmpty())  //do nothing if there are no commands
    return;

  //check if loop detection is enabled
  //for quick connections, loop detection is always enabled
  cConnPrefs *cp = sess->connection()->getConnPrefs();
  bool detectLoops = ((cp == 0) ? true : cp->limitTriggers());

  if (detectLoops)
  {
    //if time diff between triggers was less than 1 second, remember it...
    if (time.elapsed () < 1000)
      count++;
    else
      count = 0;
    time.restart ();
  }
  else
    count = 0;  //no loop detection -> no trigger counting
  if (count == MAXTRIGGERS)
  {
    _enabled = false;
    recursionAlert = true;  //we're in recursion!!!
  }
  else
  {
    //okay, send the commands!
    QStringList::const_iterator it;
    for (it = commands.begin(); it != commands.end(); ++it)
      sess->cmdproc()->processTriggerInput(*it);
  }
}

void cTriggerList::processRecursion ()
{
  //triggers are now disabled; they will be reenabled when this function ends
  KMessageBox::error (kmuddy, i18n ("Trigger loop has been detected. "
      "Triggers will be temporarily disabled until you close this dialog box.\n\n"
      "Note that you can disable this check in Profile / MUD Preferences."), i18n ("Trigger loop detected"));
}
