/***************************************************************************
                          cvariablelist.h  -  list of variables
                             -------------------
    begin                : Po sep 8 2003
    copyright            : (C) 2003 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CVARIABLELIST_H
#define CVARIABLELIST_H

#include "csaveablelist.h"

#include <map>

class cVariable;

/**
This class represents the list of variables.
  *@author Tomas Mecir
  */

//protected inheritance, because we don't want others to mess things up
//by using inherited methods that don't support our enhancements
class cVariableList : protected cSaveableList {
   Q_OBJECT
public:
  cVariableList (QString filename, cSession *s);
  ~cVariableList ();
  virtual void processCommands (const QStringList &command) {};
  QString getValue (const QString &varname);
  int getIntValue (const QString &varname);
  bool exists (const QString &varname);
  void set (const QString &varname, const QString &value);
  void unset (const QString &varname);
  /** increases value by delta */
  void inc (const QString &varname, int delta);
  /** decreases value by delta; value can become negative */
  void dec (const QString &varname, int delta);
  /** resource support; provides one unit of a resource */
  void provideResource (const QString &varname);
  /** resource support; requests one resource, if available */
  bool requestResource (const QString &varname);
  QStringList getList ();
  /** expand all variables in the string and return the result */
  QString expandVariables (const QString &string, bool recursive = true);

  //needed to make some brain-dead compiler compile this...
  friend class cConnPrefs;
signals:
  void variableChanged (const QString &name, const QString &value);
protected:
  QString doExpandVariables (const QString &string, int recursionCounter);
  QString processVariable (const QString &varname);
  void initMapping ();
  /** variable mapping, used to speed up access to variables */
  std::map<QString, cVariable *> vars;
};

#endif
