//
// C++ Implementation: cvartrigger
//
// Description: One variable trigger.
//
//
// Author: Tomas Mecir <kmuddy@kmuddy.org>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "cvartrigger.h"

#include <kconfig.h>

cVarTrigger::cVarTrigger (cSession *s)
{
  sess = s;
}


cVarTrigger::~cVarTrigger()
{
}

cSaveableField *cVarTrigger::newInstance ()
{
  return new cVarTrigger (sess);
}

//always returns QString::null
QString cVarTrigger::performAction ()
{
  commandstoexec.clear();

  int commands = 0;
  bool haveNonPlain = false;
  QStringList::iterator it;
  for (it = newtext.begin(); it != newtext.end(); ++it)
  {
    QString nt = *it;
    expandPseudoVariables (nt);
    commandstoexec.push_back (nt);
    commands++;
    if (! (nt.isEmpty())) haveNonPlain = true;
  }

  //empty to-sent text
  if ((commands == 1) && (!haveNonPlain))
  {
    //send nothing
    commandstoexec.clear ();
    return QString::null;
  }

  return QString::null;
}

void cVarTrigger::load (KConfig *config, const QString &group)
{
  config->setGroup (group);
  setVarName (config->readEntry ("Variable name", ""));

  newtext.clear();
  int cmdcount = config->readNumEntry ("Command count", 0);
  for (int i = 1; i <= cmdcount; i++)
  {
    QString cmdline = config->readEntry ("Command line " +
          QString::number (i), "");
    newtext.push_back (cmdline);
  }
}

void cVarTrigger::save (KConfig *config, const QString &group)
{
  config->setGroup (group);
  config->writeEntry ("Variable name", var);

  int cmdcount = newtext.count();
  config->writeEntry ("Command count", cmdcount);
  QStringList::iterator it = newtext.begin();
  for (int i = 1; i <= cmdcount; i++)
  {
    config->writeEntry ("Command line " + QString::number (i), *it);
    it++;
  }

}

void cVarTrigger::setVarName (const QString &varname)
{
  QString v = varname;
  //remove all leading $-characters, if any
  while ((!v.isEmpty()) && (v[0] == '$'))
    v = v.mid (1);
  //set the variable name, if it's non-empty
  if (!v.isEmpty())
    var = v;
}


#include "cvartrigger.moc"
