/***************************************************************************
                          dlgaliastrigger.cpp  -  alias/trigger dialog
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : So sep 14 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../dialogs/dlgaliastrigger.h"
#include "../dialogs/dlgeditaliastrigger.h"
#include "../dialogs/dlgeditaction.h"
#include "../dialogs/dlgintnuminput.h"

#include "../caliaslist.h"
#include "../calias.h"
#include "../ctrigger.h"
#include "../ctriggerlist.h"
#include "../caction.h"
#include "../cactionlist.h"
#include "../csession.h"

#include <qlabel.h>
#include <qlayout.h>
#include <klocale.h>
#include <kpushbutton.h>
#include <kapp.h>
#include <kiconloader.h>
#include <kmessagebox.h>

dlgAliasTrigger::dlgAliasTrigger (cAliasList *alist, QWidget *parent, const char *name) :
    KDialogBase (parent, name, true, i18n ("Aliases"), Ok, Ok, true)
{
  //store alias list
  al = alist;
  tl = NULL;
  acl = NULL;
  list = al;

  createDialog ();
}

dlgAliasTrigger::dlgAliasTrigger (cTriggerList *tlist, QWidget *parent, const char *name) :
    KDialogBase (parent, name, true, i18n ("Triggers"), Ok, Ok, true)
{
  //store trigger list
  al = NULL;
  tl = tlist;
  acl = NULL;
  list = tl;

  createDialog ();
}

dlgAliasTrigger::dlgAliasTrigger (cActionList *aclist, QWidget *parent, const char *name) :
    KDialogBase (parent, name, true, i18n ("Dynamic actions"), Ok, Ok, true)
{
  //store trigger list
  al = NULL;
  tl = NULL;
  acl = aclist;
  list = acl;

  createDialog ();
}

void dlgAliasTrigger::createDialog ()
{
  //initial dialog size
  setInitialSize (QSize (500, 400));

  //create main dialog's widget
  QWidget *page = new QWidget (this);
  QGridLayout *layout = new QGridLayout (page, 3, 2);

  setMainWidget (page);

  //change button text
  setButtonOKText (i18n ("Done"));

  //create widgets
  QLabel *label = new QLabel ((al != NULL) ? i18n ("&List of aliases") :
        ((tl != NULL) ? i18n ("&List of triggers") : i18n ("&List of actions")), page);
  box = new KListBox (page);
  label->setBuddy (box);

  lblcurpos = new QLabel (i18n ("Current position: "), page);
  
  QFrame *buttons = new QFrame (page);
  QVBoxLayout *buttonslayout = new QVBoxLayout (buttons);

  KIconLoader *icons = (KApplication::kApplication())->iconLoader ();
  btadd = new KPushButton (i18n ("&Add..."), buttons);
  btmodify = new KPushButton (i18n ("&Modify..."), buttons);
  btdelete = new KPushButton (i18n ("De&lete"), buttons);
  bt10up = new KPushButton (icons->loadIcon ("up", KIcon::Small),
        i18n ("Move 10 u&p"), buttons);
  btup = new KPushButton (icons->loadIcon ("up", KIcon::Small),
        i18n ("Move &up"), buttons);
  btdown = new KPushButton (icons->loadIcon ("down", KIcon::Small),
        i18n ("Move &down"), buttons);
  bt10down = new KPushButton (icons->loadIcon ("down", KIcon::Small),
        i18n ("Move 10 do&wn"), buttons);
  btmoveto = new KPushButton (i18n ("Move &to..."), buttons);
  
  buttonslayout->setSpacing (5);
  buttonslayout->addWidget (btadd);
  buttonslayout->addWidget (btmodify);
  buttonslayout->addWidget (btdelete);
  buttonslayout->addStretch (1);
  buttonslayout->addWidget (bt10up);
  buttonslayout->addWidget (btup);
  buttonslayout->addWidget (btdown);
  buttonslayout->addWidget (bt10down);
  buttonslayout->addStretch (1);
  buttonslayout->addWidget (btmoveto);
  buttonslayout->addStretch (10);

  layout->setColStretch (0, 10);
  layout->setRowStretch (1, 10);
  layout->setSpacing (5);

  //place out widgets
  layout->addWidget (label, 0, 0);
  layout->addWidget (box, 1, 0);
  layout->addWidget (buttons, 1, 1);
  layout->addWidget (lblcurpos, 2, 0);

  //update the listbox
  updateMe ();

  //connect() the whole stuff!
  connect (btadd, SIGNAL (clicked ()), this, SLOT (add ()));
  connect (btmodify, SIGNAL (clicked ()), this, SLOT (modify ()));
  connect (btdelete, SIGNAL (clicked ()), this, SLOT (remove ()));
  connect (btup, SIGNAL (clicked ()), this, SLOT (up ()));
  connect (btdown, SIGNAL (clicked ()), this, SLOT (down ()));
  connect (bt10up, SIGNAL (clicked ()), this, SLOT (up10 ()));
  connect (bt10down, SIGNAL (clicked ()), this, SLOT (down10 ()));
  connect (btmoveto, SIGNAL (clicked ()), this, SLOT (moveto ()));
  connect (box, SIGNAL (doubleClicked (QListBoxItem *)), this, SLOT (modify ()));
  connect (box, SIGNAL (highlighted (int)), this, SLOT (changePositionInfo (int)));
}

dlgAliasTrigger::~dlgAliasTrigger()
{
  //NOTHING HERE
}

void dlgAliasTrigger::add ()
{
  //we show Edit alias/trigger dialog
  dlgEditAliasTrigger *edit = NULL;
  cSaveableField *item = 0;
  if (al != NULL)
  {
    item = new cAlias (al->session()->sendcmd());
    edit = new dlgEditAliasTrigger ((cAlias *) item, al->session(), this);
  }
  if (tl != NULL)
  {
    item = new cTrigger (tl->session());
    edit = new dlgEditAliasTrigger ((cTrigger *) item, tl->session(), this);
  }
  if (acl == NULL)
  {
    if ((edit != NULL) && (edit->showMe ()))
    {
      //item now contains new data, so just add it to the list!
      list->addToEnd (item);
      updateMe ();
    }
    else
      delete item;
    if (edit != NULL)
      delete edit;
  }
  else
  {
    //action list is special
    item = new cAction;
    dlgEditAction *edac = new dlgEditAction ((cAction *)item, this);
    if (edac->showMe ())
    {
      list->addToEnd (item);
      updateMe ();
    }
    acl->rebuildToolbar();
  }
  
  int it = list->count () - 1;
  box->setSelected (it, true);
  box->setCurrentItem (it);
}

void dlgAliasTrigger::modify ()
{
  int it = box->currentItem ();
  if (it < 0) return;
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough
  list->reset ();
  for (int i = 0; i < it; i++)
    (*list)++;
  cSaveableField *field = **list;
  if (acl == NULL)
  {
    dlgEditAliasTrigger *edit = 0;
    if (al != NULL)
      edit = new dlgEditAliasTrigger ((cAlias *) field, al->session(), this);
    if (tl != NULL)
      edit = new dlgEditAliasTrigger ((cTrigger *) field, tl->session(), this);
    if (edit->showMe ())
      //if he pressed OK, field already contains new data -> we only have to
      //update the listbox...
      updateMe ();
    delete edit;
  }
  else
  {
    dlgEditAction *edit = new dlgEditAction ((cAction *) field, this);
    if (edit->showMe ())
    {
      //if he pressed OK, field already contains new data -> we only have to
      //update the listbox and the toolbar...
      updateMe ();
      acl->rebuildToolbar ();
    }
    delete edit;
  }
  box->setSelected (it, true);
  box->setCurrentItem (it);
}

void dlgAliasTrigger::remove ()
{
  //get that item
  int it = box->currentItem ();
  if (it < 0) return;
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough

  //ask: really?
  QString q1, q2;
  if (al != NULL)
  {
    q1 = i18n("Do you really want to delete this alias?");
    q2 = i18n ("Delete alias");
  }
  if (tl != NULL)
  {
    q1 = i18n("Do you really want to delete this trigger?");
    q2 = i18n ("Delete trigger");
  }
  if (acl != NULL)
  {
    q1 = i18n("Do you really want to delete this action?");
    q2 = i18n ("Delete action");
  }  
  int n = KMessageBox::questionYesNoCancel (this, q1, q2);
  if (n == KMessageBox::Yes)
  {
    //really!
    //current item MUST be found AFTER  the message box invocation; otherwise
    //some trigger matching could modify that
    list->reset ();
    for (int i = 0; i < it; i++)
      (*list)++;
    list->removeCurrent ();
    updateMe ();
    if (acl != NULL)
      acl->rebuildToolbar ();    
  }
}

void dlgAliasTrigger::up ()
{
  int it = box->currentItem ();
  if (it < 0) return;
  if (it == 0) return;    //current item is the topmost one
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough
  list->reset ();
  for (int i = 0; i < it; i++)
    (*list)++;
  list->moveCurrentToFront ();
  updateMe ();
  box->setCurrentItem (it - 1);
  if (acl != NULL)
    acl->rebuildToolbar ();
}

void dlgAliasTrigger::down ()
{
  int it = box->currentItem ();
  if (it < 0) return;
  if (it == box->count () - 1) return;    //current item is the bottommost one
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough
  list->reset ();
  for (int i = 0; i < it; i++)
    (*list)++;
  list->moveCurrentToBack ();
  updateMe ();
  box->setCurrentItem (it + 1);
  if (acl != NULL)
    acl->rebuildToolbar ();
}

void dlgAliasTrigger::up10 ()
{
  for (int i = 0; i < 10; i++)
    up ();
}

void dlgAliasTrigger::down10 ()
{
  for (int i = 0; i < 10; i++)
    down ();
}

void dlgAliasTrigger::moveto ()
{
  if (box->currentItem() == -1)
    return;  //nothing selected -> nothing to do
  
  bool ok;
  int newPos = dlgIntNumInput::getNumber (i18n ("Enter new position index:"),
      0, box->count() - 1, box->currentItem (), &ok, this);

  //get item
  list->reset ();
  for (int i = 0; i < box->currentItem(); i++)
    (*list)++;
  cSaveableField *item = **list;

  //remove item from list (but don't delete it!)
  list->removeCurrent (true);

  //Good. Now find new position.
  if (newPos == 0)   //very first pos
    list->addToBegin (item);
  else
  {
    list->reset ();
    for (int i = 0; i < newPos - 1; i++)
      (*list)++;
    list->addAfterCurrent (item);
  }
  updateMe ();
  box->setCurrentItem (newPos);
  
  if (acl != NULL)
    acl->rebuildToolbar ();
}

void dlgAliasTrigger::changePositionInfo (int newpos)
{
  lblcurpos->setText (i18n ("Current position: ") + QString::number(newpos));
}

void dlgAliasTrigger::updateMe ()
{
  //remember old index
  int idx = box->currentItem ();

  box->clear ();
  for (list->reset(); (*list); (*list)++)
  {
    if (acl == NULL)
    {
      //fetch alias/trigger text and replacement text (1st line only)
      QStringList newtext = (**list)->getNewText ();
      QString line1;
      if (!newtext.isEmpty())
        line1 = newtext.first();
      if (newtext.count() > 1)
        //there are more commands to send
        line1 += "  [...]";
      QString name = (**list)->getText () + " -> " + line1;
      box->insertItem (name);
    }
    else
    {  //acl equals list
      box->insertItem (((cAction *) **acl)->getCaption ());
    }
  }

  //make one item active, trying to be intelligent...
  box->setCurrentItem ((idx == -1) ? 0 : idx);
  if ((box->currentItem () == -1) && (idx != -1))
    box->setCurrentItem (idx - 1);
  if (box->currentItem () == -1)
    box->setCurrentItem (0);
}
