/***************************************************************************
                          dlgappsettings.cpp  -  application settings
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : St aug 14 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../dialogs/dlgappsettings.h"
#include "../cglobalsettings.h"
#include "../cprofiles.h"
#include "cdirlist.h"

//a lot of include files...
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qframe.h>
#include <qhbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qvgroupbox.h>
#include <qwhatsthis.h>

#include <kapplication.h>
#include <kcolorbutton.h>
#include <kfontdialog.h>
#include <kiconloader.h>
#include <kkeydialog.h>
#include <klineedit.h>
#include <klocale.h>
#include <knuminput.h>
#include <kpushbutton.h>
#include <kstandarddirs.h>

extern cGlobalSettings *globalsettings;

//color list
QString cltext[16] = { i18n("Black"), i18n("Red"), i18n("Green"),
    i18n("Yellow"), i18n("Blue"), i18n("Magenta"), i18n("Cyan"), i18n("Gray"),
    i18n("Dark gray"), i18n("Bright red"), i18n("Bright green"),
    i18n("Bright yellow"), i18n("Bright blue"), i18n("Bright magenta"),
    i18n("Bright cyan"), i18n("White") };

dlgAppSettings::dlgAppSettings(QWidget *parent, const char *name ) :
    KDialogBase (IconList, i18n ("Application settings"), Ok|Cancel|Apply,
    Ok, parent, name, true)
{
  //initial dialog size
  setInitialSize (QSize (600, 400));

  //prepare dialog pages
    //KIcon::Toolbar is the closest match... I hope it's okay...
  KIconLoader *icons = (KApplication::kApplication())->iconLoader ();
  QFrame *frmwindow = addPage (i18n ("Window"), QString::null,
      icons->loadIcon ("winprops", KIcon::Toolbar, 32));
  QFrame *frmoutputarea = addPage (i18n ("Output area"), QString::null,
      icons->loadIcon ("text_left", KIcon::Toolbar, 32));
  QFrame *frmfont = addPage (i18n ("Font"), QString::null, 
      icons->loadIcon ("fonts", KIcon::Toolbar, 32));
  QFrame *frminput = addPage (i18n ("Input line"), QString::null, 
      icons->loadIcon ("editclear", KIcon::Toolbar, 32));
  QHBox *frmcolors = addHBoxPage (i18n ("Colors"), QString::null, 
      icons->loadIcon ("colorize", KIcon::Toolbar, 32));
  QFrame *frmchars = addPage (i18n ("Strings"), QString::null,
      icons->loadIcon ("text", KIcon::Toolbar, 32));
#ifdef HAVE_SOUND
  QFrame *frmsound = addPage (i18n ("Sound"), QString::null,
      icons->loadIcon ("multimedia", KIcon::Toolbar, 32));
#endif
  QFrame *frmshortcuts = addPage (i18n ("Shortcuts"), QString::null, 
      icons->loadIcon ("key_bindings", KIcon::Toolbar, 32));

  //page 1
  QGridLayout *winlayout = new QGridLayout (frmwindow, 3, 7);
   //always tabbar
  chkalwaystabbar = new QCheckBox (i18n ("S&how the tab bar even if not needed"), frmwindow);
  QWhatsThis::add (chkalwaystabbar, i18n ("If this is checked, the tab bar will "
      "be always shown. Otherwise, it will be hidden if it contains only one tab."));
   //auxiliary input
  chkauxinput = new QCheckBox (i18n ("Show au&xiliary input line"), frmwindow);
  QWhatsThis::add (chkauxinput, i18n ("You'll have two input lines instead of one."));
   //always notify
  chkalwaysnotify = new QCheckBox (i18n ("&Always notify"), frmwindow);
  QWhatsThis::add (chkalwaysnotify, i18n ("If this is checked, output notification "
      "will be used whenever some text comes from the server. Otherwise, it will "
      "only be used when a notification trigger is activated. Note that this "
      "option only takes effect if global or local notification is enabled."));
   //global notify
  chkglobalnotify = new QCheckBox (i18n ("&Global notification"), frmwindow);
  QWhatsThis::add (chkglobalnotify, i18n ("When some text comes from the server "
      "and KMuddy is not currently active, window caption and the taskbar will "
      "try to attract your attention (some characters will blink there)."));
   //local notify
  chklocalnotify = new QCheckBox (i18n ("&Local notification"), frmwindow);
  QWhatsThis::add (chklocalnotify, i18n ("When some text comes from the server "
      "and this session is not currently active, its tab will try to attract "
      "your attention (some characters will blink there)."));
  //auto-connect
  QLabel *lblac = new QLabel (i18n ("Auto-connect on startup"), frmwindow);
  autoconnect = new QComboBox (frmwindow);
  lblac->setBuddy (autoconnect);
  QWhatsThis::add (autoconnect, i18n ("With this option set, the selected "
      "connection will be automatically initiated immediately after starting "
      "KMuddy. Handy when you connect using one profile most of the time."));
  autoconnect->insertItem (i18n ("(disabled)"));
  cProfiles p;
  autoconnect->insertStringList (p.getprofiles());
  
   //add widgets to the layout
  winlayout->setSpacing (10);
  winlayout->addMultiCellWidget (chkalwaystabbar, 0, 0, 0, 1);
  winlayout->addMultiCellWidget (chkauxinput, 1, 1, 0, 1);
  winlayout->addMultiCellWidget (chkalwaysnotify, 2, 2, 0, 1);
  winlayout->addMultiCellWidget (chkglobalnotify, 3, 3, 0, 1);
  winlayout->addMultiCellWidget (chklocalnotify, 4, 4, 0, 1);
  winlayout->addWidget (lblac, 5, 0);
  winlayout->addWidget (autoconnect, 5, 1);
  winlayout->setRowStretch (6, 10);

  //page 2
  QVBoxLayout *outlayout = new QVBoxLayout (frmoutputarea);
   //colors
  QWidget *wcolors = new QWidget (frmoutputarea);
  QGridLayout *wincolorlayout = new QGridLayout (wcolors, 6, 3);
  QStringList colorlist;
  for (int i = 0; i < 16; i++)
    colorlist.append (cltext[i]);
  QLabel *wcolorlabel[5];
    //bg color
  wcolorlabel[0] = new QLabel (i18n ("Default &background color"), wcolors);
  combo[0] = new QComboBox (false, wcolors);
  combo[0]->insertStringList (colorlist);
  QWhatsThis::add (combo[0], i18n ("This color will be used as a standard background color."));
    //fg color
  wcolorlabel[1] = new QLabel (i18n ("Default &text color"), wcolors);
  combo[1] = new QComboBox (false, wcolors);
  combo[1]->insertStringList (colorlist);
  QWhatsThis::add (combo[1], i18n ("This color will be used to display standard text with no defined color."));
    //echo color
  wcolorlabel[2] = new QLabel (i18n ("Command &echo color"), wcolors);
  combo[2] = new QComboBox (false, wcolors);
  combo[2]->insertStringList (colorlist);
  QWhatsThis::add (combo[2], i18n ("This color will be used to display commands that you have entered."));
    //system color
  wcolorlabel[3] = new QLabel (i18n ("&System message color"), wcolors);
  combo[3] = new QComboBox (false, wcolors);
  combo[3]->insertStringList (colorlist);
  QWhatsThis::add (combo[3], i18n ("This color will be used for various system messages."));
    //script output color
  wcolorlabel[4] = new QLabel (i18n ("S&cript output color"), wcolors);
  combo[4] = new QComboBox (false, wcolors);
  combo[4]->insertStringList (colorlist);
  QWhatsThis::add (combo[4], i18n ("This color will be used to display script output that is not sent to the server."));
    //insert'em!
  wincolorlayout->setSpacing (10);
  for (int i = 0; i < 5; i++)
  {
    wcolorlabel[i]->setBuddy (combo[i]);
    wincolorlayout->addWidget (wcolorlabel[i], i, 0);
    wincolorlayout->addWidget (combo[i], i, 1);
  }
  wincolorlayout->setRowStretch (5, 10);
  wincolorlayout->setColStretch (2, 3);
   //cmd echo and system messages
  chkcmdecho = new QCheckBox (i18n ("Enable co&mmand echo"), frmoutputarea);
  QWhatsThis::add (chkcmdecho, i18n ("Enables displaying of commands in the "
      "console.<p><b>Please note:</b> This will also disable command echo "
      "in session transcript."));
  chkmessages = new QCheckBox (i18n ("Enable s&ystem messages"), frmoutputarea);
  QWhatsThis::add (chkmessages, i18n ("Enables displaying of system messages "
      "in the console."));

    //blinking
  chkblinking = new QCheckBox (i18n ("Enable b&linking"), frmoutputarea);
  QWhatsThis::add (chkblinking, i18n ("Enables support for blinking. "
      "This is disabled by default, because most players don't like it."));
  
   //word wrapping
  chkwrap = new QCheckBox (i18n ("Enable &word wrapping"), frmoutputarea);
  QWhatsThis::add (chkwrap, i18n ("If a line that is longer than current console "
      "width is to be displayed, it wraps to the next line. With this option "
      "enabled, whole words will move to the next line, instead of cutting "
      "words in the middle."));

   //indentation
  edindent = new KIntNumInput (frmoutputarea);
  edindent->setLabel (i18n ("&Indentation"),
      KNumInput::AlignLeft | KNumInput::AlignVCenter);
  edindent->setMinValue (0);
  edindent->setMaxValue (10);
  edindent->setSpecialValueText (i18n ("None"));
  //edindent shouldn't be too wide
  edindent->setMaximumSize (edindent->minimumSizeHint ());
  QWhatsThis::add (edindent, i18n ("When a line is too long, part of it will "
      "be moved to the next line. Indentation says how many spaces will be "
      "put to the beginning of the continued line."));

   //history size
  edhistory = new KIntNumInput (frmoutputarea);
  edhistory->setLabel (i18n ("&History buffer size"),
      KNumInput::AlignLeft | KNumInput::AlignVCenter);
  edhistory->setRange (100, 10000, 100, false);
  edhistory->setSuffix (" " + i18n ("lines"));
  //edhistory shouldn't be too wide too
  edhistory->setMaximumSize (edhistory->minimumSizeHint ());
  QWhatsThis::add (edhistory, i18n ("Size of output scroll-back buffer. "
      "Please note that this setting will only affect new connections, "
      "existing ones will keep their history setting."));
    
   //add widgets to main layout
  outlayout->setSpacing (10);
  outlayout->addWidget (wcolors);
  outlayout->addWidget (chkcmdecho);
  outlayout->addWidget (chkmessages);
  outlayout->addWidget (chkblinking);
  outlayout->addWidget (chkwrap);
  outlayout->addWidget (edindent);
  outlayout->addWidget (edhistory);
  outlayout->addStretch (1);
  
  //page 3
  QGridLayout *fontlayout = new QGridLayout (frmfont, 4, 3);
   //row 1
  QLabel *fl1 = new QLabel (i18n ("&Server output"), frmfont);
  fonted1 = new QLabel (frmfont);
  fonted1->setFrameStyle (QFrame::StyledPanel | QFrame::Sunken);
  KPushButton *bfont1 = new KPushButton (i18n ("Choose..."), frmfont);
  fl1->setBuddy (bfont1);
  connect (bfont1, SIGNAL (clicked()), this, SLOT (fontchooser1 ()));
  QString serveroutput = i18n ("Here you can select a font that will be used for main display.");
  QWhatsThis::add (fonted1, serveroutput);
  QWhatsThis::add (bfont1, serveroutput);
   //row 2  
  QLabel *fl2 = new QLabel (i18n ("&Input line"), frmfont);
  fonted2 = new QLabel (frmfont);
  fonted2->setFrameStyle (QFrame::StyledPanel | QFrame::Sunken);
  KPushButton *bfont2 = new KPushButton (i18n ("Choose..."), frmfont);
  fl2->setBuddy (bfont2);
  connect (bfont2, SIGNAL (clicked()), this, SLOT (fontchooser2 ()));
  QString inputline = i18n ("Here you can select a font that will be used for the input line.");
  QWhatsThis::add (fonted2, inputline);
  QWhatsThis::add (bfont2, inputline);
   //row 3
  QLabel *fl3 = new QLabel (i18n ("&Multi-line input"), frmfont);
  fonted3 = new QLabel (frmfont);
  fonted3->setFrameStyle (QFrame::StyledPanel | QFrame::Sunken);
  KPushButton *bfont3 = new KPushButton (i18n ("Choose..."), frmfont);
  fl3->setBuddy (bfont3);
  connect (bfont3, SIGNAL (clicked()), this, SLOT (fontchooser3 ()));
  QString minputline = i18n ("Here you can select a font that will be used for the multi-line input window.");
  QWhatsThis::add (fonted3, minputline);
  QWhatsThis::add (bfont3, minputline);
   //place widgets
  fontlayout->setSpacing (10);
  fontlayout->setColStretch (1, 10);
  fontlayout->addWidget (fl1, 0, 0);
  fontlayout->addWidget (fl2, 1, 0);
  fontlayout->addWidget (fl3, 2, 0);
  fontlayout->addWidget (fonted1, 0, 1);
  fontlayout->addWidget (fonted2, 1, 1);
  fontlayout->addWidget (fonted3, 2, 1);
  fontlayout->addWidget (bfont1, 0, 2);
  fontlayout->addWidget (bfont2, 1, 2);
  fontlayout->addWidget (bfont3, 2, 2);
  fontlayout->setRowStretch (3, 10);
  
  //page 4
  QVBoxLayout *inputlayout = new QVBoxLayout (frminput);
  chkkeeptext = new QCheckBox (i18n ("&Keep sent text"), frminput);
  QWhatsThis::add (chkkeeptext, i18n ("If you enable this option, commands "
        "will remain in the input line after you send them (by pressing "
        "ENTER); otherwise it will be cleared."));
  chkselectkepttext = new QCheckBox (i18n ("&Select kept text after it's sent"), frminput);
  QWhatsThis::add (chkselectkepttext, i18n ("If you enable this option, entered "
        "command will be selected. This allows you to immediately type another "
        "command without deleting the old one and still be able to reuse previous "
        "command easily."));
  connect (chkkeeptext, SIGNAL (toggled (bool)), chkselectkepttext, SLOT (setEnabled (bool)));
  chkarrowshistory = new QCheckBox (i18n ("&Cursor keys browse history"), frminput);
  QWhatsThis::add (chkarrowshistory, i18n ("If you enable this option, you "
        "will be able to choose previously entered commands with Up and Down keys."));
  chktelnetpaste = new QCheckBox (i18n ("Telnet-style &paste"), frminput);
  QWhatsThis::add (chktelnetpaste, i18n ("Enables telnet-style pasting, i.e. "
      "if multi-line text is pasted, all lines but the last are sent as "
      "commands immediately."));
  chktrimspaces = new QCheckBox (i18n ("T&rim spaces"), frminput);
  QWhatsThis::add (chktrimspaces, i18n ("Enables trimming of spaces from "
      "commands. Note that with this option disabled, you should avoid "
      "including spaces when typing multiple commands on one line, or you "
      "will encounter problems with special sequences not being detected "
      "properly."));
  chkautocompletion = new QCheckBox (i18n ("Enable &auto-completion"), frminput);
  QString autocompldesc = i18n ("Auto-completion is a feature that offers "
      "you possible completions for commands, as you type them. It can also "
      "finish the command for you.");
  QWhatsThis::add (chkautocompletion, autocompldesc);
   //combos widget
  QWidget *inputcombos = new QWidget (frminput);
  QGridLayout *inputcombolayout = new QGridLayout (inputcombos, 4, 3);
    //AC type
  QLabel *inputlabel1 = new QLabel (i18n ("T&ype of auto-completion"), inputcombos);
  comboactype = new QComboBox (false, inputcombos);
      //types 4 and 6 don't work :(((
  comboactype->insertItem (i18n ("Auto-fill"));                     //2
  comboactype->insertItem (i18n ("Shortest-match auto-fill"));      //3
  comboactype->insertItem (i18n ("Popup list-box"));                //5
  QWhatsThis::add (comboactype, autocompldesc);
  connect (chkautocompletion, SIGNAL (toggled (bool)), comboactype, SLOT (setEnabled (bool)));
    //bg color
  QLabel *inputlabel2 = new QLabel (i18n ("&Background color"), inputcombos);
  comboinputbkcolor = new QComboBox (false, inputcombos);
  comboinputbkcolor->insertStringList (colorlist);
  QWhatsThis::add (comboinputbkcolor, i18n ("Background color of the input line"));
    //fg color
  QLabel *inputlabel3 = new QLabel (i18n ("&Text color"), inputcombos);
  comboinputcolor = new QComboBox (false, inputcombos);
  comboinputcolor->insertStringList (colorlist);
  QWhatsThis::add (comboinputcolor, i18n ("Color of text that you type in the input line."));
    //set buddies...
  inputlabel1->setBuddy (comboactype);
  inputlabel2->setBuddy (comboinputbkcolor);
  inputlabel3->setBuddy (comboinputcolor);
    //place widgets...
  inputcombolayout->addWidget (inputlabel1, 0, 0);
  inputcombolayout->addWidget (inputlabel2, 1, 0);
  inputcombolayout->addWidget (inputlabel3, 2, 0);
  inputcombolayout->addWidget (comboactype, 0, 1);
  inputcombolayout->addWidget (comboinputbkcolor, 1, 1);
  inputcombolayout->addWidget (comboinputcolor, 2, 1);
  inputcombolayout->setSpacing (10);
  inputcombolayout->setRowStretch (3, 10);
  inputcombolayout->setColStretch (2, 3);
   //place widgets on the page
  inputlayout->addWidget (chkkeeptext);
  inputlayout->addWidget (chkselectkepttext);
  inputlayout->addWidget (chkarrowshistory);
  inputlayout->addWidget (chktelnetpaste);
  inputlayout->addWidget (chktrimspaces);
  inputlayout->addWidget (chkautocompletion);
  inputlayout->addWidget (inputcombos);
  inputlayout->setSpacing (10);
  inputlayout->addStretch (10);
  
  //page 5
  QVGroupBox *colorsbox1 = new QVGroupBox (i18n ("Dark colors"), frmcolors);
  QVGroupBox *colorsbox2 = new QVGroupBox (i18n ("Bright colors"), frmcolors);
  for (int i = 0; i < 8; i++)
  {
    QHBox *colorboxn = new QHBox (colorsbox1);
    QLabel *l = new QLabel (cltext[i], colorboxn);
    cbutton[i] = new KColorButton (colorboxn);
    colorboxn->setStretchFactor (l, 2);
    colorboxn->setStretchFactor (cbutton[i], 1);
  }
  for (int i = 8; i < 16; i++)
  {
    QHBox *colorboxn = new QHBox (colorsbox2);
    QLabel *l = new QLabel (cltext[i], colorboxn);
    cbutton[i] = new KColorButton (colorboxn);
    colorboxn->setStretchFactor (l, 1);
    colorboxn->setStretchFactor (cbutton[i], 0);
  }
  
  //page 6
  QGridLayout *charlayout = new QGridLayout (frmchars, 8, 3);
   //row 1
  QLabel *chl1 = new QLabel (i18n ("&Command separator string"), frmchars);
  edcmdseparator = new KLineEdit (frmchars);
  chl1->setBuddy (edcmdseparator);
  QWhatsThis::add (edcmdseparator, i18n ("You can enter multiple commands at "
        "once, they will get executed together. Here you can enter string "
        "that can be used to separate individual commands."));
   //row 2  
  QLabel *chl2 = new QLabel (i18n ("&Speed-walk string"), frmchars);
  edspeedwalk = new KLineEdit (frmchars);
  chl2->setBuddy (edspeedwalk);
  QWhatsThis::add (edspeedwalk, i18n ("If you start a command with this string, "
        "the command will be interpreted as speed-walking. Every character "
        "is treated as one command: n, s, w, e, j(=ne), k(=se), l(=sw), m(=nw), u, d."
        "<p>Commands that will be sent for each direction can be set up in "
        "<i>MUD preferences</i> (<b>profile-based</b> connections only)."));
   //row 3
  chkemptywalk = new QCheckBox (i18n ("S&peed-walk even if empty"), frmchars);
  QWhatsThis::add (chkemptywalk, i18n ("Enabling this will cause KMuddy to "
        "expand speed-walk strings even if no speed-walk string has been "
        "given. This allows faster speed-walking, but keep in mind that "
        "EVERY command that consists only of valid speed-walk characters "
        "(news, for example) will be treated as speed-walking. To avoid "
        "this, you'll have to type at least one character in upper-case."));
   //row 4
  QLabel *chl3 = new QLabel (i18n ("Macro-ca&ll string"), frmchars);
  edscript = new KLineEdit (frmchars);
  chl3->setBuddy (edscript);
  QWhatsThis::add (edscript, i18n ("If you start a command with this string, "
        "the command will be interpreted as a macro-call.<br>"
        "Syntax is &lt;this string&gt; &lt;command&gt; "
        "[parameters]  (/exec calls external scripts)"));
   //row 5
  QLabel *chl4 = new QLabel (i18n ("&Repeater string"), frmchars);
  edmulticmd = new KLineEdit (frmchars);
  chl4->setBuddy (edmulticmd);
  QWhatsThis::add (edmulticmd, i18n ("Allows you to repeat some command. For "
        "example, typing <i>#10 say hello</i> sends that command 10 times. "
        "<br>Note that this also works with aliases and macro execution.<br>"
        "Repeater and number may be separated by a space (spaces)."));
   //row 6
  chkexpandBS = new QCheckBox (i18n ("&Expand backslashes"), frmchars);
  QWhatsThis::add (chkexpandBS, i18n ("This will enable backslash expansion, "
      "that is: \\n is end-of-line, \\t is TAB, \\\\ is \\, \\<anythisg else> "
      "means<anything else>."));
   //row 7
  chkBSonlyinAT = new QCheckBox (i18n ("&Only expand backslashes in aliases/triggers"), frmchars);
  QWhatsThis::add (chkBSonlyinAT, i18n ("This only takes effect is backslash "
      "expansion is enabled. It will limit such expansion only to aliases "
      "and triggers."));
  
   //place widgets
  charlayout->setSpacing (10);
  charlayout->setColStretch (2, 10);
  charlayout->addWidget (chl1, 0, 0);
  charlayout->addWidget (chl2, 1, 0);
  charlayout->addWidget (chl3, 3, 0);
  charlayout->addWidget (chl4, 4, 0);
  charlayout->addWidget (edcmdseparator, 0, 1);
  charlayout->addWidget (edspeedwalk, 1, 1);
  charlayout->addWidget (edscript, 3, 1);
  charlayout->addWidget (edmulticmd, 4, 1);
  charlayout->addMultiCellWidget (chkemptywalk, 2, 2, 0, 1);
  charlayout->addMultiCellWidget (chkexpandBS, 5, 5, 0, 1);
  charlayout->addMultiCellWidget (chkBSonlyinAT, 6, 6, 0, 1);

#ifdef HAVE_SOUND
  //page 7
  QVBoxLayout *soundlayout = new QVBoxLayout (frmsound);
  chkallowmsp = new QCheckBox (i18n ("&Enable sounds globally"), frmsound);
  QWhatsThis::add (chkallowmsp, i18n ("<qt>Enable playing sounds for all connections.\n"
      "Note that you can override this setting in MUD Preferences dialog.\n"
      "Also note that this will only work if your MUD supports <b>MSP</b> "
      "(MUD Sound Protocol).</qt>"));
  chkallowdownloads = new QCheckBox (i18n ("Allow &downloading sounds"), frmsound);
  QString localdir = locateLocal ("appdata", "sounds/");
  QWhatsThis::add (chkallowdownloads, i18n ("MSP version 0.3 supports "
      "downloading of sound files directly from the web or FTP. You can "
      "enable this functionality here. Note that every sound will only be "
      "downloaded once. The downloaded files are stored in your homedir in "
      "directory\n") + QString (localdir));
  dirlist = new cDirList (frmsound);

  soundlayout->setSpacing (5);
  soundlayout->addWidget (chkallowmsp);
  soundlayout->addWidget (chkallowdownloads);
  soundlayout->addWidget (dirlist);
#endif  //HAVE_SOUND
  
  //page 8
   //layout ensures that the widget fills entire available space...
  QHBoxLayout *keylayout = new QHBoxLayout (frmshortcuts);
  keys = new KKeyChooser (globalsettings->getACol (), frmshortcuts, false);
  keys->setPreferFourModifierKeys (true);
  keylayout->addWidget (keys);
}

dlgAppSettings::~dlgAppSettings()
{
  delete keys;
}

int dlgAppSettings::getcolorindex (int which)
{
  if ((which >= 0) && (which <= 4))
    return combo[which]->currentItem ();
  return 0;
}

void dlgAppSettings::setcolorindex (int which, int value)
{
  if ((which >= 0) && (which <= 4) && (value >= 0) && (value <= 15))
    combo[which]->setCurrentItem (value);
}

bool dlgAppSettings::cmdEcho ()
{
  return chkcmdecho->isChecked ();
}

bool dlgAppSettings::messages ()
{
  return chkmessages->isChecked ();
}

void dlgAppSettings::setCmdEcho (bool value)
{
  chkcmdecho->setChecked (value);
}

void dlgAppSettings::setMessages (bool value)
{
  chkmessages->setChecked (value);
}

void dlgAppSettings::setWordWrapping (bool value)
{
  chkwrap->setChecked (value);
}

bool dlgAppSettings::wordWrapping ()
{
  return chkwrap->isChecked ();
}

void dlgAppSettings::setIndentation (int value)
{
  edindent->setValue (value);
}

int dlgAppSettings::indentation ()
{
  return edindent->value ();
}

void dlgAppSettings::setHistory (int value)
{
  edhistory->setValue (value);
}

int dlgAppSettings::history ()
{
  return edhistory->value ();
}

bool dlgAppSettings::enableBlinking ()
{
  return chkblinking->isChecked ();
}

void dlgAppSettings::setEnableBlinking (bool value)
{
  chkblinking->setChecked (value);
}

bool dlgAppSettings::alwaysTabBar ()
{
  return chkalwaystabbar->isChecked ();
}

void dlgAppSettings::setAlwaysTabBar (bool value)
{
  chkalwaystabbar->setChecked (value);
}

bool dlgAppSettings::auxInput ()
{
  return chkauxinput->isChecked ();
}

void dlgAppSettings::setAuxInput (bool val)
{
  chkauxinput->setChecked (val);
}

bool dlgAppSettings::alwaysNotify ()
{
  return chkalwaysnotify->isChecked ();
}

void dlgAppSettings::setAlwaysNotify (bool value)
{
  chkalwaysnotify->setChecked (value);
}

bool dlgAppSettings::localNotify ()
{
  return chklocalnotify->isChecked ();
}

void dlgAppSettings::setLocalNotify (bool value)
{
  chklocalnotify->setChecked (value);
}

bool dlgAppSettings::globalNotify ()
{
  return chkglobalnotify->isChecked ();
}

void dlgAppSettings::setGlobalNotify (bool value)
{
  chkglobalnotify->setChecked (value);
}

QString dlgAppSettings::autoConnect ()
{
  if (autoconnect->currentItem() == 0)  //no auto-connect
    return QString::null;
  else
    return autoconnect->currentText ();
}

void dlgAppSettings::setAutoConnect (const QString &ac)
{
  if (ac.length() == 0)
  {
    autoconnect->setCurrentItem (0);
    return;
  }
  int len = autoconnect->count ();
  //compare with each item except the first one (it's the "(disabled)" one)
  for (int i = 1; i < len; i++)
  {
    if (autoconnect->text (i) == ac)
    {
      autoconnect->setCurrentItem (i);
      return;
    }
  }
  //not found
  autoconnect->setCurrentItem (0);
}

void dlgAppSettings::fontchooser1 ()
{
  if (KFontDialog::getFont (font[0], true, this))
    fonted1->setText (font[0].family() + ", " +
          QString::number(font[0].pointSize()) +
          (font[0].bold() ? (" " + i18n("Bold")) : (QString) "") +
          (font[0].italic() ? (" " + i18n("Italic")) : (QString) ""));
}

void dlgAppSettings::fontchooser2 ()
{
  if (KFontDialog::getFont (font[1], false, this))
    fonted2->setText (font[1].family() + ", " +
          QString::number(font[1].pointSize()) +
          (font[1].bold() ? (" " + i18n("Bold")) : (QString) "") +
          (font[1].italic() ? (" " + i18n("Italic")) : (QString) ""));
}

void dlgAppSettings::fontchooser3 ()
{
  if (KFontDialog::getFont (font[2], false, this))
    fonted3->setText (font[2].family() + ", " +
          QString::number(font[2].pointSize()) +
          (font[2].bold() ? (" " + i18n("Bold")) : (QString) "") +
          (font[2].italic() ? (" " + i18n("Italic")) : (QString) ""));
}

void dlgAppSettings::setFont1 (QFont f)
{
  font[0] = f;
  fonted1->setText (font[0].family() + ", " +
        QString::number(font[0].pointSize()) +
        (font[0].bold() ? (" " + i18n("Bold")) : (QString) "") +
        (font[0].italic() ? (" " + i18n("Italic")) : (QString) ""));
}

void dlgAppSettings::setFont2 (QFont f)
{
  font[1] = f;
  fonted2->setText (font[1].family() + ", " +
        QString::number(font[1].pointSize()) +
        (font[1].bold() ? (" " + i18n("Bold")) : (QString) "") +
        (font[1].italic() ? (" " + i18n("Italic")) : (QString) ""));
}

void dlgAppSettings::setFont3 (QFont f)
{
  font[2] = f;
  fonted3->setText (font[2].family() + ", " +
        QString::number(font[2].pointSize()) +
        (font[2].bold() ? (" " + i18n("Bold")) : (QString) "") +
        (font[2].italic() ? (" " + i18n("Italic")) : (QString) ""));
}

QFont dlgAppSettings::font1 ()
{
  return font[0];
}

QFont dlgAppSettings::font2 ()
{
  return font[1];
}

QFont dlgAppSettings::font3 ()
{
  return font[2];
}

bool dlgAppSettings::keeptext ()
{
  return chkkeeptext->isChecked ();
}

bool dlgAppSettings::selectkepttext ()
{
  return chkselectkepttext->isChecked ();
}

bool dlgAppSettings::arrowsHistory ()
{
  return chkarrowshistory->isChecked ();
}

bool dlgAppSettings::autocompletion ()
{
  return chkautocompletion->isChecked ();
}

void dlgAppSettings::setKeepText (bool value)
{
  chkkeeptext->setChecked (value);
  chkselectkepttext->setEnabled (value);
}

void dlgAppSettings::setSelectKeptText (bool value)
{
  chkselectkepttext->setChecked (value);
}

void dlgAppSettings::setArrowsHistory (bool value)
{
  chkarrowshistory->setChecked (value);
}

void dlgAppSettings::setAutoCompletion (bool value)
{
  chkautocompletion->setChecked (value);
  comboactype->setEnabled (value);
}

int dlgAppSettings::actype ()
{
  switch (comboactype->currentItem ()) {
    case 0: return 2; break;
    case 1: return 3; break;
    case 2: return 5; break;
  };
  return 1; //1 is no AC
}

bool dlgAppSettings::telnetPaste ()
{
  return chktelnetpaste->isChecked ();
}

void dlgAppSettings::setTelnetPaste (bool value)
{
  chktelnetpaste->setChecked (value);
}

bool dlgAppSettings::trimSpaces ()
{
  return chktrimspaces->isChecked ();
}

void dlgAppSettings::setTrimSpaces (bool value)
{
  chktrimspaces->setChecked (value);
}

int dlgAppSettings::inputbkcolor ()
{
  return comboinputbkcolor->currentItem ();
}

int dlgAppSettings::inputcolor ()
{
  return comboinputcolor->currentItem ();
}

void dlgAppSettings::setACType (int value)
{
  switch (value) {
    case 2: comboactype->setCurrentItem (0); break;
    case 3: comboactype->setCurrentItem (1); break;
    case 5: comboactype->setCurrentItem (2); break;
  };
}

void dlgAppSettings::setInputBkColor (int value)
{
  comboinputbkcolor->setCurrentItem (value);
}

void dlgAppSettings::setInputColor (int value)
{
  comboinputcolor->setCurrentItem (value);
}

void dlgAppSettings::setColor (int i, QColor color)
{
  cbutton[i]->setColor (color);
}

QColor dlgAppSettings::color (int i)
{
  return cbutton[i]->color ();
}

QString dlgAppSettings::cmdseparator ()
{
  return edcmdseparator->text().stripWhiteSpace();
}

QString dlgAppSettings::speedwalk ()
{
  return edspeedwalk->text().stripWhiteSpace();
}

QString dlgAppSettings::script ()
{
  return edscript->text().stripWhiteSpace();
}

QString dlgAppSettings::multicmd ()
{
  return edmulticmd->text().stripWhiteSpace();
}

void dlgAppSettings::setCmdSeparator (QString what)
{
  edcmdseparator->setText (what);
}

void dlgAppSettings::setSpeedWalk (QString what)
{
  edspeedwalk->setText (what);
}

void dlgAppSettings::setScript (QString what)
{
  edscript->setText (what);
}

void dlgAppSettings::setMultiCmd (QString what)
{
  edmulticmd->setText (what);
}

bool dlgAppSettings::allowEmptyWalkStr ()
{
  return chkemptywalk->isChecked();
}

bool dlgAppSettings::expandBS ()
{
  return chkexpandBS->isChecked();
}

bool dlgAppSettings::BSOnlyInAT ()
{
  return chkBSonlyinAT->isChecked ();
}

void dlgAppSettings::setAllowEmptyWalkStr (bool val)
{
  chkemptywalk->setChecked (val);
}

void dlgAppSettings::setExpandBS (bool val)
{
  chkexpandBS->setChecked (val);
}

void dlgAppSettings::setBSOnlyInAT (bool val)
{
  chkBSonlyinAT->setChecked (val);
}

#ifdef HAVE_SOUND

const QStringList & dlgAppSettings::soundDirs ()
{
  return dirlist->getDirList();
}

bool dlgAppSettings::allowMSP ()
{
  return chkallowmsp->isChecked ();
}

bool dlgAppSettings::allowDownloads ()
{
  return chkallowdownloads->isChecked ();
}

void dlgAppSettings::setSoundDirs (const QStringList &list)
{
  dirlist->setDirList(list);
}

void dlgAppSettings::setAllowMSP (bool val)
{
  chkallowmsp->setChecked (val);
}

void dlgAppSettings::setAllowDownloads (bool val)
{
  chkallowdownloads->setChecked (val);
}

#endif  //HAVE_SOUND

void dlgAppSettings::applyKeys ()
{
  keys->commitChanges ();
}
