/***************************************************************************
                          dlgappsettings.h  -  application settings
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : St aug 14 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DLGAPPSETTINGS_H
#define DLGAPPSETTINGS_H

#include <config.h>

#include <qwidget.h>
#include <qfont.h>
#include <kdialogbase.h>

class QLabel;
class QComboBox;
class QCheckBox;
class KColorButton;
class KLineEdit;
class KIntNumInput;
class KKeyChooser;
class cDirList;

//TODO: constructor is a bit TOO complicated & long - should be splitted
//into more functions (1 function per page)!!!

/**
Dialog with main application settings.

  *@author Tomas Mecir
  */

class dlgAppSettings : public KDialogBase  {
   Q_OBJECT
public: 
  dlgAppSettings(QWidget *parent=0, const char *name=0);
  ~dlgAppSettings();

//functions to get/set values
//Window
  bool alwaysTabBar ();
  void setAlwaysTabBar (bool value);
  bool auxInput ();
  void setAuxInput (bool val);
  bool alwaysNotify ();
  void setAlwaysNotify (bool value);
  bool localNotify ();
  void setLocalNotify (bool value);
  bool globalNotify ();
  void setGlobalNotify (bool value);
  QString autoConnect ();
  void setAutoConnect (const QString &ac);

//Output area
  int getcolorindex (int which);
  void setcolorindex (int which, int value);
  bool cmdEcho ();
  bool messages ();
  void setCmdEcho (bool value);
  void setMessages (bool value);
  bool enableBlinking ();
  void setEnableBlinking (bool value);
  void setWordWrapping (bool value);
  bool wordWrapping ();
  void setIndentation (int value);
  int indentation ();
  void setHistory (int value);
  int history ();
  
//Font
  void setFont1 (QFont f);
  void setFont2 (QFont f);
  void setFont3 (QFont f);
  QFont font1 ();
  QFont font2 ();
  QFont font3 ();

//Inputline
  bool keeptext ();
  bool selectkepttext ();
  bool arrowsHistory ();
  bool autocompletion ();
  int actype ();
  void setKeepText (bool value);
  void setSelectKeptText (bool value);
  void setArrowsHistory (bool value);
  void setAutoCompletion (bool value);
  void setACType (int value);
  bool telnetPaste ();
  void setTelnetPaste (bool value);
  bool trimSpaces ();
  void setTrimSpaces (bool value);
  int inputbkcolor ();
  int inputcolor ();
  void setInputBkColor (int value);
  void setInputColor (int value);

//Colors
  void setColor (int i, QColor color);
  QColor color (int i);
  
//Characters
  QString cmdseparator ();
  QString speedwalk ();
  QString script ();
  QString multicmd ();
  void setCmdSeparator (QString what);
  void setSpeedWalk (QString what);
  void setScript (QString what);
  void setMultiCmd (QString what);

  bool allowEmptyWalkStr ();
  bool expandBS ();
  bool BSOnlyInAT ();
  void setAllowEmptyWalkStr (bool val);
  void setExpandBS (bool val);
  void setBSOnlyInAT (bool val);

#ifdef HAVE_SOUND
//MSP
  const QStringList &soundDirs ();
  bool allowMSP ();
  bool allowDownloads ();
  void setSoundDirs (const QStringList &list);
  void setAllowMSP (bool val);
  void setAllowDownloads (bool val);
#endif
  
//Shortcuts
  void applyKeys ();
protected:
//Window
  QCheckBox *chkalwaystabbar, *chkauxinput;
  QCheckBox *chkalwaysnotify, *chkglobalnotify, *chklocalnotify;
  QComboBox *autoconnect;
//Output area
  QComboBox *combo[5];
  QCheckBox *chkcmdecho, *chkmessages;
  QCheckBox *chkblinking;
  QCheckBox *chkwrap;
  KIntNumInput *edindent, *edhistory;
//Font
  QLabel *fonted1, *fonted2, *fonted3;
  QFont font[3];
//Inputline
  QCheckBox *chkkeeptext, *chkselectkepttext, *chkautocompletion;
  QCheckBox *chkarrowshistory;
  QCheckBox *chktelnetpaste;
  QCheckBox *chktrimspaces;
  QComboBox *comboactype, *comboinputbkcolor, *comboinputcolor;
//Colors
  KColorButton *cbutton[16];
//Characters
  KLineEdit *edcmdseparator, *edspeedwalk, *edscript, *edmulticmd;
  QCheckBox *chkemptywalk, *chkexpandBS, *chkBSonlyinAT;
//MSP
  QCheckBox *chkallowmsp, *chkallowdownloads;
  cDirList *dirlist;

//Shortcuts
  KKeyChooser *keys;
protected slots:
  void fontchooser1 ();
  void fontchooser2 ();
  void fontchooser3 ();
};

#endif
