/***************************************************************************
                          dlgconnect.cpp  -  Connect dialog
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Ut Jul 23 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../dialogs/dlgconnect.h"

#include "../cprofiles.h"
#include "../cconnectioninfo.h"

#include <qcheckbox.h>
#include <qlayout.h>
#include <qstringlist.h>
#include <qvbox.h>
#include <qwhatsthis.h>
#include <klistview.h>
#include <klocale.h>
#include <kpushbutton.h>

QString dlgConnect::lastSelected = QString::null;

dlgConnect::dlgConnect(QWidget *parent, const char *name ) :
    KDialogBase (parent, name, true, i18n ("Connect"), Ok|Cancel)
{
  //initial dialog size
  setInitialSize (QSize (500, 400));

  //create main dialog's widget
  QWidget *page = new QWidget (this);
  QGridLayout *layout = new QGridLayout (page, 3, 2);

  setMainWidget (page);

  //put some widgets there
  lw = new KListView (page);
  QWhatsThis::add (lw,
    i18n ("This list shows currently defined profiles.<p><b>Profiles</b> "
      "allow you to speed up connecting to your MUD, as well as to use "
      "more advanced features like <i>aliases</i> or <i>triggers</i>."));
  QVBox *vb = new QVBox (page);
  vb->setSpacing (5);
  addButton = new KPushButton (i18n ("&New profile"), vb);
  modifyButton = new KPushButton (i18n ("&Modify profile"), vb);
  deleteButton = new KPushButton (i18n ("&Delete profile"), vb);
  duplicateButton = new KPushButton (i18n ("&Duplicate profile"), vb);

  chkSendNothing = new QCheckBox (i18n ("Do not &send login sequence"), page);
  QWhatsThis::add (chkSendNothing, i18n ("Login sequence won't be sent for "
      "this connect. Useful when you're creating a new character and you "
      "want to use QuickConnect for some reason."));
  chkSendNothing->setChecked (false);
  
  layout->setRowStretch (0, 0);
  layout->setRowStretch (1, 5);
  layout->setSpacing (5);
  
  layout->addMultiCellWidget (lw, 0, 1, 0, 0);
  layout->addWidget (vb, 0, 1);
  layout->addWidget (chkSendNothing, 2, 0);
  
  //update button text
  setButtonOKText (i18n ("&Connect"),
      i18n ("Establishes connection with specified parameters."));
  setButtonCancelText (i18n ("C&lose"), i18n ("Closes this dialog box without connecting."));
  enableButtonSeparator (true);

  //connect signals
  connect (addButton, SIGNAL(clicked()), this, SIGNAL(addPressed()));
  connect (modifyButton, SIGNAL(clicked()), this, SIGNAL(modifyPressed()));
  connect (deleteButton, SIGNAL(clicked()), this, SIGNAL(deletePressed()));
  connect (duplicateButton, SIGNAL(clicked()), this, SIGNAL(duplicatePressed()));
  connect (lw, SIGNAL (selectionChanged()), this, SLOT (selectionChanged()));
  connect (lw, SIGNAL (doubleClicked (QListViewItem *)), this, SLOT (doubleClicked (QListViewItem *)));
          
  //update profile list
  plist = new QStringList;
  firstUpdate = true;
  updateList ();
  
  //restore last selected item, if any
  setSelectedItem (lastSelected);
  
  lw->setFocus ();
  selectionChanged ();
}

dlgConnect::~dlgConnect()
{
  delete plist;
}

QString dlgConnect::selName ()
{
  QListViewItem *lvi = lw->selectedItem ();
  if (lvi == NULL)
    return QString::null;
  return lvi->text (0);
}

bool dlgConnect::sendNothing ()
{
  return chkSendNothing->isChecked ();
}

void dlgConnect::setSelectedItem (const QString &name)
{
  QListViewItem *lvi = lw->firstChild ();
  if (name != QString::null)
  {
    while (lvi != 0)
    {
      if (lvi->text (0) == name)
      {
        lw->setCurrentItem (lvi);
        lw->setSelected (lvi, true);
        break;
      }
      lvi = lvi->itemBelow ();
    }
  }
  else
    if (lvi != 0)
    {
      lw->setCurrentItem (lvi);
      lw->setSelected (lvi, true);
    }
}

void dlgConnect::updateList ()
{
  //get item position
  int itemPos = -1;
  QListViewItem *lvi = lw->selectedItem ();
  if (lvi != 0)
  {
    QListViewItem *lvi2 = lw->firstChild ();
    itemPos = 0;
    while (lvi2 != 0)
    {
      if (lvi == lvi2)
        break;
      lvi2 = lvi2->itemBelow ();
      itemPos++;
    }
    if (lvi2 == 0)
      itemPos = -1; //???
  }
  
  //erase old data
  lw->clear ();
  
  //prepare columns, if needed
  //there are four: Profile name, server, port and login
  if (firstUpdate)
  {
    lw->addColumn (i18n ("Profile name"));
    lw->addColumn (i18n ("Server"));
    lw->addColumn (i18n ("Port"));
    lw->addColumn (i18n ("Login name"));
  }
  firstUpdate = false;
  lw->setSorting (0);
  lw->setShowSortIndicator (true);
      
  //prepare cConnection object, update profile list
  cProfiles *cp = new cProfiles ();
  *plist = cp->getprofiles();
  
  //now I fill in data, item by item...
  //for each item:
  int count = -1;
  for (QStringList::Iterator it = plist->begin (); it != plist->end (); ++it)
  {
    QString name = *it;

    cConnectionInfo * cci = cp->getProfileConnInfo (name);
    
    //get its properties
    QString server, port, login;
    server = cci->server ();
    port = QString::number (cci->port ());
    login = cci->login ();

    delete cci;
                
    //fill'em in!
    QListViewItem *lvi = new QListViewItem (lw, name, server, port, login);
    count++;
    if (count == itemPos)
      lw->setSelected (lvi, true);
  }
  
  //tidy up the mess...
  delete cp;
}

void dlgConnect::selectionChanged ()
{
  //enable/disable Connect button
  QListViewItem *lvi = lw->selectedItem ();
  if (lvi == NULL)
    enableButtonOK (false);
  else
  {
    enableButtonOK (true);
    lastSelected = lvi->text (0);
  }
}

void dlgConnect::doubleClicked (QListViewItem *i)
{
  if (i != 0)
    slotOk ();
}
