/***************************************************************************
                          dlgeditaliastrigger.cpp  -  edit alias/trigger
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Ne sep 22 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../dialogs/dlgeditaliastrigger.h"

#include "../calias.h"
#include "../ctrigger.h"
#include "../csession.h"
#include "../cgroup.h"
#include "../cgrouplist.h"
#include "../cconnection.h"
#include "../cconnprefs.h"
#include "../cwindowlist.h"

#include "cnumberedlistitem.h"

#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qwhatsthis.h>
#include <qstringlist.h>

#include <kapp.h>
#include <kfiledialog.h>
#include <klineedit.h>
#include <klistview.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <knuminput.h>
#include <ktextedit.h>

//color list
QString cltextk[17] = { i18n("Keep"), i18n("Black"), i18n("Red"), i18n("Green"),
    i18n("Yellow"), i18n("Blue"), i18n("Magenta"), i18n("Cyan"), i18n("Gray"),
    i18n("Dark gray"), i18n("Bright red"), i18n("Bright green"),
    i18n("Bright yellow"), i18n("Bright blue"), i18n("Bright magenta"),
    i18n("Bright cyan"), i18n("White") };

QString ltype[7] = { i18n("Whole line"), i18n("Matching text"),
    i18n("Prefix"), i18n("Suffix"), i18n("Trimmed prefix"),
    i18n("Trimmed suffix"), i18n ("Backreference") };

dlgEditAliasTrigger::dlgEditAliasTrigger (cAlias *a, cSession *s, QWidget *parent, const char *name) :
    KDialogBase (Tabbed, i18n ("Edit alias"), Ok|Cancel, Ok, parent, name)
{
  isalias = true;
  alias = a;
  trigger = NULL;
  sess = s;

  createDialog ();
}

dlgEditAliasTrigger::dlgEditAliasTrigger (cTrigger *t, cSession *s, QWidget *parent, const char *name) :
    KDialogBase (Tabbed, i18n ("Edit trigger"), Ok|Cancel, Ok, parent, name)
{
  isalias = false;
  alias = NULL;
  trigger = t;
  sess = s;

  createDialog ();
}

void dlgEditAliasTrigger::createDialog ()
{
  //initial dialog size
  setInitialSize (QSize (400, 400));

  //create pages
  QFrame *basicpage = addPage (i18n ("Basic"));
  QFrame *optionspage = addPage (i18n ("Options"));
  QFrame *colorpage = 0, *specialpage = 0, *windowpage = 0;
  if (!isalias)  //some trigger-only pages
  {
    colorpage = addPage (i18n ("Color replacement"));
    specialpage = addPage (i18n ("Special trigger"));
    windowpage = addPage (i18n ("Output windows"));
  }

  firstUpdate = true;
  
  //basic page
  QGridLayout *basiclayout = new QGridLayout (basicpage, 5, 2);

   //command
  cl = new QLabel (isalias ? i18n ("&Alias text") : i18n ("&Trigger text"), basicpage);
  cmd = new KLineEdit (basicpage);
  cl->setBuddy (cmd);

  if (isalias)
    QWhatsThis::add (cmd, i18n ("Command that will be replaced if you enter it."));
  else
    QWhatsThis::add (cmd, i18n ("Text that activate the trigger if it's received from the server."));
  
   //comparison type
  ctl = new QLabel ("&Comparison type", basicpage);
  type = new QComboBox (basicpage);
  ctl->setBuddy (type);
  type->clear ();
  type->insertItem (i18n ("Exact match"));
  type->insertItem (i18n ("Sub-string"));
  type->insertItem (i18n ("Begins with"));
  type->insertItem (i18n ("Ends with"));
  type->insertItem (i18n ("Regular expression"));
  if (isalias)
    QWhatsThis::add (type, i18n ("Type of matching. Alias will only be activated "
        "if this passes this type of test. Note that aliases are always matched as whole words."));
  else
    QWhatsThis::add (type, i18n ("Type of matching. Trigger will only be activated "
        "if this passes this type of test."));

   //replacement command(s)
  rcl = new QLabel (isalias ? i18n ("&Replacement text(s)") : i18n ("&Send command(s)"), basicpage);
  rcmd = new KTextEdit (basicpage);
  rcmd->setTextFormat (Qt::PlainText);
  rcmd->setWordWrap (QTextEdit::NoWrap);
  rcl->setBuddy (rcmd);
  if (isalias)
    QWhatsThis::add (rcmd, i18n ("Command(s) that will replace command you've entered."));
  else
    QWhatsThis::add (rcmd, i18n ("Command(s) sent to the MUD when this trigger gets activated."));

  //test area
  testarea = new QGroupBox (1, Horizontal, i18n ("Test area"), basicpage);
  textlabel = new QLabel (i18n ("&Text: "), testarea);
  text = new KLineEdit (testarea);
  textlabel->setBuddy (text);
  matched = new QLabel ("", testarea);
  replacement = new QLabel ("", testarea);
  variables = new QListView (testarea);
  variables->addColumn (i18n ("Name"));
  variables->addColumn (i18n ("Value"));
  variables->setAllColumnsShowFocus (true);
  QWhatsThis::add (testarea, i18n("This is testing area. You can test your alias or "
        "trigger here to see if it does what you want it to do. Simply "
        "type in some text and see what happens."));

  basiclayout->setSpacing (10);
  basiclayout->addWidget (cl, 0, 0);
  basiclayout->addWidget (cmd, 0, 1);
  basiclayout->addWidget (ctl, 1, 0);
  basiclayout->addWidget (type, 1, 1);
  basiclayout->addWidget (rcl, 2, 0);
  basiclayout->addMultiCellWidget (rcmd, 3, 3, 0, 1);
  basiclayout->setRowStretch (3, 10);
  basiclayout->addMultiCellWidget (testarea, 4, 4, 0, 1);

  //options page
  QGridLayout *optionslayout = new QGridLayout (optionspage, 3, 2);

  options = new QButtonGroup (2, Vertical, i18n ("&Options"), optionspage);
  check1 = new QCheckBox (isalias ? i18n ("Send original command") : i18n ("Continue if matched"), options);
  if (isalias)
    QWhatsThis::add (check1, i18n ("If this alias matches your command, "
          "a replacement command will be sent. If you enable this option, "
          "both original and replacement commands are sent (original command "
          "is sent first)."));
  else
    QWhatsThis::add (check1, i18n ("Triggers are matched in order that you specify. "
        "If you enable this checkbox, matching will continue even if the text "
        "matches this trigger. Note that if other triggers match this text "
        "too, more replacement commands will be sent, so be careful if you "
        "use this often."));

  check2 = new QCheckBox (isalias ? i18n ("Whole words only") : i18n ("Continue if not matched"), options);
  if (isalias)
    QWhatsThis::add (check2, i18n("When enabled, this alias will only match "
    "the text if there are spaces or nothing before/after the matched string."));
  else
    QWhatsThis::add (check2, i18n("Triggers are matched in order that you specify. "
        "If you enable this checkbox, matching will continue if the text "
        "didn't match this trigger. Disable this if you want to enable "
        "activation of further triggers only under some circumstances."));

  check3 = new QCheckBox (i18n ("Case sensitive"), options);
  QWhatsThis::add (check3, i18n ("When this option is on, upper case and "
      "lower case letters are treated as different characters, otherwise "
      "they're considered to be the same."));

  check4 = new QCheckBox (isalias ? i18n ("Include prefix/suffix") :
      i18n ("Show, don't send"), options);
  QWhatsThis::add (check4, isalias ? i18n ("Prefix/suffix of matched text "
      "will be automatically appended to the expanded text.") :
      i18n ("When enabled, the text won't be send to the MUD, it will be "
      "only displayed on the screen."));

   //group
  gl = new QLabel ("&Group", optionspage);
  groups = new QComboBox (optionspage);
  gl->setBuddy (groups);
  groups->clear ();
  cSaveableList *list;
  if (isalias)
    list = sess->connection()->getConnPrefs()->aliasGroupList();
  else
    list = sess->connection()->getConnPrefs()->triggerGroupList();
  if (list != 0)    //should never be 0, but just in case...
  {
    for (list->reset(); *list; (*list)++)
    {
      //fill in group names!
      cGroup *group = (cGroup *) **list;
      groups->insertItem (group->getText());
    }
  }
  if (isalias)
    QWhatsThis::add (groups, i18n ("Choose the appropriate group for your alias. "
        "Groups can be used to activate/deactivate certain aliases."));
  else
    QWhatsThis::add (groups, i18n ("Choose the appropriate group for your trigger. "
        "Groups can be used to activate/deactivate certain triggers."));
  optionslayout->setSpacing (10);
  optionslayout->addMultiCellWidget (options, 0, 0, 0, 1);
  optionslayout->addWidget (gl, 1, 0);
  optionslayout->addWidget (groups, 1, 1);
  optionslayout->setColStretch (1, 10);
  optionslayout->setRowStretch (2, 10);
  
  //trigger-only pages
  if (!isalias)
  {
    //color replacement page
    QVBoxLayout *colorlayout = new QVBoxLayout (colorpage);

    chkiscolor = new QCheckBox (i18n ("&Enable color replacement"), colorpage);
    colorgroup = new QGroupBox (1, Horizontal, i18n ("Color replacement"), colorpage);

    QStringList types;
    for (int i = 0; i < 7; i++)
      types.append (ltype[i]);
    QStringList colorlist;
    for (int i = 0; i < 17; i++)
      colorlist.append (cltextk[i]);

    QWidget *onecolor = new QWidget (colorgroup);
    QGridLayout *onecolorlayout = new QGridLayout (onecolor, 3, 3, 0, 10);
    clrtype = new QComboBox (false, onecolor);
    clrtype->insertStringList (types);
    fgselect = new QComboBox (false, onecolor);
    fgselect->insertStringList (colorlist);
    bgselect = new QComboBox (false, onecolor);
    bgselect->insertStringList (colorlist);
    backrefno = new KIntNumInput (onecolor);
    backrefno->setLabel (i18n ("&Back-reference"), KNumInput::AlignLeft);
    backrefno->setRange (0, 100, 1, false);
    QPushButton *btadd = new QPushButton (i18n ("&Add"), onecolor);
    QPushButton *btremove = new QPushButton (i18n ("&Remove"), onecolor);
    QLabel *l1 = new QLabel (i18n ("&Colorize"), onecolor);
    l1->setBuddy (clrtype);
    QLabel *l2 = new QLabel (i18n ("&Foreground"), onecolor);
    l2->setBuddy (fgselect);
    QLabel *l3 = new QLabel (i18n ("&Background"), onecolor);
    l3->setBuddy (bgselect);

    onecolorlayout->addWidget (l1, 0, 0);
    onecolorlayout->addWidget (l2, 0, 1);
    onecolorlayout->addWidget (l3, 0, 2);
    onecolorlayout->addWidget (clrtype, 1, 0);
    onecolorlayout->addWidget (fgselect, 1, 1);
    onecolorlayout->addWidget (bgselect, 1, 2);
    onecolorlayout->addWidget (backrefno, 2, 0);
    onecolorlayout->addWidget (btadd, 2, 1);
    onecolorlayout->addWidget (btremove, 2, 2);

    colorizations = new KListView (colorgroup);

    colorlayout->setSpacing (10);
    colorlayout->addWidget (chkiscolor);
    colorlayout->addWidget (colorgroup);

    connect (chkiscolor, SIGNAL (toggled (bool)), colorgroup, SLOT (setEnabled (bool)));
    connect (btadd, SIGNAL (clicked ()), this, SLOT (addColorization ()));
    connect (btremove, SIGNAL (clicked ()), this, SLOT (removeColorization ()));
    

    //special trigger page

#ifdef HAVE_SOUND
    QGridLayout *speciallayout = new QGridLayout (specialpage, 5, 3);
#else
    QGridLayout *speciallayout = new QGridLayout (specialpage, 4, 3);
#endif
    chkgag = new QCheckBox (i18n ("Don't show (&gag) the line"), specialpage);
    QWhatsThis::add (chkgag, i18n ("This will prevent the matched line "
        "from being displayed in the output window."));
    chknotify = new QCheckBox (i18n ("&Notify"), specialpage);
    QWhatsThis::add (chknotify, i18n ("This setting only takes effect if you "
      "have disabled the \"Always notify\" option in Global settings and "
      "at least one of global/local notification is enabled. It will "
      "activate notification if the KMuddy window and/or this session "
      "is not active at the moment.\nUseful if you only want to be notified "
      "of some events (tells, combat, ...)."));

    chkprompt = new QCheckBox (i18n ("&Prompt-detection trigger"), specialpage);
    QWhatsThis::add (chkprompt, i18n ("Enabling this will turn this trigger "
      "into a prompt-detection trigger, that is, text will be passed to these "
      "triggers even if no newline was received, the text will be displayed "
      "as a prompt if the trigger gets matched, and most other options will "
      "NOT be taken into effect - prompt-detect trigger cannot send out "
      "text, gag lines, do notification. Colorization is still possible."));
#ifdef HAVE_SOUND
    chksound = new QCheckBox (i18n ("Play s&ound file"), specialpage);
    QWhatsThis::add (chksound, i18n ("This trigger will play a sound. Useful "
      "as a notification of important events, or to assign sounds to some "
      "events."));
    edsoundname = new KLineEdit (specialpage);
    QPushButton *button = new QPushButton ("&Browse...", specialpage);
    connect (button, SIGNAL (clicked ()), this, SLOT (browseForSoundFile ()));
#endif

    speciallayout->setSpacing (10);
    speciallayout->addWidget (chkgag, 0, 0);
    speciallayout->addWidget (chknotify, 1, 0);
    speciallayout->addWidget (chkprompt, 2, 0);
#ifdef HAVE_SOUND
    speciallayout->addWidget (chksound, 3, 0);
    speciallayout->addWidget (edsoundname, 3, 1);
    speciallayout->addWidget (button, 3, 2);
    speciallayout->setRowStretch (4, 5);
#else
    speciallayout->setRowStretch (3, 5);
#endif
    
    // output window page
    
    QGridLayout *windowlayout = new QGridLayout (windowpage, 2, 2);
    
    chkwindow = new QCheckBox (i18n ("Send output to a separate window"), windowpage);
    QWhatsThis::add (chkwindow, i18n ("This will send the matched text to a separate output window"));
    connect (chkwindow, SIGNAL (toggled (bool)), this, SLOT (updateWindowDialog (bool)));
    
    outwindow = new QGroupBox(1, Horizontal, i18n ("Output Windows"), windowpage);
    
    wll = new QLabel("&Select window", outwindow);
    windowlist = new QComboBox(outwindow);
    wll->setBuddy(windowlist);
    windowlist->clear();
    QStringList wlist = sess->connection()->getConnPrefs()->windowList()->windowList();
    windowlist->insertStringList(wlist);
    
    chkgagoutput = new QCheckBox (i18n ("&Gag output in main window"), outwindow);
    QWhatsThis::add (chkgagoutput, i18n ("This will cause output sent to separate window to not to "
        "be displayed in main KMuddy session window"));

    wnl = new QLabel ("&Window name", outwindow);
    wname = new KLineEdit (outwindow);
    wnl->setBuddy(wname);

    createwindow = new QPushButton ("&Create window", outwindow);
    connect (createwindow, SIGNAL (clicked ()), this, SLOT (createOutputWindow ()));

    windowlayout->setSpacing(10);
    windowlayout->addWidget(chkwindow, 0, 0);
    windowlayout->addMultiCellWidget(outwindow, 1, 1, 0, 2);

    updateWindowDialog(false);
  }

  //make testarea work!
  connect (text, SIGNAL (textChanged (const QString &)), this, SLOT (updateTest (const QString &)));
  connect (cmd, SIGNAL (textChanged (const QString &)), this, SLOT (updateTest (const QString &)));
  connect (rcmd, SIGNAL (textChanged ()), this, SLOT (updateTest ()));
  connect (type, SIGNAL (activated (const QString &)), this, SLOT (updateTest (const QString &)));
  connect (check3, SIGNAL (toggled(bool)), this, SLOT (updateTest (bool)));
  if (isalias)
  {
    connect (check2, SIGNAL (toggled(bool)), this, SLOT (updateTest (bool)));
    connect (check4, SIGNAL (toggled(bool)), this, SLOT (updateTest (bool)));
  }

}

dlgEditAliasTrigger::~dlgEditAliasTrigger ()
{

}

bool dlgEditAliasTrigger::showMe ()
//fills and shows the dialog, then applies changes if needed
{
  //fill in the dialog
  int num;
  cGroupList *list;
  if (isalias)
  {
    cmd->setText (alias->getText ());

    //taken from dlgEditProfile :)
    QStringList newtext = alias->getNewText ();
    rcmd->clear ();
    QStringList::iterator it;
    for (it = newtext.begin(); it != newtext.end(); ++it)
      if (it != newtext.fromLast())
        rcmd->insertParagraph (*it, -1);
      else
        rcmd->append (*it);

    type->setCurrentItem (alias->getType ());
    check1->setChecked (alias->sendOriginal ());
    check2->setChecked (alias->wholeWords ());
    check3->setChecked (alias->caseSensitive ());
    check4->setChecked (alias->includePrefixSuffix ());
    
    //groupname is a bit more complicated...
    //here we only set  required params; most of the code is common for
    //both aliases and triggers, so it's included in the common part...
    num = alias->group();
    list = sess->connection()->getConnPrefs()->aliasGroupList();
  }
  else
  {
    cmd->setText (trigger->getText ());

    //taken from dlgEditProfile :)
    QStringList newtext = trigger->getNewText ();
    rcmd->clear ();
    QStringList::iterator it;
    for (it = newtext.begin(); it != newtext.end(); ++it)
      if (it != newtext.fromLast())
        rcmd->insertParagraph (*it, -1);
      else
        rcmd->append (*it);

    type->setCurrentItem (trigger->getType ());
    check1->setChecked (trigger->continueIfMatch ());
    check2->setChecked (trigger->continueIfNoMatch ());
    check3->setChecked (trigger->caseSensitive ());
    check4->setChecked (trigger->dontSend ());

    //groupname is a bit more complicated...
    //here we only set required params; most of the code is common for
    //both aliases and triggers, so it's included in the common part...
    num = trigger->group();
    list = sess->connection()->getConnPrefs()->triggerGroupList();

    //special trigger settings
    chkiscolor->setChecked (trigger->isColorTrigger());
    colorgroup->setEnabled (chkiscolor->isChecked ());
    cnum = trigger->getColorizationsCount();
    for (int i = 0; i < cnum; i++)
    {
      ctype[i] = trigger->getColorizationVariable(i);
      int color = trigger->getColorizationColor(i);
      fgcolor[i] = color % 256;
      bgcolor[i] = color / 256;
    }
    updateColorizationsList ();
    chkgag->setChecked (trigger->isGagTrigger());
    chknotify->setChecked (trigger->isNotifyTrigger());
    chkprompt->setChecked (trigger->isPromptDetectTrigger());
#ifdef HAVE_SOUND
    chksound->setChecked (trigger->isSoundTrigger());
    edsoundname->setText (trigger->soundFileName());
#endif

    chkwindow->setChecked (trigger->isOutputWindowTrigger());
    chkgagoutput->setChecked (trigger->isGagOutputWindow());
    int pos = 0;
    QStringList twlist = sess->connection()->getConnPrefs()->windowList()->windowList();
    for ( QStringList::Iterator it = twlist.begin(); it != twlist.end(); ++it )
    {
      if(*it == trigger->getOutputWindowName())
        break;
      pos++;
    }
    //set position, if in range
    if (it != twlist.end())
      windowlist->setCurrentItem (pos);
  }

  //here we set the correct group
  int idx = -1;
  bool foundIt = false;
  //find the group wih requested ID
  for (list->reset(); *list; (*list)++)
  {
    idx++;  //update idx
    cGroup *g = (cGroup *) **list;
    if (g->number() == num) //FOUND!
    {
      foundIt = true;
      break;
    }
  }
  //if we haven't found, then the correct value is 0
  if (!foundIt)
    idx = 0;

  //this could fail is there is no such group. But this would mean a serious
  //flaw somewhere in the program, so I ignore it and hope for the best...
  // in case of that flaw, there will be no current item
  groups->setCurrentItem (idx);
  
  //show it!
  int n = exec ();

  //update structure if needed
  if (n == Accepted)
  {
    int groupnumber;
    //first we find out the group number, so that we can store it in cAlias
    //or cTrigger...
    list->reset();
    for (int i = 0; i < groups->currentItem(); i++)
      (*list)++;
    cGroup *g = (cGroup *) **list;
    groupnumber = g->number();
    
    if (isalias)
    {
      alias->setText (cmd->text ());

      //taken from dlgEditProfile
      QStringList ntext;
      int pars = rcmd->paragraphs ();
      for (int i = 0; i < pars; i++)
        ntext << (rcmd->text(i).stripWhiteSpace());
      alias->setNewText (ntext);

      alias->setType (type->currentItem ());
      alias->setSendOriginal (check1->isChecked ());
      alias->setWholeWords (check2->isChecked ());
      alias->setCaseSensitive (check3->isChecked ());
      alias->setIncludePrefixSuffix (check4->isChecked ());
      alias->setGroup (groupnumber);
      //activate the change of group
      list->updateAssignedList ();
    }
    else
    {
      trigger->setText (cmd->text ());

      //taken from dlgEditProfile
      QStringList ntext;
      int pars = rcmd->paragraphs ();
      for (int i = 0; i < pars; i++)
        ntext << (rcmd->text(i).stripWhiteSpace());
      trigger->setNewText (ntext);

      trigger->setType (type->currentItem ());
      trigger->setContinueIfMatch (check1->isChecked ());
      trigger->setContinueIfNoMatch (check2->isChecked ());
      trigger->setCaseSensitive (check3->isChecked ());
      trigger->setDontSend (check4->isChecked ());
      trigger->setGroup (groupnumber);
      //activate the change of group
      list->updateAssignedList();

      //special trigger settings
      bool ct = chkiscolor->isChecked ();
      trigger->setColorTrigger (ct);
      trigger->clearColorizations ();
      if (ct)
        for (int i = 0; i < cnum; i++)
          trigger->addColorization (ctype[i], bgcolor[i] * 256 + fgcolor[i]);
      trigger->setGagTrigger (chkgag->isChecked ());
      trigger->setNotifyTrigger (chknotify->isChecked ());
      trigger->setPromptDetectTrigger (chkprompt->isChecked ());
#ifdef HAVE_SOUND
      trigger->setSoundTrigger (chksound->isChecked ());
      trigger->setSoundFileName (edsoundname->text ());
#endif
      trigger->setOutputWindowTrigger (chkwindow->isChecked());
      trigger->setGagOutputWindow(chkgagoutput->isChecked());
      trigger->setOutputWindowName(windowlist->currentText());
    }
    return true;
  }
  else
    return false;
}

void dlgEditAliasTrigger::updateTest ()
{
  QString s;
  updateTest (s);
}

void dlgEditAliasTrigger::updateTest (bool)
{
  //this slot is activated when WholeWords status changed in alias mode
  QString s;
  updateTest (s);
}

void dlgEditAliasTrigger::updateTest (const QString &)
{
  QString txt = text->text();
  static bool inside = false;
  if (inside)
    return;
  inside = true;
  //create a new alias/trigger
  cSaveableField *test;
  if (isalias)
    test = new cAlias (sess->sendcmd());
  else
    test = new cTrigger (sess);

  //set it up with dialog data
  test->setText (cmd->text ());
  QStringList ntext;
  int pars = rcmd->paragraphs ();
  for (int i = 0; i < pars; i++)
    ntext << (rcmd->text(i).stripWhiteSpace());
  test->setNewText (ntext);
  test->setType (type->currentItem ());
  test->setNoGroupMode ();
  test->setCaseSensitive (check3->isChecked ());
  if (isalias)
  {
    ((cAlias *) test)->setWholeWords (check2->isChecked ());
    ((cAlias *) test)->setIncludePrefixSuffix (check4->isChecked ());
  }
  //try to match the text against that object
  //we cast to cSaveableField to prevent problems with prompt-detection
  //triggers
  bool ismatch = test->cSaveableField::match (txt);
  
  //write results of the matching
  variables->clear ();
  if (ismatch)
  {
    matched->setText (isalias ? i18n ("This text matches your alias.") :
        i18n ("This text matches your trigger."));
    QStringList ntext = test->getNewText();
    QString nt;
    bool prefixsuffix = isalias ? check4->isChecked() : false;
    QStringList::iterator it;
    //we cannot use QStringList::join because of prefix/suffix stuff
    for (it = ntext.begin(); it != ntext.end(); ++it)
    {
      QString text = *it;
      test->expandPseudoVariables (text);
      if (prefixsuffix)
        text = test->getPrefix() + text + test->getSuffix();
      nt += text + "\n  ";
    }
    
    replacement->setText (i18n ("Command: ") + nt);
    new QListViewItem (variables, "$matched", test->processVariable ("matched"));
    new QListViewItem (variables, "$prefix", test->processVariable ("prefix"));
    new QListViewItem (variables, "$suffix", test->processVariable ("suffix"));
    new QListViewItem (variables, "$prefixtrim", test->processVariable ("prefixtrim"));
    new QListViewItem (variables, "$suffixtrim", test->processVariable ("suffixtrim"));
    if (test->getType() == cSaveableField::regexp)
      for (unsigned int i = 0; i < test->getBackRefList().count(); i++)
        new QListViewItem (variables, "$"+QString::number(i), test->getBackRefList()[i]);
  }
  else
  {
    matched->setText (isalias ? i18n ("This text did not match your alias.") :
        i18n ("This text did not match your trigger."));
    replacement->setText ("");
  }
  
  //tidy up the mess
  delete test;
  inside = false;
}

void dlgEditAliasTrigger::addColorization ()
{
  if (cnum == MAX_COLORIZATIONS)  //limit reached!
  {
    KMessageBox::sorry (this, i18n ("Maximum number of colorizations reached."));
    return;
  }
  int fg = fgselect->currentItem () - 1;
  int bg = bgselect->currentItem () - 1;
  if (fg == -1) fg = 255;
  if (bg == -1) bg = 255;
  QString varname;
  int t = clrtype->currentItem ();

  switch (t) {
    case 0: varname = "$line"; break;
    case 1: varname = "$matched"; break;
    case 2: varname = "$prefix"; break;
    case 3: varname = "$suffix"; break;
    case 4: varname = "$prefixtrim"; break;
    case 5: varname = "$suffixtrim"; break;
    case 6: varname = "$"; break;
  };
  if (t == 6)
    varname += QString::number (backrefno->value ());

  //values are ready, fill'em in!
  fgcolor[cnum] = fg;
  bgcolor[cnum] = bg;
  ctype[cnum] = varname;
  cnum++;
  updateColorizationsList ();
}

void dlgEditAliasTrigger::removeColorization ()
{
  cNumberedListItem *lwi = (cNumberedListItem *) colorizations->selectedItem ();
  if (lwi == 0)
    return;
  int num = lwi->number ();
  for (int i = num; i < cnum - 1; i++)
  {
    ctype[i] = ctype[i + 1];
    fgcolor[i] = fgcolor[i + 1];
    bgcolor[i] = bgcolor[i + 1];
  }
  cnum--;
  //this will also fix item numbers
  updateColorizationsList ();
}

void dlgEditAliasTrigger::updateColorizationsList ()
{
  //erase old data
  colorizations->clear ();

  //prepare columns, if needed
  //there are three: Type, fg-color and bg-color
  if (firstUpdate)
  {
    colorizations->addColumn (i18n ("Type"));
    colorizations->addColumn (i18n ("Foreground"));
    colorizations->addColumn (i18n ("Background"));
  }
  firstUpdate = false;
  colorizations->setSorting (-1);

  //now I fill in data, item by item...
  //for each item:
  cNumberedListItem *prevnli = 0;
  int count = 0;
  for (int i = 0; i < cnum; i++)
  {
    QString stype, sfg, sbg;

    //prepare strings
    QString varname = ctype[i];
    if (varname == "$line")
      stype = ltype[0];
    else if (varname == "$matched")
      stype = ltype[1];
    else if (varname == "$prefix")
      stype = ltype[2];
    else if (varname == "$suffix")
      stype = ltype[3];
    else if (varname == "$prefixtrim")
      stype = ltype[4];
    else if (varname == "$suffixtrim")
      stype = ltype[5];
    else if (varname[0] == '$')
    {
      //now it *should* be a back-reference
      varname = varname.mid(1);
      bool ok;
      varname.toInt (&ok);
      if (ok) //back-ref -> good :)
        stype = ltype[6] + " " + varname;
      else
        stype = i18n ("Unknown");
    }

    //fg/bg color
    int fg = fgcolor[i];
    int bg = bgcolor[i];
    fg = (fg == 255) ? 0 : fg+1;
    bg = (bg == 255) ? 0 : bg+1;
    sfg = cltextk[fg];
    sbg = cltextk[bg];

    //fill'em in!
    cNumberedListItem *nli = new cNumberedListItem (colorizations, prevnli, stype, sfg, sbg);
    prevnli = nli;
    nli->setNumber (count);
    count++;
    if (count == cnum - 1)  //last item shall be selected
      colorizations->setSelected (nli, true);
  }
}

void dlgEditAliasTrigger::browseForSoundFile ()
{
#ifdef HAVE_SOUND
  //open some file and place its name to the edit box
  QString fName = KFileDialog::getOpenFileName (QString::null,
        "audio/x-wav", this, i18n ("Choose sound file"));
  if (!(fName.isEmpty()))
    edsoundname->setText (fName);
#endif
}

void dlgEditAliasTrigger::updateWindowDialog(bool how)
{
  windowlist->setEnabled(how);
  chkgagoutput->setEnabled(how);
  createwindow->setEnabled(how);
  wname->setEnabled(how);
}

void dlgEditAliasTrigger::createOutputWindow ()
{
  QString winname;
  
  winname = wname->text();
  if(!winname.isEmpty())
  {
    sess->connection()->getConnPrefs()->windowList()->add(wname->text());
    windowlist->clear();
    QStringList wlist = sess->connection()->getConnPrefs()->windowList()->windowList();
    windowlist->insertStringList(wlist);
  }
  else
    KMessageBox::sorry(this, i18n("Window name is empty!"));
}
