/***************************************************************************
                          dlgeditprofile.cpp  -  Edit Profile dialog
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : P Aug 9 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../dialogs/dlgeditprofile.h"

//needed by gcc 3.2
#define max(a,b) (((a) >= (b)) ? (a) : (b))

#include <qlayout.h>
#include <qlabel.h>
#include <qtextedit.h>
#include <qwhatsthis.h>

#include <klocale.h>
#include <kpassdlg.h>
#include <klineedit.h>
#include <krestrictedline.h>
#include <kmessagebox.h>

dlgEditProfile::dlgEditProfile(bool isNameEnabled, QWidget *parent, const char *name ) :
    KDialogBase (parent, name, true, i18n ("Edit profile"), Ok|Cancel)
{
  //initial dialog size
  setInitialSize (QSize (350, 350));

  //create main dialog's widget
  QWidget *page = new QWidget (this);
  QGridLayout *layout = new QGridLayout (page, 2, 7);

  setMainWidget (page);

  //put some edit boxes there
  QLabel *l1 = new QLabel (i18n ("&Profile name:"), page);
  ed1 = new KLineEdit (page);
  QWhatsThis::add (ed1, i18n ("Enter profile name here. You can enter any "
  "name, but it must be <b>unique</b>."));
  QLabel *l2 = new QLabel (i18n ("&Server:"), page);
  ed2 = new KLineEdit (page);
  QWhatsThis::add (ed2, i18n ("Enter address of the server where your MUD is running."));
  QLabel *l3 = new QLabel (i18n ("&Port:"), page);
  ed3 = new KRestrictedLine (page, 0, "0123456789");
  QWhatsThis::add (ed3, i18n ("Enter port on which it's running (usually some 4-digit number)."));
  QLabel *l4 = new QLabel (i18n ("&Login name:"), page);
  ed4 = new KLineEdit (page);
  QWhatsThis::add (ed4, i18n ("Enter your login name on the MUD here.<p><i>This setting is optional.</i>"));
  QLabel *l5 = new QLabel (i18n ("&Password:"), page);
  ed5 = new KLineEdit (page);
  ed5->setEchoMode (KLineEdit::Password);
  QWhatsThis::add (ed5, i18n ("Enter your password on the MUD here.<p><i>This setting is optional.</i>"));
  QLabel *l6 = new QLabel (i18n ("Connect sequence"), page);
  connstr = new QTextEdit (page);
  connstr->setTextFormat (Qt::PlainText);
  connstr->setWordWrap (QTextEdit::NoWrap);
  QWhatsThis::add (connstr, i18n ("Commands to be sent to the MUD after "
      "logging in.<br><b>$name</b> expands to username.<br><b>$password</b> "
      "expands to your password.<p><b>Important:</b> these commands are sent "
      "exactly as you type them, with leading/trailing spaces removed. "
      "No aliases, no macros, no speed-walk. Also note that empty lines "
      "will also be sent, including the very last line."));
  
  ed3->setFixedWidth (80);

  l1->setBuddy (ed1);
  l2->setBuddy (ed2);
  l3->setBuddy (ed3);
  l4->setBuddy (ed4);
  l5->setBuddy (ed5);
  l6->setBuddy (connstr);
  
  //disable Name if needed
  if (!isNameEnabled)
  {
    l1->setEnabled (false);
    ed1->setEnabled (false);
  }
  
  layout->setSpacing (10);
  layout->addWidget (l1, 0, 0);
  layout->addWidget (l2, 1, 0);
  layout->addWidget (l3, 2, 0);
  layout->addWidget (l4, 3, 0);
  layout->addWidget (l5, 4, 0);
  layout->addWidget (l6, 5, 0);
  layout->addWidget (ed1, 0, 1);
  layout->addWidget (ed2, 1, 1);
  layout->addWidget (ed3, 2, 1);
  layout->addWidget (ed4, 3, 1);
  layout->addWidget (ed5, 4, 1);
  layout->addMultiCellWidget (connstr, 6, 6, 0, 1);

  //humm, this one seems to be causing some X Error 42 BadMatch...
  //I have no idea why does it happen; however, everything seems
  //to be working perfectly...
  if (isNameEnabled)
    ed1->setFocus ();
  else
    ed2->setFocus ();
}

dlgEditProfile::~dlgEditProfile()
{
  //nothing here
}

QString dlgEditProfile::name ()
{
  return ed1->text ();
}

QString dlgEditProfile::server ()
{
  return ed2->text();
}

int dlgEditProfile::port ()
{
  return ed3->text().toInt ();
}

QString dlgEditProfile::login ()
{
  return ed4->text ();
}

QString dlgEditProfile::password ()
{
  return ed5->text ();
}

QStringList dlgEditProfile::connectionString ()
{
  QStringList list;
  int pars = connstr->paragraphs ();
  for (int i = 0; i < pars; i++)
    list << (connstr->text(i).stripWhiteSpace());
  return list;
}

void dlgEditProfile::setName (QString name)
{
  ed1->setText (name);
}

void dlgEditProfile::setServer (QString server)
{
  ed2->setText (server);
}

void dlgEditProfile::setPort (int port)
{
  ed3->setText (QString::number (port));
}

void dlgEditProfile::setLogin (QString login)
{
  ed4->setText (login);
}

void dlgEditProfile::setPassword (QString password)
{
  ed5->setText (password);
}

void dlgEditProfile::setConnectionString (QStringList conn)
{
  connstr->clear ();
  QStringList::iterator it;
  for (it = conn.begin(); it != conn.end(); ++it)
    if (it != conn.fromLast())
      connstr->insertParagraph (*it, -1);
    else
      //insertParagraph appends a newline character; this is not acceptable
      //for last call, as we keep getting more and more lines on every
      //dialog invocation. So we call method append() for last line.
      //Note that using append everywhere won't help, as we would have to
      //add \n char for all params except last
      connstr->append (*it);
}

void dlgEditProfile::slotOk ()
{
  QString s = name ().simplifyWhiteSpace ();
  bool failed = false;
  if (s == "")
  {
    failed = true;
    KMessageBox::error (this, i18n ("Profile name cannot be empty."), i18n ("Edit profile"));
  }
  //looks like filenames can contain '/' char, but who knows?
  if (s.contains ('/'))
  {
    KMessageBox::error (this, i18n ("Profile name cannot contain '/' character."), i18n ("Edit profile"));
    failed = true;
  }
    
  //if name is okay, accept() the dialog...
  if (!failed)
  {
    accept ();
    emit okClicked ();
  }
}
