/***************************************************************************
                          dlgeditscript.cpp  -  Edit Script dialog
                             -------------------
    begin                : Ne dec 15 2002
    copyright            : (C) 2002-2003 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../dialogs/dlgeditscript.h"

#include "../cconnection.h"
#include "../cconnprefs.h"
#include "../cscript.h"
#include "../cscriptlist.h"
#include "../csession.h"

#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qwhatsthis.h>
#include <kdirselectdialog.h>
#include <kfiledialog.h>
#include <klineedit.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <krestrictedline.h>

dlgEditScript::dlgEditScript (cScript *s, cSession *_sess, QWidget *parent, const char *name) :
    KDialogBase (parent, name, true, i18n ("Edit script information"), Ok|Cancel, Ok)
{
  script = s;
  sess = _sess;
  
  //initial dialog size
  setInitialSize (QSize (400, 450));

  //create main dialog's widget
  QWidget *page = new QWidget (this);
  QGridLayout *layout = new QGridLayout (page, 12, 3);

  setMainWidget (page);

  //name
  label[0] = new QLabel (i18n ("Script &name"), page);
  sname = new KRestrictedLine (page);
  //name cannot contain spaces and similar stuff, so that you can use it
  //to launch the script
  sname->setValidChars ("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_");
  label[0]->setBuddy (sname);
  QWhatsThis::add (sname, i18n ("Name of this script."));
  sname->setFocus();
  
  //comment
  label[1] = new QLabel (i18n ("Co&mment"), page);
  comment = new KLineEdit (page);
  label[1]->setBuddy (comment);
  QWhatsThis::add (sname, i18n ("Your comment for this script. Write any text here."));

  //command
  label[2] = new QLabel (i18n ("&Command to execute"), page);
  cmd = new KLineEdit (page);
  label[2]->setBuddy (cmd);
  QPushButton *cmdbutton = new QPushButton (i18n ("Browse..."), page);
  QWhatsThis::add (cmd, i18n ("Command to execute, including parameters. "
      "Note that you can specify more parameters when executing the command "
      "from the input line, if you enable the Allow parameters checkbox."));

  //workdir
  label[3] = new QLabel (i18n ("&Working directory"), page);
  workdir = new KLineEdit (page);
  label[3]->setBuddy (workdir);
  QPushButton *wrkbutton = new QPushButton (i18n ("Browse..."), page);
  QWhatsThis::add (workdir, i18n ("Working directory for the script. Script "
      "will behave exactly as if it was run from this directory in console.\n\n"
      "If your script doesn't work with any files, then you don't need to modify this."));

  //options groupbox
  QButtonGroup *options = new QButtonGroup (2, Horizontal, i18n ("&Options"), page);

  //shell-expansion
  shellexpand = new QCheckBox (i18n ("Enable shell expansion"), options);
  QWhatsThis::add (shellexpand, i18n ("Enables shell expansion of parameters. "
      "Things like wildcards and so..."));
  //adv.communication
  useadvcomm = new QCheckBox (i18n ("Enable communication"), options);
  QWhatsThis::add (useadvcomm, i18n ("Enables identification of lines of "
      "server output. Refer to the manual for more information."));
  //allow params
  allowparams = new QCheckBox (i18n ("Allow custom parameters"), options);
  QWhatsThis::add (allowparams, i18n ("Custom parameters can be specified "
      "on the input line."));
  //single-instance
  singleinstance = new QCheckBox (i18n ("Single-instance script"), options);
  QWhatsThis::add (singleinstance, i18n ("If an instance of this script "
      "is already running, another one can not be started."));
  //preprocessor script
  chkpreprocessor = new QCheckBox (i18n ("Preprocessor script"), options);
  QWhatsThis::add (chkpreprocessor, i18n ("Turns this script into a "
      "preprocessor script. This script receives all the input from the MUD "
      "before most other processing - only MCCP/MSP and telnet sequences "
      "are removed. Text returned by this script will then be used instead "
      "of the received text. Note that only one preprocessor script can be "
      "running at a time, and that most other options don't take effect "
      "for a preprocessor script."));
  
  //options groupbox
  QButtonGroup *io = new QButtonGroup (2, Horizontal, i18n ("&Input/Output"), page);

  //enable input
  enableinput = new QCheckBox (i18n ("Enable script input"), io);
  QWhatsThis::add (enableinput, i18n ("Enables sending server output to "
      "the script."));
  //send user commands
  sendusercommands = new QCheckBox (i18n ("Send user commands"), io);
  QWhatsThis::add (sendusercommands, i18n ("If script input is enabled, "
      "this enables sending of all sent commands to the script."));
  //enable output
  enablestdout = new QCheckBox (i18n ("Enable script output"), io);
  QWhatsThis::add (enablestdout, i18n ("Enables identification of lines of "
      "server output. Refer to the manual for more information."));
  //send output
  sendstdout = new QCheckBox (i18n ("Send script output"), io);
  QWhatsThis::add (sendstdout, i18n ("Send script output to the MUD server. "
      "If disabled, output will only be displayed but not sent."));
  //enable error
  enablestderr = new QCheckBox (i18n ("Enable error output"), io);
  QWhatsThis::add (enablestderr, i18n ("If enabled, server output will also "
      "include error output."));
  //send error
  sendstderr = new QCheckBox (i18n ("Send error output"), io);
  QWhatsThis::add (sendstderr, i18n ("If enabled, error output will "
      "also be sent to the MUD server."));
  //no flow control
  chknoflowcontrol = new QCheckBox (i18n ("No flow control"), io);
  QWhatsThis::add (chknoflowcontrol, i18n ("<p>If enabled, flow control will "
      "<b>NOT</b> be used for this script. Flow control ensures, that scripts "
      "are kept in sync, that means, that if two lines arrive from the "
      "server, the second one is not sent to the scripts until all running "
      "scripts have received the first one. However, in some situations, "
      "it's better not to use this feature.</p><p>If you don't need to "
      "disable flow control for some reason, leave this unchecked.</p>"));
  //allow variables
  chkallowvars = new QCheckBox (i18n ("Communicate variables"), io);
  QWhatsThis::add (chknoflowcontrol, i18n ("If enabled, a UNIX domain "
      "socket will be created, allowing you to obtain and modify actual "
      "variable values. More information in the documentation."));
  
  //output prefix
  label[4] = new QLabel (i18n ("Output pre&fix"), page);
  prefix = new KLineEdit (page);
  label[4]->setBuddy (prefix);
  QWhatsThis::add (prefix, i18n ("When sending script output to the server, "
      "this prefix will be prepended before each line."));
  //output suffix
  label[5] = new QLabel (i18n ("Output &suffix"), page);
  suffix = new KLineEdit (page);
  label[5]->setBuddy (suffix);
  QWhatsThis::add (suffix, i18n ("When sending script output to the server, "
      "this suffix will be appended to each line."));
  
  label[6] = new QLabel (i18n ("Matching &text"), page);
  matchtext = new KLineEdit (page);
  label[6]->setBuddy (matchtext);
  QWhatsThis::add (matchtext, i18n ("Matching text. Text will only be sent to "
      "the script if it passes the test (or if it doesn't pass, see Positive "
      "matching checkbox)."));

  //comparison type
  label[7] = new QLabel ("Com&parison type", page);
  type = new QComboBox (page);
  label[7]->setBuddy (type);
  type->clear ();
  type->insertItem (i18n ("Exact match"));
  type->insertItem (i18n ("Sub-string"));
  type->insertItem (i18n ("Begins with"));
  type->insertItem (i18n ("Ends with"));
  type->insertItem (i18n ("Regular expression"));
  QWhatsThis::add (type, i18n ("Type of matching. Text will only be sent to "
      "the script if it passes the test."));

  //positive matching
  onlyifmatch = new QCheckBox (i18n ("Positi&ve matching"), page);
  QWhatsThis::add (onlyifmatch, i18n ("If this option is checked, text is "
    "only sent to the script if it matches the test. Otherwise, it's sent "
    "if it doesn't match the text.<br><br>Note that this option only takes "
    "effect if you enter some matching text."));

  //activate some of enabling/disabling...
  connect (enableinput, SIGNAL (toggled (bool)), sendusercommands, SLOT (setEnabled (bool)));
  connect (enableinput, SIGNAL (toggled (bool)), useadvcomm, SLOT (setEnabled (bool)));
  connect (enablestdout, SIGNAL (toggled (bool)), sendstdout, SLOT (setEnabled (bool)));
  connect (enablestderr, SIGNAL (toggled (bool)), sendstderr, SLOT (setEnabled (bool)));

  //activate Browse... buttons
  connect (cmdbutton, SIGNAL (clicked ()), this, SLOT (browse1()));
  connect (wrkbutton, SIGNAL (clicked ()), this, SLOT (browse2()));
    
  //place'em there!
  layout->setSpacing (5);
  layout->addWidget (label[0], 0, 0);
  layout->addMultiCellWidget (sname, 0, 0, 1, 2);
  layout->addWidget (label[1], 1, 0);
  layout->addMultiCellWidget (comment, 1, 1, 1, 2);
  layout->addWidget (label[2], 2, 0);
  layout->addWidget (cmd, 2, 1);
  layout->addWidget (cmdbutton, 2, 2);
  layout->addWidget (label[3], 3, 0);
  layout->addWidget (workdir, 3, 1);
  layout->addWidget (wrkbutton, 3, 2);
  layout->addMultiCellWidget (options, 4, 4, 0, 2);
  layout->addMultiCellWidget (io, 5, 5, 0, 2);
  layout->addWidget (label[4], 6, 0);
  layout->addMultiCellWidget (prefix, 6, 6, 1, 2);
  layout->addWidget (label[5], 7, 0);
  layout->addMultiCellWidget (suffix, 7, 7, 1, 2);
  layout->addWidget (label[6], 8, 0);
  layout->addMultiCellWidget (matchtext, 8, 8, 1, 2);
  layout->addWidget (label[7], 9, 0);
  layout->addMultiCellWidget (type, 9, 9, 1, 2);
  layout->addMultiCellWidget (onlyifmatch, 10, 10, 0, 2);
  layout->setRowStretch (11, 10);
}

dlgEditScript::~dlgEditScript ()
{

}

bool dlgEditScript::showMe ()
{
  //fill the dialog
  sname->setText (script->getName());
  comment->setText (script->getComment());
  cmd->setText (script->getCommand());
  workdir->setText (script->getWorkDir());
  prefix->setText (script->getPrefix());
  suffix->setText (script->getSuffix());
  
  enableinput->setChecked (script->getEnableInput());
  enablestdout->setChecked (script->getEnableOutput());
  sendstdout->setChecked (script->getSendOutput());
  sendstdout->setEnabled (script->getEnableOutput());
  enablestderr->setChecked (script->getEnableError());
  sendstderr->setChecked (script->getSendError());
  sendstderr->setEnabled (script->getEnableError());
  sendusercommands->setChecked (script->getSendUserCommands());
  sendusercommands->setEnabled (script->getEnableInput());
  chknoflowcontrol->setChecked (script->getNoFlowControl());
  chkallowvars->setChecked (script->getAllowVars());
  
  shellexpand->setChecked (script->getShellExpansion());
  useadvcomm->setChecked (script->getUseAdvComm());
  useadvcomm->setEnabled (script->getEnableInput());
  allowparams->setChecked (script->getAllowParams());
  singleinstance->setChecked (script->getSingleInstance());
  chkpreprocessor->setChecked (script->isPreprocessor());
  
  matchtext->setText (script->getText());
  type->setCurrentItem (script->getType());
  onlyifmatch->setChecked (script->getOnlyIfMatch());

  //show the dialog
  if (exec ())
  {
    //accept changes, if needed
    script->setName (sname->text());
    script->setComment (comment->text());
    script->setCommand(cmd->text());
    script->setWorkDir(workdir->text());
    script->setPrefix(prefix->text());
    script->setSuffix(suffix->text());

    script->setEnableInput(enableinput->isChecked());
    script->setEnableOutput(enablestdout->isChecked());
    script->setSendOutput(sendstdout->isChecked());
    script->setEnableError(enablestderr->isChecked());
    script->setSendError(sendstderr->isChecked());
    script->setSendUserCommands(sendusercommands->isChecked());
    script->setNoFlowControl(chknoflowcontrol->isChecked());
    script->setAllowVars(chkallowvars->isChecked ());
    
    script->setShellExpansion(shellexpand->isChecked());
    script->setUseAdvComm(useadvcomm->isChecked());
    script->setAllowParams(allowparams->isChecked());
    script->setSingleInstance(singleinstance->isChecked());
    script->setIsPreprocessor(chkpreprocessor->isChecked());
    
    script->setText(matchtext->text());
    script->setType(type->currentItem());
    script->setOnlyIfMatch(onlyifmatch->isChecked());
    return true;
  }
  return false;
}

void dlgEditScript::browse1 ()
{
  //open some executable and place its name to the edit box
  QString fName = KFileDialog::getOpenFileName
        (sess->connection()->getConnPrefs()->scriptDir(),
        "application/x-executable", this, i18n ("Choose script"));
  if (!(fName.isEmpty()))
    cmd->setText (fName);
}

void dlgEditScript::browse2 ()
{
  QString fName = KFileDialog::getExistingDirectory (workdir->text(),
        this, i18n ("Choose working directory"));
  workdir->setText (fName);
}

void dlgEditScript::slotOk ()
{
  //look if scriptname has been filled
  QString s, s2;
  s = sname->text();
  //no whitespace here...
  if (s.length() == 0)
  {
    KMessageBox::sorry (this, i18n ("You have to specify script name."));
    cmd->setFocus ();
    return;
  }
  //look if scriptname is unique
  if (s != script->getName())   //old scriptname is always allowed
    if (sess->connection()->getConnPrefs()->scriptList()->nameExists (s))
    {
      KMessageBox::sorry (this, i18n ("Sorry, but a script with this name already exists."));
      return;
    }
    
  //look if script file has been filled
  s = cmd->text();
  s = s.stripWhiteSpace ();
  if (s.length() == 0)
  {
    KMessageBox::sorry (this, i18n ("You have to specify an executable file."));
    cmd->setFocus ();
    return;
  }
  //OK, we can accept!
  QDialog::accept ();
}
