/***************************************************************************
                          dlgeditshortcut.cpp  -  Edit macro key
                             -------------------
    begin                : Pi mj 30 2003
    copyright            : (C) 2003 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../dialogs/dlgeditshortcut.h"

#include "../dialogs/dlggrabkey.h"

#include "../cshortcut.h"
#include "../kmuddy.h"

#include <qcheckbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qwhatsthis.h>

#include <klineedit.h>
#include <klocale.h>
#include <kpushbutton.h>

extern KMuddy *kmuddy;

dlgEditShortcut::dlgEditShortcut (cShortcut *s, QWidget *parent, const char *name) :
    KDialogBase (parent, name, true, i18n ("Edit macro key"), Ok|Cancel, Ok)
{
  shortcut = s;

  createDialog ();
}

dlgEditShortcut::~dlgEditShortcut ()
{

}

void dlgEditShortcut::createDialog ()
{
  //initial dialog size
  setInitialSize (QSize (300, 150));

  //create main dialog's widget
  QWidget *page = new QWidget (this);
  QGridLayout *layout = new QGridLayout (page, 5, 3);

  setMainWidget (page);

  //command
  QLabel *lbl1 = new QLabel (i18n ("&Command"), page);
  cmd = new KLineEdit (page);
  lbl1->setBuddy (cmd);
  QWhatsThis::add (cmd, i18n ("Command that will be executed when you press the"
      "defined key combination.\n"
      "Command can include aliases, script calls and similar stuff."));

  //key
  QLabel *lbl2 = new QLabel (i18n ("Shortcut"), page);
  lblkey = new QLabel (page);
  lblkey->setFrameStyle (QFrame::StyledPanel | QFrame::Sunken);
  lblkey->setLineWidth (2);
  btgrab = new KPushButton (i18n ("Grab shortcut..."), page);
  connect (btgrab, SIGNAL (clicked ()), this, SLOT (grabKey ()));

  //checkboxes
  chksendit = new QCheckBox (i18n ("&Send command"), page);
  QWhatsThis::add (chksendit, i18n ("If checked, the command will be sent "
      "to the MUD. Otherwise, it will be put in the inputline."));
  chkoverwrite = new QCheckBox (i18n ("&Overwrite existing text"), page);
  QWhatsThis::add (chkoverwrite, i18n ("If there already is some text in "
      "the inputline, should it be overwritten?"));
  connect (chksendit, SIGNAL (toggled (bool)), chkoverwrite, SLOT (setDisabled (bool)));
      
  cmd->setFocus ();
  
  //place'em there!
  layout->setSpacing (5);
  layout->addWidget (lbl1, 0, 0);
  layout->addMultiCellWidget (cmd, 0, 0, 1, 2);
  layout->addWidget (lbl2, 1, 0);
  layout->addWidget (lblkey, 1, 1);
  layout->addWidget (btgrab, 1, 2);
  layout->addMultiCellWidget (chksendit, 2, 2, 0, 1);
  layout->addMultiCellWidget (chkoverwrite, 3, 3, 0, 1);
  layout->setRowStretch (4, 10);
}

bool dlgEditShortcut::showMe ()
//fills and shows the dialog, then applies changes if needed
{
  //fill in the dialog
  cmd->setText (shortcut->getText ());
  key = shortcut->key();
  state = shortcut->state();  
  lblkey->setText (cShortcut::keyToString (key, state));
  chksendit->setChecked (shortcut->sendIt());
  chkoverwrite->setChecked (shortcut->overwriteInput());
  chkoverwrite->setDisabled (chksendit->isChecked ());
  
  //show it!
  int n = exec ();

  //update structure if needed
  if (n == Accepted)
  {
    shortcut->setText (cmd->text ());
    shortcut->setKey (key);
    shortcut->setState (state);
    shortcut->setSendIt (chksendit->isChecked ());
    shortcut->setOverwriteInput(chkoverwrite->isChecked ());
    return true;
  }
  else
    return false;
}

void dlgEditShortcut::grabKey ()
{
  dlgGrabKey *gk = new dlgGrabKey (this);
  if (gk->exec ())
  {
    key = gk->key();
    state = gk->state();
    state = state & (Qt::KeyButtonMask | Qt::Keypad);
    lblkey->setText (cShortcut::keyToString (key, state));
  }
  delete gk;
  cmd->setFocus ();
}
