/***************************************************************************
                          dlgeditvariable.cpp  -  Edit Variable dialog
                             -------------------
    begin                : St sep 10 2003
    copyright            : (C) 2003 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../dialogs/dlgeditvariable.h"

#include <qlabel.h>
#include <qlayout.h>
#include <qwhatsthis.h>

#include <klineedit.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <krestrictedline.h>

dlgEditVariable::dlgEditVariable (QWidget *parent, const char *name) :
    KDialogBase (parent, name, true, i18n ("Edit variable"), Ok|Cancel, Ok)
{
  createDialog ();
}

dlgEditVariable::~dlgEditVariable ()
{

}

void dlgEditVariable::createDialog ()
{
  //initial dialog size
  setInitialSize (QSize (300, 150));

  //create main dialog's widget
  QWidget *page = new QWidget (this);
  QGridLayout *layout = new QGridLayout (page, 3, 2);

  setMainWidget (page);

  //command
  lbl1 = new QLabel (i18n ("&Name"), page);
  edname = new KRestrictedLine (page);
  edname->setValidChars ("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_");
  lbl1->setBuddy (edname);
  QWhatsThis::add (edname, i18n ("Name of the variable."));

  //interval
  QLabel *lbl2 = new QLabel (i18n ("&Value"), page);
  edvalue = new KLineEdit (page);
  lbl2->setBuddy (edvalue);
  QWhatsThis::add (edvalue, i18n ("Value of this variable. Note that this "
      "can also be set using the /set command."));

  //place'em there!
  layout->setSpacing (5);
  layout->addWidget (lbl1, 0, 0);
  layout->addWidget (edname, 0, 1);
  layout->addWidget (lbl2, 1, 0);
  layout->addWidget (edvalue, 1, 1);
  layout->setRowStretch (2, 10);
}

bool dlgEditVariable::showMe ()
//fills and shows the dialog, then applies changes if needed
{
  //focus
  if (edname->isEnabled ())
    edname->setFocus ();
  else
    edvalue->setFocus ();
  
  //show it!
  int n = exec ();

  if (n == Accepted)
    return true;
  return false;
}

void dlgEditVariable::setName (const QString &n)
{
  edname->setText (n);
}

QString dlgEditVariable::getName ()
{
  return edname->text ();
}

void dlgEditVariable::setValue (const QString &v)
{
  edvalue->setText (v);
}

QString dlgEditVariable::getValue ()
{
  return edvalue->text ();
}

void dlgEditVariable::setModifyMode ()
{
  lbl1->setEnabled (false);
  edname->setEnabled (false);
}

void dlgEditVariable::slotOk ()
{
  //look if variable name has been filled
  QString n;
  n = edname->text();
  //no whitespace here...
  if (n.length() == 0)
  {
    KMessageBox::sorry (this, i18n ("You have to specify variable name."));
    edname->setFocus ();
    return;
  }
  
  //duplicity check is in dlgVariables

  //OK, we can accept!
  QDialog::accept ();
}
