//
// C++ Implementation: dlgeditvartrig
//
// Description:
//
//
// Author: Tomas Mecir <kmuddy@kmuddy.org>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "dlgeditvartrig.h"

#include "../cvartrigger.h"

#include <klocale.h>
#include <krestrictedline.h>
#include <ktextedit.h>

#include <qlabel.h>
#include <qlayout.h>
#include <qwhatsthis.h>

dlgEditVarTrig::dlgEditVarTrig (cVarTrigger *vt, QWidget *parent, const char *name) :
    KDialogBase (parent, name, true, i18n ("Edit variable trigger"), Ok|Cancel, Ok)
{
  vartrig = vt;

  createDialog ();
}


dlgEditVarTrig::~dlgEditVarTrig ()
{

}

void dlgEditVarTrig::createDialog ()
{
  //initial dialog size
  setInitialSize (QSize (300, 150));

  //create main dialog's widget
  QWidget *page = new QWidget (this);
  QGridLayout *layout = new QGridLayout (page, 3, 2);

  setMainWidget (page);

  //widgets, row 1
  QLabel *lbl1 = new QLabel (i18n ("&Variable"), page);
  variable = new KRestrictedLine (page);
  variable->setValidChars ("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_");
  lbl1->setBuddy (variable);
  QWhatsThis::add (variable, i18n ("Name of the variable, that this trigger will be reacting on."));

  //widgets, row 2
  QLabel *lbl2 = new QLabel (i18n ("&Commands"), page);
  cmds = new KTextEdit (page);
  lbl2->setBuddy (cmds);
  QWhatsThis::add (cmds, i18n ("Commands that will be executed when this trigger matches."));

  //fill in the layout
  layout->setSpacing (5);
  layout->addWidget (lbl1, 0, 0);
  layout->addWidget (variable, 0, 1);
  layout->addWidget (lbl2, 1, 0);
  layout->addWidget (cmds, 1, 1);
}

bool dlgEditVarTrig::showMe ()
{
  //variable name
  variable->setText (vartrig->varName());

  //commands to exec
  //taken from dlgEditAliasTrigger :)
  QStringList newtext = vartrig->getNewText ();
  cmds->clear ();
  QStringList::iterator it;
  for (it = newtext.begin(); it != newtext.end(); ++it)
    if (it != newtext.fromLast())
      cmds->insertParagraph (*it, -1);
    else
      cmds->append (*it);

  //show it!
  int n = exec ();

  //update structure if needed
  if (n == Accepted)
  {
    vartrig->setVarName (variable->text());

    //taken from dlgEditAliasTrigger
    QStringList ntext;
    int pars = cmds->paragraphs ();
    for (int i = 0; i < pars; i++)
      ntext << (cmds->text(i).stripWhiteSpace());
    vartrig->setNewText (ntext);
    return true;
  }
  return false;
}

#include "dlgeditvartrig.moc"
