/***************************************************************************
                          dlgimportexportprofile.cpp  -  description
                             -------------------
    begin                : Ne jl 20 2003
    copyright            : (C) 2003 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../dialogs/dlgimportexportprofile.h"

#include <kfiledialog.h>
#include <klineedit.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kpushbutton.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlayout.h>

#include "../cprofiles.h"

dlgImportExportProfile::dlgImportExportProfile (bool isImport,
    QWidget *parent, const char *name ) :
    KDialogBase (parent, name, true,
    isImport ? i18n ("Import profile") : i18n ("Export profile"),
    Ok|Cancel, Ok)
{
  import = isImport;
  
  //initial dialog size
  setInitialSize (QSize (300, 150));

  //create main dialog's widget
  QWidget *page = new QWidget (this);
  QGridLayout *layout = new QGridLayout (page, 3, 3);

  setMainWidget (page);

  QLabel *l1 = new QLabel (i18n ("&Profile name"), page);
  if (isImport)
  {
    edprofile = new KLineEdit (page);
    l1->setBuddy (edprofile);
  }
  else
  {
    cbprofile = new QComboBox (page);
    cProfiles p;
    cbprofile->insertStringList (p.getprofiles());
    l1->setBuddy (cbprofile);
  }
  QLabel *l2 = new QLabel (isImport ? i18n ("&Import from") :
      i18n ("&Export to"), page);
  edfname = new KLineEdit (page);
  l2->setBuddy (edfname);
  KPushButton *button = new KPushButton (i18n ("Browse..."), page);
  layout->setSpacing (10);
  layout->addWidget (l1, 0, 0);
  layout->addWidget (l2, 1, 0);
  if (isImport)
    layout->addWidget (edprofile, 0, 1);
  else
    layout->addWidget (cbprofile, 0, 1);
  layout->addWidget (edfname, 1, 1);
  layout->addWidget (button, 1, 2);
  layout->setRowStretch (2, 10);

  connect (button, SIGNAL (clicked()), this, SLOT (browse ()));
}                                                                       

dlgImportExportProfile::~dlgImportExportProfile ()
{

}

void dlgImportExportProfile::doThings ()
{
  if (exec() == QDialog::Accepted)
  {
    QString fName = edfname->text();
    QString pName;
    if (import)
      pName = edprofile->text ();
    else
      pName = cbprofile->currentText ();
    QString msg;
    cProfiles p;
    if (import)
    {
      if (p.importProfile (pName, fName))
        msg = QString::null;
      else
        msg = p.lastError();
    }
    else
    {
      if (p.exportProfile (pName, fName))
        msg = QString::null;
      else
        msg = p.lastError();
    }
    if (msg == QString::null)
      KMessageBox::information (this, import ?
        i18n ("Profile has been successfully imported.") :
        i18n ("Profile has been successfully exported."));
    else
      KMessageBox::sorry (this, msg);
  }
}

void dlgImportExportProfile::browse ()
{
  if (import)
  {
    QString n = KFileDialog::getOpenFileName (QString::null,
        QString::null, this, i18n ("Choose file with profile"));
    if (!(n.isEmpty()))
      edfname->setText (n);
  }
  else
  {
    QString n = KFileDialog::getSaveFileName (QString::null,
        QString::null, this, i18n ("Enter target file"));
    if (!(n.isEmpty()))
      edfname->setText (n);
  }
}
