/***************************************************************************
                          dlgmultiline.cpp  -  multi-line input dialog
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Po Aug 26 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../dialogs/dlgmultiline.h"

#include <qlabel.h>
#include <qlayout.h>
#include <qtextedit.h>
#include <qwhatsthis.h>
#include <klineedit.h>
#include <klocale.h>

dlgMultiLine::dlgMultiLine (QWidget *parent, const char *name) :
    KDialogBase (parent, name, false, i18n ("Multi-line input"),
        Apply|Cancel|User1, Default)
{
  //initial size
  setInitialSize (QSize (400, 250));

  //main widget
  QWidget *page = new QWidget (this);
  QVBoxLayout *layout = new QVBoxLayout (page);

  setMainWidget (page);

  setFocusPolicy (QWidget::StrongFocus);
  
  //editor
  editor = new QTextEdit (page);
  editor->setWordWrap (QTextEdit::NoWrap);
  QWhatsThis::add (editor, i18n ("Enter commands that are to be sent to the MUD. "
  "Each line represents one command."));
  editor->setFocus ();
    
  layout->setSpacing (5);
  layout->addWidget (editor);

  //prefix/suffix
  QHBoxLayout *pslayout = new QHBoxLayout (layout);
  QLabel *l1 = new QLabel (i18n ("&Prefix:"), page);
  prefix = new KLineEdit (page);
  l1->setBuddy (prefix);
  QLabel *l2 = new QLabel (i18n ("&Suffix:"), page);
  suffix = new KLineEdit (page);
  l2->setBuddy (suffix);
  pslayout->setSpacing (5);
  pslayout->addWidget (l1);
  pslayout->addWidget (prefix);
  pslayout->addSpacing (5);
  pslayout->addWidget (l2);
  pslayout->addWidget (suffix);

  QString hlptext = i18n ("Prefix and suffix may contain text, that will "
      "be added to the beginning (prefix) or ending (suffix) of every "
      "line written to the multi-line editor.");
  
  QWhatsThis::add (prefix, hlptext);
  QWhatsThis::add (suffix, hlptext);
  
  //button texts
  setButtonApplyText (i18n ("&Send"), i18n ("Sends current text to the MUD."));
  setButtonCancelText (i18n ("&Close"), i18n ("Closes this window."));
  setButtonText (User1, i18n ("C&lear"));
  setButtonTip (User1, i18n ("Clears window contents."));
  setButtonWhatsThis (Apply, i18n ("This will send all the commands you've "
    "entered to the MUD. If you've entered prefix and/or suffix, they will be added "
    "to the beginning and ending of each command."));
}

dlgMultiLine::~dlgMultiLine ()
{

}

void dlgMultiLine::setFont (const QFont &font)
{
  editor->setFont (font);
}

void dlgMultiLine::clear ()
{
  editor->clear ();
}

int dlgMultiLine::lines ()
{
  return editor->lines ();
}

QString dlgMultiLine::line (int index)
{
  //this constructs a line, including prefix/suffix, and returns it...
  QString txt;
  QString s1, s2, s3;
  //prefix
  s1 = prefix->text ();
  s2 = editor->text (index);
  s3 = suffix->text ();
  if (s1.length() > 0)
  {
    txt += s1;
    if (s1[s1.length() - 1].latin1() != ' ')
      txt += " ";
  }
  //command
  txt += s2;
  //suffix
  if (s3.length() > 0)
  {
    if (s3[0].latin1() != ' ')
      txt += " ";
    txt += s3;
  }
  return txt;
}
