/***************************************************************************
                          dlgprofilesettings.cpp  -  profile settings
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : P oct 18 2002
    copyright            : (C) 2002-2003 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../dialogs/dlgprofilesettings.h"

#include "cdirlist.h"

#include <qcheckbox.h>
#include <qframe.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qwhatsthis.h>
#include <kapplication.h>
#include <kfiledialog.h>
#include <kiconloader.h>
#include <klineedit.h>
#include <klocale.h>

dlgProfileSettings::dlgProfileSettings (QWidget *parent, const char *name) :
    KDialogBase (IconList, i18n ("MUD preferences"), Ok|Cancel|Apply,
    Ok, parent, name, true)
{
  //initial dialog size
  setInitialSize (QSize (450, 350));

  //prepare dialog pages
    //KIcon::Toolbar is the closest match... I hope it's okay...
  KIconLoader *icons = (KApplication::kApplication())->iconLoader ();
  QFrame *frmconn = addPage (i18n ("Connection"), QString::null,
      icons->loadIcon ("connect_established", KIcon::Toolbar, 32));
  QFrame *frmcommands = addPage (i18n ("Commands"), QString::null,
      icons->loadIcon ("gear", KIcon::Toolbar, 32));
  QFrame *frmdirs = addPage (i18n ("Directories"), QString::null,
      icons->loadIcon ("folder", KIcon::Toolbar, 32));
#ifdef HAVE_SOUND
  QFrame *frmsound = addPage (i18n ("Sound"), QString::null,
      icons->loadIcon ("multimedia", KIcon::Toolbar, 32));
#endif

  //page 1
  QVBoxLayout *connlayout = new QVBoxLayout (frmconn);
  useansi = new QCheckBox (i18n ("&Use colorized output (if available)"), frmconn);
  limittriggers = new QCheckBox (i18n ("Enable &trigger loop detection"), frmconn);
  QWhatsThis::add (limittriggers, i18n ("This option will automatically "
      "disable triggers if a possible loop is detected, that is, if 50 triggers "
      "are activated with delays less than 1 second. If you disable this and "
      "accidentally make a loop, you'll have to disable triggers on "
      "yourself to get out of the loop."));
  limitrepeater = new QCheckBox (i18n ("&Do not allow more than 100 repeated commands"), frmconn);
  QWhatsThis::add (limitrepeater, "<qt>With this option turned on, usage of repeater string "
      "is limited to one hundred repeats. This is here to prevent you from sending "
      "lots of commands accidentally. Note that you can always cancel those "
      "unwanted commands with <b>Edit / Cancel pending commands</b>.</qt>");
  
  // LPMud prompt handling
  lpmudstyle = new QCheckBox (i18n ("&Auto-append newlines after prompt"), frmconn);
  QWhatsThis::add (lpmudstyle, i18n ("Enables adding linefeeds afer GA-terminated prompts "
				     "to have properly aligned output."));
  
  promptlabel = new QCheckBox (i18n ("&Show prompt in a separate field"), frmconn);
  QWhatsThis::add (promptlabel, "<qt>You will have prompt displayed in a special field "
		   "on the left from your primary input line.</qt>");
  
  statusprompt = new QCheckBox (i18n ("Show prompt in the s&tatus bar"), frmconn);
  QWhatsThis::add (statusprompt, "<qt>Enables prompt display in the status bar.</qt>");
  
  consoleprompt = new QCheckBox (i18n ("Show prompt in the &console"), frmconn);
  QWhatsThis::add (consoleprompt, "<qt>Enables prompt display in the console.</qt>");
  
  autoadvtranscript = new QCheckBox (i18n ("Start advanced transcript when session starts"), frmconn);
  QWhatsThis::add (autoadvtranscript, "<qt>Starts advanced logging of the session upon session "
		   "startup</qt>");
  
  connlayout->setSpacing (10);
  connlayout->addWidget (useansi);
  connlayout->addWidget (limittriggers);
  connlayout->addWidget (limitrepeater);
  connlayout->addWidget (lpmudstyle);
  connlayout->addWidget (promptlabel);
  connlayout->addWidget (statusprompt);
  connlayout->addWidget (consoleprompt);
  connlayout->addWidget (autoadvtranscript);
  connlayout->addStretch (10);

  //page 2
  QGridLayout *commlayout = new QGridLayout (frmcommands, 12, 2);

  QLabel *cl[10];
  QString texts[10] = { i18n ("&North"), i18n ("No&rtheast"), i18n ("&East"),
        i18n ("So&utheast"), i18n ("&South"), i18n ("Sou&thwest"),
        i18n ("&West"), i18n ("Nort&hwest"), i18n ("&Up"), i18n ("&Down")};
  QString mcommandhelp = i18n ("This is a movement command. It is used"
        " by speed-walking and will also be used by the auto-mapper.");
  QString qcommandhelp = i18n ("This command will be automatically sent "
        "if you disconnect in some other way than entering the quit command. "
        "This prevents the net-dead (also called link-dead) state "
        "(provided that it works...).");
  for (int i = 0; i < 10; i++)
  {
    cl[i] = new QLabel (texts[i], frmcommands);
    edcmd[i] = new KLineEdit ("", frmcommands);
    cl[i]->setBuddy (edcmd[i]);
    QWhatsThis::add (edcmd[i], mcommandhelp);
  }
  QLabel *ql = new QLabel (i18n ("&Quit"), frmcommands);
  edquit = new KLineEdit ("", frmcommands);
  ql->setBuddy (edquit);
  QWhatsThis::add (edquit, qcommandhelp);

  commlayout->setColStretch (1, 10);
  commlayout->setSpacing (5);
  for (int i = 0; i < 10; i++)
  {
    commlayout->addWidget (cl[i], i, 0);
    commlayout->addWidget (edcmd[i], i, 1);
  }
  commlayout->addWidget (ql, 10, 0);
  commlayout->addWidget (edquit, 10, 1);
  commlayout->setRowStretch (11, 10);

  //page 3
  QGridLayout *dirslayout = new QGridLayout (frmdirs, 4, 3);

  QLabel *sl1 = new QLabel (i18n ("Default &script location"), frmdirs);
  edscriptdir = new KLineEdit (frmdirs);
  sl1->setBuddy (edscriptdir);
  QPushButton *locbutton = new QPushButton (i18n ("Browse..."), frmdirs);
  QWhatsThis::add (edscriptdir, i18n ("Default directory where we'll look for "
      "scripts."));

  QLabel *sl2 = new QLabel (i18n ("Default &working directory"), frmdirs);
  edscriptworkdir = new KLineEdit (frmdirs);
  sl2->setBuddy (edscriptworkdir);
  QPushButton *wrkbutton = new QPushButton (i18n ("Browse..."), frmdirs);
  QWhatsThis::add (edscriptworkdir, i18n ("Default working directory for your "
      "scripts. Making this the same as above is usually a good choice."));

  QLabel *sl3 = new QLabel (i18n ("Default &transcript directory"), frmdirs);
  edtranscriptdir = new KLineEdit (frmdirs);
  sl3->setBuddy (edtranscriptdir);
  QPushButton *trnbutton = new QPushButton (i18n ("Browse..."), frmdirs);
  QWhatsThis::add (edtranscriptdir, i18n ("Directory where session "
        "transcripts will be saved by default."));

  connect (locbutton, SIGNAL (clicked()), this, SLOT (choosedir1()));
  connect (wrkbutton, SIGNAL (clicked()), this, SLOT (choosedir2()));
  connect (trnbutton, SIGNAL (clicked()), this, SLOT (choosedir3()));
  
  dirslayout->setSpacing (5);
  dirslayout->addWidget (sl1, 0, 0);
  dirslayout->addWidget (edscriptdir, 0, 1);
  dirslayout->addWidget (locbutton, 0, 2);
  dirslayout->addWidget (sl2, 1, 0);
  dirslayout->addWidget (edscriptworkdir, 1, 1);
  dirslayout->addWidget (wrkbutton, 1, 2);
  dirslayout->addWidget (sl3, 2, 0);
  dirslayout->addWidget (edtranscriptdir, 2, 1);
  dirslayout->addWidget (trnbutton, 2, 2);
  dirslayout->setRowStretch (3, 10);

#ifdef HAVE_SOUND
  //page 4
  QVBoxLayout *soundlayout = new QVBoxLayout (frmsound);
  usemsp = new QCheckBox (i18n ("&Enable sounds for this profile"), frmsound);
  QWhatsThis::add (usemsp, i18n ("Enable playing sounds with this profile.\n"
      "Note that this will only wirk if you enable sounds in global preferences.\n"
      "Also note that this will only work if your MUD supports <b>MSP</b> "
      "(MUD Sound Protocol)."));
  dirlist = new cDirList (frmsound);
  QLabel *lblmspwarning = new QLabel (i18n ("Only enable these if you need them:"), frmsound);
  alwaysmsp = new QCheckBox (i18n ("&Use MSP even if not negotiated"), frmsound);
  QWhatsThis::add (alwaysmsp, i18n ("When this option is enabled, KMuddy "
      "will interpret MSP sequences all the time, even if no negotiation "
      "took place. Useful for MUDs that don't support telnet negotiation of "
      "MSP.\nBut beware - only turn this on if your MUD really supports "
      "MSP to prevent other players from sending malicious MSP sequences, "
      "if you allow downloading of sounds."));
  midlinemsp = new QCheckBox (i18n ("Allow &mid-line MSP sequences"), frmsound);
  QWhatsThis::add (midlinemsp, i18n ("Mid-line MSP sequences are a zMUD's "
      "extension to the MSP spec that some MUDs seem to be using. You can "
      "enable support for this feature here, but ensure that other players "
      "cannot say/tell/whatever these sequences, or disable downloading "
      "support; otherwise players could instruct your client to download "
      "any file on the net..."));
      
  soundlayout->setSpacing (5);
  soundlayout->addWidget (usemsp);
  soundlayout->addWidget (dirlist);
  soundlayout->addWidget (lblmspwarning);
  soundlayout->addWidget (alwaysmsp);
  soundlayout->addWidget (midlinemsp);
#endif //HAVE_SOUND
}

dlgProfileSettings::~dlgProfileSettings ()
{

}

bool dlgProfileSettings::useAnsi ()
{
  return useansi->isChecked ();
}

void dlgProfileSettings::setUseAnsi (bool val)
{
  useansi->setChecked (val);
}

bool dlgProfileSettings::limitTriggers ()
{
  return limittriggers->isChecked ();
}

void dlgProfileSettings::setLimitTriggers (bool val)
{
  limittriggers->setChecked (val);
}

bool dlgProfileSettings::limitRepeater ()
{
  return limitrepeater->isChecked ();
}

void dlgProfileSettings::setLimitRepeater (bool val)
{
  limitrepeater->setChecked (val);
}

bool dlgProfileSettings::promptLabel()
{
  return promptlabel->isChecked ();
}

void dlgProfileSettings::setPromptLabel(bool val)
{
  promptlabel->setChecked (val);
}

bool dlgProfileSettings::LPMudStyle()
{
  return lpmudstyle->isChecked();
}

void dlgProfileSettings::setLPMudStyle(bool val)
{
  lpmudstyle->setChecked(val);
}

bool dlgProfileSettings::statusPrompt()
{
  return statusprompt->isChecked();
}

void dlgProfileSettings::setStatusPrompt(bool val)
{
  statusprompt->setChecked(val);
}

bool dlgProfileSettings::consolePrompt()
{
  return consoleprompt->isChecked();
}

void dlgProfileSettings::setConsolePrompt(bool val)
{
  consoleprompt->setChecked(val);
}

bool dlgProfileSettings::autoAdvTranscript()
{
  return autoadvtranscript->isChecked();
}

void dlgProfileSettings::setAutoAdvTranscript(bool val)
{
  autoadvtranscript->setChecked(val);
}
QString dlgProfileSettings::movementCmd (int which)
{
  if ((which >= 0) && (which <= 9))
    return edcmd[which]->text ();
  else
    return "";
}

void dlgProfileSettings::setMovementCmd (int which, QString val)
{
  if ((which >= 0) && (which <= 9))
    edcmd[which]->setText (val);
}

QString dlgProfileSettings::quitCmd ()
{
  return edquit->text ();
}

void dlgProfileSettings::setQuitCmd (QString val)
{
  edquit->setText (val);
}

QString dlgProfileSettings::scriptDir ()
{
  return edscriptdir->text ();
}

void dlgProfileSettings::setScriptDir (const QString &s)
{
  edscriptdir->setText (s);
}

QString dlgProfileSettings::scriptWorkDir ()
{
  return edscriptworkdir->text ();
}

void dlgProfileSettings::setTranscriptDir (const QString &s)
{
  edtranscriptdir->setText (s);
}

QString dlgProfileSettings::transcriptDir ()
{
  return edtranscriptdir->text ();
}

void dlgProfileSettings::setScriptWorkDir (const QString &s)
{
  edscriptworkdir->setText (s);
}

void dlgProfileSettings::choosedir1 ()
{
  edscriptdir->setText (KFileDialog::getExistingDirectory (edscriptdir->text(), this, i18n ("Choose default script location")));
}

void dlgProfileSettings::choosedir2 ()
{
  edscriptworkdir->setText (KFileDialog::getExistingDirectory (edscriptworkdir->text(), this, i18n ("Choose default script working directory")));
}

void dlgProfileSettings::choosedir3 ()
{
  edtranscriptdir->setText (KFileDialog::getExistingDirectory (edtranscriptdir->text(), this, i18n ("Choose default transcript directory")));
}

#ifdef HAVE_SOUND

bool dlgProfileSettings::useMSP ()
{
  return usemsp->isChecked ();
}

void dlgProfileSettings::setUseMSP (bool val)
{
  usemsp->setChecked (val);
}

bool dlgProfileSettings::alwaysMSP ()
{
  return alwaysmsp->isChecked ();
}

void dlgProfileSettings::setAlwaysMSP (bool val)
{
  alwaysmsp->setChecked (val);
}

bool dlgProfileSettings::midlineMSP ()
{
  return midlinemsp->isChecked ();
}

void dlgProfileSettings::setMidlineMSP (bool val)
{
  midlinemsp->setChecked (val);
}

const QStringList &dlgProfileSettings::soundDirList ()
{
  return dirlist->getDirList ();
}

void dlgProfileSettings::setSoundDirList (const QStringList &list)
{
  dirlist->setDirList (list);
}

#endif  //HAVE_SOUND

