/***************************************************************************
                          dlgrunninglist.cpp  -  Running Scripts dialog
                             -------------------
    begin                : So jan 18 2003
    copyright            : (C) 2003 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../dialogs/dlgrunninglist.h"

#include "../crunninglist.h"
#include "cnumberedlistitem.h"

#include <qlabel.h>
#include <qlayout.h>
#include <qmenubar.h>
#include <qpopupmenu.h>

#include <klistview.h>
#include <klocale.h>

dlgRunningList::dlgRunningList (cRunningList *list,
    QWidget *parent, const char *name) :
    KDialogBase (parent, name, false, i18n ("Running scripts"), 0)
{
  //initial size
  setInitialSize (QSize (150, 300));

  //main widget
  QWidget *page = new QWidget (this);
  QVBoxLayout *layout = new QVBoxLayout (page);

  setMainWidget (page);

  setFocusPolicy (QWidget::StrongFocus);

  QLabel *label = new QLabel (i18n ("&Running scripts"), page);
  view = new KListView (page);
  label->setBuddy (view);
  view->addColumn (i18n ("Name"));
  view->addColumn (i18n ("Status"));
  view->setColumnWidthMode (0, QListView::Maximum);
  view->setColumnWidthMode (1, QListView::Maximum);
  
  //menubar (please don't throw things at me if you don't like menubars in dialogs)
  //doing this the old way - no KActions, no configurable shortcuts...
  QMenuBar *menubar = new QMenuBar (this);
  pop = new QPopupMenu (this);
  pop->insertItem (i18n ("&Suspend"), this, SLOT (suspendScript ()));
  pop->insertItem (i18n ("&Resume"), this, SLOT (resumeScript ()));
  pop->insertItem (i18n ("&Terminate"), this, SLOT (terminateScript ()));
  pop->insertItem (i18n ("&Kill"), this, SLOT (killScript ()));

  menubar->insertItem (i18n ("&Script"), pop, 1);

  connect( view, SIGNAL(contextMenu(KListView*, QListViewItem*, const QPoint&)), this, SLOT(showRMBMenu(KListView*, QListViewItem*, const QPoint&)) );
  
  layout->setSpacing (5);
  layout->addWidget (label);
  layout->addWidget (view);

  //now update the runninglist and the listview
  rlist = 0;
  switchRunningList (list);
}

dlgRunningList::~dlgRunningList()
{
  //nothing here
}

void dlgRunningList::switchRunningList (cRunningList *newlist)
{
  //close old signal-slot connection
  if (rlist != 0)
  {
    disconnect (rlist, SIGNAL (stateChanged()), this, SLOT (updateView()));
    disconnect (rlist, SIGNAL (destroyed()), this, SLOT (listDestroyed()));
  }
  rlist = newlist;
  //and open a new one
  if (rlist != 0)
  {
    connect (rlist, SIGNAL (stateChanged()), this, SLOT (updateView()));
    connect (rlist, SIGNAL (destroyed()), this, SLOT (listDestroyed()));
  }
  updateView ();
}

void dlgRunningList::suspendScript ()
{
  if (rlist == 0) //if there is no runninglist, there's nothing to do
    return;

  cNumberedListItem *item = (cNumberedListItem *) view->selectedItem ();
  if (item == 0)  //do nothing if there is no selected item
    return;
  int id = item->number ();

  rlist->suspend (id);
}

void dlgRunningList::resumeScript ()
{
  if (rlist == 0) //if there is no runninglist, there's nothing to do
    return;

  cNumberedListItem *item = (cNumberedListItem *) view->selectedItem ();
  if (item == 0)  //do nothing if there is no selected item
    return;
  int id = item->number ();

  rlist->resume (id);
}

void dlgRunningList::terminateScript ()
{
  if (rlist == 0) //if there is no runninglist, there's nothing to do
    return;

  cNumberedListItem *item = (cNumberedListItem *) view->selectedItem ();
  if (item == 0)  //do nothing if there is no selected item
    return;
  int id = item->number ();

  rlist->terminate (id);
}

void dlgRunningList::killScript ()
{
  if (rlist == 0) //if there is no runninglist, there's nothing to do
    return;

  cNumberedListItem *item = (cNumberedListItem *) view->selectedItem ();
  if (item == 0)  //do nothing if there is no selected item
    return;
  int id = item->number ();

  rlist->kill (id);
}

void dlgRunningList::updateView ()
{
  view->clear ();

  if (rlist == 0) //if there is no runninglist, there's nothing more do
    return;

  view->setSelectionMode (QListView::Single);
  int id = rlist->getFirstId();
  while (id != 0) {
    cNumberedListItem *item;
    //note that getRunningScript(id) called by name and getStatus is
    //optimized to handle this situation in O(1) (i.e. constant-time)
    item = new cNumberedListItem (view, rlist->name(id), rlist->getStatus(id));
    item->setNumber (id); //set group number
    id = rlist->getNextId();
  }
}

void dlgRunningList::listDestroyed ()
//we need this function to prevent crash when having 2+ profile-based
//connections and closing one that has running scripts
{
  //all signal-slot connections shall be closed automatically...
  rlist = 0;
  updateView ();
}

void dlgRunningList::showRMBMenu(KListView*, QListViewItem* item, const QPoint& point)
{
  if( item )
    pop->popup( point );
}
