/***************************************************************************
                          dlgscripts.cpp  -  Scripts dialog
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : Ne Dec 22 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* THIS CODE IS BASED ON THE dlgAliasTrigger CLASS CODE */

#include "dlgscripts.h"

#include "../dialogs/dlgeditscript.h"

#include "../cconnection.h"
#include "../cconnprefs.h"
#include "../cscript.h"
#include "../cscriptlist.h"
#include "../csession.h"

#include <qlayout.h>
#include <klocale.h>
#include <qlabel.h>
#include <kpushbutton.h>
#include <kapp.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <map>

dlgScripts::dlgScripts (cScriptList *slist, QWidget *parent, const char *name) :
    KDialogBase (parent, name, true, i18n ("Scripts"), Ok, Ok, true)
{
  //store script list
  list = slist;

  createDialog ();
}

void dlgScripts::createDialog ()
{
  //initial dialog size
  setInitialSize (QSize (500, 400));

  //create main dialog's widget
  QWidget *page = new QWidget (this);
  QGridLayout *layout = new QGridLayout (page, 2, 2);

  setMainWidget (page);

  //change button text
  setButtonOKText (i18n ("&Done"));

  //create widgets
  QLabel *label = new QLabel (i18n ("&List of scripts:"), page);
  box = new KListBox (page);
  box->setFocus();
  label->setBuddy (box);
  QFrame *buttons = new QFrame (page);
  QVBoxLayout *buttonslayout = new QVBoxLayout (buttons);

  KIconLoader *icons = (KApplication::kApplication())->iconLoader ();
  btadd = new KPushButton (i18n ("&Add..."), buttons);
  btmodify = new KPushButton (i18n ("&Modify..."), buttons);
  btdelete = new KPushButton (i18n ("Dele&te"), buttons);
  btup = new KPushButton (icons->loadIcon ("up", KIcon::Small),
        i18n ("Move &up"), buttons);
  btdown = new KPushButton (icons->loadIcon ("down", KIcon::Small),
        i18n ("Move &down"), buttons);
  btsort = new KPushButton (i18n ("&Sort..."), buttons);
  btrun = new KPushButton (icons->loadIcon ("run", KIcon::Small),
        i18n ("&Run!"), buttons);

  buttonslayout->setSpacing (5);
  buttonslayout->addWidget (btadd);
  buttonslayout->addWidget (btmodify);
  buttonslayout->addWidget (btdelete);
  buttonslayout->addStretch (1);
  buttonslayout->addWidget (btup);
  buttonslayout->addWidget (btdown);
  buttonslayout->addWidget (btsort);
  buttonslayout->addStretch (1);
  buttonslayout->addWidget (btrun);
  buttonslayout->addStretch (10);

  layout->setColStretch (0, 10);
  layout->setRowStretch (1, 10);
  layout->setSpacing (5);

  //place out widgets
  layout->addWidget (label, 0, 0);
  layout->addWidget (box, 1, 0);
  layout->addWidget (buttons, 1, 1);

  //update the listbox
  updateMe ();

  //connect() the whole stuff!
  connect (btadd, SIGNAL (clicked ()), this, SLOT (add ()));
  connect (btmodify, SIGNAL (clicked ()), this, SLOT (modify ()));
  connect (btdelete, SIGNAL (clicked ()), this, SLOT (remove ()));
  connect (btrun, SIGNAL (clicked ()), this, SLOT (run ()));
  connect (btup, SIGNAL (clicked ()), this, SLOT (up ()));
  connect (btdown, SIGNAL (clicked ()), this, SLOT (down ()));
  connect (btsort, SIGNAL (clicked ()), this, SLOT (sort ()));
  connect (box, SIGNAL (doubleClicked (QListBoxItem *)), this, SLOT (modify ()));
  connect (box, SIGNAL (returnPressed (QListBoxItem *)), this, SLOT (modify ()));
}

dlgScripts::~dlgScripts()
{
  //NOTHING HERE
}

void dlgScripts::add ()
{
  //we show Edit script dialog
  dlgEditScript *edit = NULL;
  cScript *item;
  item = new cScript (list->session());
  item->setWorkDir (list->session()->connection()->getConnPrefs()->workDir());
  edit = new dlgEditScript (item, list->session(), this);
  if ((edit != NULL) && (edit->showMe ()))
  {
    //item now contains new data, so just add it to the list!
    list->addToEnd (item);
    updateMe ();
  }
  else
    delete item;
  if (edit != NULL)
    delete edit;

  int it = list->count () - 1;
  box->setSelected (it, true);
  box->setCurrentItem (it);
}

void dlgScripts::modify ()
{
  int it = box->currentItem ();
  if (it < 0) return;
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough
  list->reset ();
  for (int i = 0; i < it; i++)
    (*list)++;
  cScript *field = (cScript *) **list;
  dlgEditScript *edit;
  edit = new dlgEditScript (field, list->session(), this);
  if (edit->showMe ())
    //if he pressed OK, field already contains new data -> we only have to
    //update the listbox...
    updateMe ();
  delete edit;

  box->setSelected (it, true);
  box->setCurrentItem (it);
}

void dlgScripts::remove ()
{
  //get that item
  int it = box->currentItem ();
  if (it < 0) return;
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough
  list->reset ();
  for (int i = 0; i < it; i++)
    (*list)++;

  //ask: really?
  QString q1, q2;
  q1 = i18n("Do you really want to delete this script?");
  q2 = i18n ("Delete script");
  int n = KMessageBox::questionYesNoCancel (this, q1, q2);
  if (n == KMessageBox::Yes)
  {
    //really!
    list->removeCurrent ();
    updateMe ();
  }
}

void dlgScripts::run ()
{
  int it = box->currentItem ();
  if (it < 0) return;
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough
  list->reset ();
  //find it
  for (int i = 0; i < it; i++)
    (*list)++;
  //run it
  if (list != 0)
    list->runCurrent (); //NO PARAMETERS SUPPORT (YET)
}

void dlgScripts::up ()
{
  int it = box->currentItem ();
  if (it < 0) return;
  if (it == 0) return;    //current item is the topmost one
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough
  list->reset ();
  for (int i = 0; i < it; i++)
    (*list)++;
  list->moveCurrentToFront ();
  updateMe ();
  box->setCurrentItem (it - 1);
}

void dlgScripts::down ()
{
  int it = box->currentItem ();
  if (it < 0) return;
  if (it == box->count () - 1) return;    //current item is the bottommost one
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough
  list->reset ();
  for (int i = 0; i < it; i++)
    (*list)++;
  list->moveCurrentToBack ();
  updateMe ();
  box->setCurrentItem (it + 1);
}

void dlgScripts::sort ()
{
  QString q1, q2;
  q1 = i18n("Do you really want to sort the scripts alphabetically?");
  q2 = i18n ("Sort scripts");
  int n = KMessageBox::questionYesNoCancel (this, q1, q2);
  if (n == KMessageBox::Yes)
  {
    //really, sort!

    //the map template implements a sorted associative container with
    //O(log n) insertion time. We take advantage of this to easily create
    //an O(n log n) sorting algorithm.

    //First of all, we fill the map.
    std::map<QString, cScript *> scriptMap;
    std::map<QString, cScript *>::iterator it;
    for (list->reset(); (*list); (*list)++)
      scriptMap.insert
          (std::pair<QString, cScript *> (((cScript *) **list)->getName(),
          (cScript *) **list));

    //Good, map is filled. Now clear the list without deleting anything.
    list->clear (true);

    //And now re-create that list
    for (it = scriptMap.begin(); it != scriptMap.end(); ++it)
      list->addToEnd (it->second);

    //Good. Update the listbox and we're done.
    box->setCurrentItem (0);
    updateMe ();
  }
}

void dlgScripts::updateMe ()
{
  //remember old index
  int idx = box->currentItem ();

  box->clear ();
  for (list->reset(); (*list); (*list)++)
  {
    //fetch script name and comment
    QString name = ((cScript*)**list)->getName() + " (" + ((cScript*)**list)->getComment() + ")";
    box->insertItem (name);
  }

  //make one item active, trying to be intelligent...
  box->setCurrentItem ((idx == -1) ? 0 : idx);
  if ((box->currentItem () == -1) && (idx != -1))
    box->setCurrentItem (idx - 1);
  if (box->currentItem () == -1)
    box->setCurrentItem (0);
}
