/***************************************************************************
                          dlgstatistics.cpp  -  Statistics dialog
                             -------------------
    begin                : Pi mar 14 2003
    copyright            : (C) 2003 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../dialogs/dlgstatistics.h"

#include "../csendcmd.h"
#include "../csession.h"
#include "../cstatus.h"
#include "../ctelnet.h"
#include "../ctextprocessor.h"
#include "../kmuddy.h"

#include <config.h>

#include <qlabel.h>
#include <qlayout.h>
#include <qtimer.h>
#include <klistview.h>
#include <klocale.h>

extern KMuddy *kmuddy;

dlgStatistics::dlgStatistics (QWidget *parent, const char *name ) :
    KDialogBase (parent, name, false, i18n ("Connection statistics"), 0)
{
  //initial size
  setInitialSize (QSize (250, 300));

  //main widget
  QWidget *page = new QWidget (this);
  QVBoxLayout *layout = new QVBoxLayout (page);

  setMainWidget (page);

  setFocusPolicy (QWidget::StrongFocus);

  QLabel *label = new QLabel (i18n ("&Statistics, updated every second:"), page);
  view = new KListView (page);
  label->setBuddy (view);
  view->addColumn (i18n ("Counter"));
  view->addColumn (i18n ("Value"));
  view->setColumnWidthMode (0, QListView::Maximum);
  view->setColumnWidthMode (1, QListView::Maximum);
  view->setSorting (-1);
  view->setSelectionMode (QListView::NoSelection);
  
  layout->setSpacing (5);
  layout->addWidget (label);
  layout->addWidget (view);

  //establish auto-updating of data
  updateTimer = new QTimer;
  connect (updateTimer, SIGNAL (timeout()), this, SLOT (update()));
  updateTimer->start (1000);
  
  //update the view!
  update ();  
}

dlgStatistics::~dlgStatistics ()
{
  updateTimer->stop ();
  delete updateTimer;
}

void dlgStatistics::update ()
{
  view->clear ();

  view->setSelectionMode (QListView::NoSelection);

  //fetch required information
  cSession *sess = kmuddy->getSession (kmuddy->activeSession());

  int compr = sess->telnet()->compressedBytes();
  int decompr = sess->telnet()->uncompressedBytes();
  QString connTime = sess->status()->connTimeString();
  int sentbytes = sess->telnet()->sentBytes();
  int sentcmds = sess->sendcmd()->sentCommands();
  int recvlines = sess->textproc()->linesReceived();
  bool usingmccp = sess->telnet()->usingMCCP();
  int mccpver = sess->telnet()->MCCPVer();
#ifdef HAVE_SOUND
  bool usingmsp = sess->telnet()->usingMSP();
#endif
  
  //fill in those information

  //pointer is used to store last item; by default, new items are added to
  //the top of the view, which is wrong
  QListViewItem *pointer = 0;
  pointer = new QListViewItem (view, pointer, i18n ("Using MCCP"),
      usingmccp ? i18n ("Yes") : i18n ("No"));
  pointer = new QListViewItem (view, pointer, i18n ("MCCP version"),
      usingmccp ? QString::number (mccpver) : i18n ("N/A"));
#ifdef HAVE_SOUND
  pointer = new QListViewItem (view, pointer, i18n ("Using MSP"),
      usingmsp ? i18n ("Yes") : i18n ("No"));
#endif
  pointer = new QListViewItem (view, pointer, i18n ("Sent bytes"),
      QString::number (sentbytes));
  pointer = new QListViewItem (view, pointer, i18n ("Sent commands"),
      QString::number (sentcmds));
  pointer = new QListViewItem (view, pointer, i18n ("Received bytes"),
      QString::number (compr));
  pointer = new QListViewItem (view, pointer, i18n ("... after decompression"),
      QString::number (decompr));
  pointer = new QListViewItem (view, pointer, i18n ("Received lines"),
      QString::number (recvlines));
  pointer = new QListViewItem (view, pointer, i18n ("Connection time"),
      connTime);

  view->triggerUpdate ();
}
