/***************************************************************************
                          dlgtranscript.cpp  -  Session Transcript dialog
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : P nov 1 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../dialogs/dlgtranscript.h"

#include <qbutton.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qwhatsthis.h>
#include <kfiledialog.h>
#include <klineedit.h>
#include <klocale.h>

dlgTranscript::dlgTranscript (QWidget *parent, const char *name) :
    KDialogBase (parent, name, true, i18n ("Session Transcript"), Ok|Cancel|Apply)
{
  //initial dialog size
  setInitialSize (QSize (400, 200));

  //create main dialog's widget
  QWidget *page = new QWidget (this);
  QVBoxLayout *layout = new QVBoxLayout (page);

  setMainWidget (page);

  //enable transcript checkbox
  enabletranscript = new QCheckBox (i18n ("&Enable session transcript"), page);
  connect (enabletranscript, SIGNAL (toggled (bool)), this, SLOT (updateDialog (bool)));
  QWhatsThis::add (enabletranscript, i18n ("Enables or disabled the session transcript. "
      "Note that session transcript is automatically disabled when you disconnect."));
  
  //transcript settings
  transbox = new QGroupBox (1, Horizontal, i18n ("Transcript settings"), page);
  transbox->setInsideSpacing (5);
  overwrite = new QCheckBox (i18n ("&Overwrite existing file"), transbox);
  QWhatsThis::add (overwrite, i18n ("If you enable this option, existing "
      "transcript file will be overwritten. Otherwise, new transcript will "
      "be appended to the end of an existing file (if it exists :))."));
  allowansi = new QCheckBox (i18n ("&Include ANSI color sequences"), transbox);
  QWhatsThis::add (allowansi, i18n ("Enabling this will cause color sequences "
      "to be included in the transcript file. You will usually want to enable "
      "this option, unless you'll be viewing it in a viewer that doesn't support "
      "ANSI colors."));
  includedump = new QCheckBox (i18n ("Include buffer &dump"), transbox);
  QWhatsThis::add (includedump, i18n ("This will cause KMuddy to add "
      "complete buffer dump to the beginning of the transcript file.\n"
      "Note that buffer dump currently does not support storing ANSI "
      "sequences."));
  
  //the file name
  QFrame *fileframe = new QFrame (transbox);
  QHBoxLayout *filelayout = new QHBoxLayout (fileframe);
  lblname = new QLabel (i18n ("&File:"), fileframe);
  fname = new KLineEdit (fileframe);
  lblname->setBuddy (fname);
  filebutton = new QPushButton (i18n ("&Browse..."), fileframe);

  filelayout->setSpacing (5);
  filelayout->addWidget (lblname);
  filelayout->addWidget (fname);
  filelayout->addWidget (filebutton);

  //connect the Browse button!
  connect (filebutton, SIGNAL (clicked ()), this, SLOT (browseFiles ()));
  
  layout->setSpacing (5);
  layout->addWidget (enabletranscript);
  layout->addWidget (transbox);

  enableadvtranscript = new QCheckBox (i18n ("Enable &advanced transcript"), page);
  connect (enableadvtranscript, SIGNAL (toggled (bool)), this, SLOT (updateAdvDialog (bool)));
  advtrans = new QGroupBox(1, Horizontal, i18n ("Advanced transcript settings"), page);
  advtrans->setInsideSpacing (5);
  
  rotatedaily = new QCheckBox (i18n ("&Rotate log at midnight every day"), advtrans);
  QWhatsThis::add (rotatedaily, i18n ("This will cause KMuddy to rotate session transcript "
				      "every day at midnight"));
  
  includetimestamp = new QCheckBox (i18n ("Prepend lines with &timestamp"), advtrans);
  QWhatsThis::add (includetimestamp, i18n ("This option will cause KMuddy to insert local "
					   "timestamp in front of every line of logging"));
  
  layout->addWidget (enableadvtranscript);
  layout->addWidget (advtrans);
  
  //initial dialog settings...
  enabletranscript->setChecked (false);
  enableadvtranscript->setChecked (false);
  updateDialog (false);
  updateAdvDialog (false);
}

dlgTranscript::~dlgTranscript ()
{

}

void dlgTranscript::updateDialog (bool how)
{
  overwrite->setEnabled (how);
  allowansi->setEnabled (how);
  fname->setEnabled (how);
  transbox->setEnabled (how);
  lblname->setEnabled (how);
  filebutton->setEnabled (how);
}

void dlgTranscript::updateAdvDialog (bool how)
{
  rotatedaily->setEnabled(how);
  includetimestamp->setEnabled(how);
}

void dlgTranscript::browseFiles ()
{
  fname->setText (KFileDialog::getSaveFileName ());
}

QString dlgTranscript::getFName ()
{
  return fname->text ();
}

void dlgTranscript::setFName (const QString &filename)
{
  fname->setText (filename);
}

bool dlgTranscript::isEnabled ()
{
  return enabletranscript->isChecked ();
}

void dlgTranscript::setEnabled (bool val)
{
  enabletranscript->setChecked (val);
}

bool dlgTranscript::isOverwrite ()
{
  return overwrite->isChecked ();
}

void dlgTranscript::setOverwrite (bool val)
{
  overwrite->setChecked (val);
}

bool dlgTranscript::isAllowANSI ()
{
  return allowansi->isChecked ();
}

void dlgTranscript::setAllowANSI (bool val)
{
  allowansi->setChecked (val);
}

bool dlgTranscript::includeDump ()
{
  return includedump->isChecked ();
}

void dlgTranscript::setIncludeDump (bool val)
{
  includedump->setChecked (val);
}

void dlgTranscript::setAdvTranscript(bool val)
{
  enableadvtranscript->setChecked(val);
}

bool dlgTranscript::advTranscript()
{
  return enableadvtranscript->isChecked();
}

void dlgTranscript::setRotateDaily(bool val)
{
  rotatedaily->setChecked(val);
}

bool dlgTranscript::rotateDaily()
{
  return rotatedaily->isChecked();
}

void dlgTranscript::setIncludeTimestamp(bool val)
{
  includetimestamp->setChecked(val);
}

bool dlgTranscript::includeTimestamp()
{
  return includetimestamp->isChecked();
}
