/***************************************************************************
                          dlgvariables.cpp  -  Variables dialog
                             -------------------
    begin                : St sep 10 2003
    copyright            : (C) 2003 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "../dialogs/dlgvariables.h"
#include "../dialogs/dlgeditvariable.h"
#include "../cvariablelist.h"

#include <qlabel.h>
#include <qlayout.h>
#include <klocale.h>
#include <kpushbutton.h>
#include <kapp.h>
#include <kmessagebox.h>

dlgVariables::dlgVariables (cVariableList *vlist, QWidget *parent, const char *name) :
    KDialogBase (parent, name, true, i18n ("Variables"), Ok, Ok, true)
{
  vl = vlist;
  
  createDialog ();
}

dlgVariables::~dlgVariables ()
{
}

void dlgVariables::createDialog ()
{
  //initial dialog size
  setInitialSize (QSize (500, 400));

  //create main dialog's widget
  QWidget *page = new QWidget (this);
  QGridLayout *layout = new QGridLayout (page, 2, 2);

  setMainWidget (page);

  //change button text
  setButtonOKText (i18n ("Done"));

  //create widgets
  QLabel *label = new QLabel (i18n ("&List of variables"), page);
  box = new KListBox (page);
  label->setBuddy (box);
  QFrame *buttons = new QFrame (page);
  QVBoxLayout *buttonslayout = new QVBoxLayout (buttons);

  btadd = new KPushButton (i18n ("&Add..."), buttons);
  btmodify = new KPushButton (i18n ("&Modify..."), buttons);
  btdelete = new KPushButton (i18n ("&Delete"), buttons);

  buttonslayout->setSpacing (5);
  buttonslayout->addWidget (btadd);
  buttonslayout->addWidget (btmodify);
  buttonslayout->addWidget (btdelete);
  buttonslayout->addStretch (10);

  layout->setColStretch (0, 10);
  layout->setRowStretch (1, 10);
  layout->setSpacing (5);

  //place out widgets
  layout->addWidget (label, 0, 0);
  layout->addWidget (box, 1, 0);
  layout->addWidget (buttons, 1, 1);

  //update the listbox
  updateMe ();
  
  //connect() the whole stuff!
  connect (btadd, SIGNAL (clicked ()), this, SLOT (add ()));
  connect (btmodify, SIGNAL (clicked ()), this, SLOT (modify ()));
  connect (btdelete, SIGNAL (clicked ()), this, SLOT (remove ()));
  connect (box, SIGNAL (doubleClicked (QListBoxItem *)), this, SLOT (modify ()));
}

void dlgVariables::add ()
{
  //we show Edit variable dialog
  dlgEditVariable *edit = new dlgEditVariable (this);
  edit->setName ("");
  edit->setValue ("");
  if (edit->showMe ())
  {
    QString varname = edit->getName ();
    if (vl->exists (varname))
    {
      //this is add(), not modify()
      KMessageBox::sorry (this, "Such variable already exists.");
      return;
    }
    //item now contains new data, so just add it to the list!
    vl->set (edit->getName (), edit->getValue ());
    updateMe (edit->getName ());
  }
  delete edit;
}

void dlgVariables::modify ()
{
  int it = box->currentItem ();
  if (it < 0) return;
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough
  QStringList vars = vl->getList ();
  QStringList::iterator iter = vars.begin();
  for (int i = 0; i < it; i++)
    iter++;
  QString varname = *iter;
  QString value = vl->getValue (varname);
  dlgEditVariable *edit = new dlgEditVariable (this);
  edit->setName (varname);
  edit->setValue (value);
  edit->setModifyMode ();   //don't allow changing variable name
  if (edit->showMe ())
  {
    vl->set (varname, edit->getValue ());
    updateMe (varname);
  }
  delete edit;

  box->setSelected (it, true);
  box->setCurrentItem (it);
}

void dlgVariables::remove ()
{
  //get that item
  int it = box->currentItem ();
  if (it < 0) return;
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough

//*** we don't ask anything - variables are easy to create :-) ***
  QStringList vars = vl->getList ();
  QStringList::iterator iter = vars.begin();
  for (int i = 0; i < it; i++)
    iter++;
  vl->unset (*iter);
  updateMe ();
}

void dlgVariables::updateMe (const QString &activeValue)
{
  //remember old index
  int idx = box->currentItem ();
  bool activated = false;
  int cur = 0;
  
  //remove old items
  box->clear ();
  
  QStringList vars = vl->getList();
  QStringList::iterator it;
  for (it = vars.begin(); it != vars.end(); ++it)
  {
    QString vname = *it;
    QString value = vl->getValue(vname);
    box->insertItem ("$" + vname + " = " + value);
    if (vname == activeValue)
    {
      box->setCurrentItem (cur);
      box->setSelected (cur, true);
      activated = true;
    }
    cur++;
  }
  
  //make one item active, trying to be intelligent...
  //but only of none has been activated yet
  if (!activated)
  {
    box->setCurrentItem ((idx == -1) ? 0 : idx);
    if ((box->currentItem () == -1) && (idx != -1))
      box->setCurrentItem (idx - 1);
    if (box->currentItem () == -1)
      box->setCurrentItem (0);
  }

  box->setFocus ();
}

