//
// C++ Implementation: dlgvartriglist
//
// Description:
//
//
// Author: Tomas Mecir <kmuddy@kmuddy.org>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "dlgvartriglist.h"

#include "dlgeditvartrig.h"
#include "../cvartrigger.h"
#include "../cvartriggerlist.h"

#include <klistbox.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kpushbutton.h>

#include <qlabel.h>
#include <qlayout.h>

dlgVarTrigList::dlgVarTrigList (cVarTriggerList *vt, QWidget *parent, const char *name) :
    KDialogBase (parent, name, true, i18n ("Variable triggers"), Ok, Ok, true)
{
  vtlist = vt;

  createDialog ();
}


dlgVarTrigList::~dlgVarTrigList()
{
}

void dlgVarTrigList::createDialog ()
{
  //initial dialog size
  setInitialSize (QSize (500, 400));

  //create main dialog's widget
  QWidget *page = new QWidget (this);
  QGridLayout *layout = new QGridLayout (page, 2, 2);

  setMainWidget (page);

  //change button text
  setButtonOKText (i18n ("Done"));

  //create widgets
  QLabel *label = new QLabel (i18n ("&List of variable triggers"), page);
  box = new KListBox (page);
  label->setBuddy (box);
  QFrame *buttons = new QFrame (page);
  QVBoxLayout *buttonslayout = new QVBoxLayout (buttons);

  btadd = new KPushButton (i18n ("&Add..."), buttons);
  btmodify = new KPushButton (i18n ("&Modify..."), buttons);
  btdelete = new KPushButton (i18n ("&Delete"), buttons);

  buttonslayout->setSpacing (5);
  buttonslayout->addWidget (btadd);
  buttonslayout->addWidget (btmodify);
  buttonslayout->addWidget (btdelete);
  buttonslayout->addStretch (10);

  layout->setColStretch (0, 10);
  layout->setRowStretch (1, 10);
  layout->setSpacing (5);

  //place out widgets
  layout->addWidget (label, 0, 0);
  layout->addWidget (box, 1, 0);
  layout->addWidget (buttons, 1, 1);

  //update the listbox
  updateMe ();

  //connect() the whole stuff!
  connect (btadd, SIGNAL (clicked ()), this, SLOT (add ()));
  connect (btmodify, SIGNAL (clicked ()), this, SLOT (modify ()));
  connect (btdelete, SIGNAL (clicked ()), this, SLOT (remove ()));
  connect (box, SIGNAL (doubleClicked (QListBoxItem *)), this, SLOT (modify ()));
}

void dlgVarTrigList::add ()
{
  //we show Edit variable trigger dialog
  cVarTrigger *vartrig = new cVarTrigger (vtlist->session());
  dlgEditVarTrig *edit = new dlgEditVarTrig (vartrig, this);
  if (edit->showMe ())
  {
    //item now contains new data, so just add it to the list!
    vtlist->addToEnd (vartrig);
    updateMe ();
  }
  else
    delete vartrig;
  delete edit;

  int it = vtlist->count () - 1;
  box->setSelected (it, true);
  box->setCurrentItem (it);
}

void dlgVarTrigList::modify ()
{
  int it = box->currentItem ();
  if (it < 0) return;
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough
  vtlist->reset ();
  for (int i = 0; i < it; i++)
    (*vtlist)++;
  cVarTrigger *vartrig = (cVarTrigger *) **vtlist;
  dlgEditVarTrig *edit = new dlgEditVarTrig (vartrig, this);
  if (edit->showMe ())
    //if he pressed OK, field already contains new data -> we only have to
    //update the listbox...
    updateMe ();
  delete edit;

  box->setSelected (it, true);
  box->setCurrentItem (it);
}

void dlgVarTrigList::remove ()
{
  //get that item
  int it = box->currentItem ();
  if (it < 0) return;
  if (!box->isSelected (it)) return;  //item must be also selected, focus is NOT enough

  //ask: really?
  int n = KMessageBox::questionYesNoCancel (this,
        i18n ("Do you really want to delete this variable trigger?"),
        i18n ("Delete variable trigger"));
  if (n == KMessageBox::Yes)
  {
    //really!
    //current item MUST be found AFTER the message box invocation; otherwise
    //someone could modify it...
    vtlist->reset ();
    for (int i = 0; i < it; i++)
      (*vtlist)++;
    vtlist->removeCurrent ();
    updateMe ();
  }
}

void dlgVarTrigList::updateMe ()
{
  //remember old index
  int idx = box->currentItem ();

  box->clear ();
  for (vtlist->reset(); (*vtlist); (*vtlist)++)
  {
    cVarTrigger *vt = (cVarTrigger *) **vtlist;

    QStringList newtext = vt->getNewText ();
    QString line1;
    if (!newtext.isEmpty())
      line1 = newtext.first();
    if (newtext.count() > 1)
      //there are more commands to send
      line1 += "  [...]";
    QString name = vt->varName () + " -> " + line1;
    box->insertItem (name);
  }

  //make one item active, trying to be intelligent...
  box->setCurrentItem ((idx == -1) ? 0 : idx);
  if ((box->currentItem () == -1) && (idx != -1))
    box->setCurrentItem (idx - 1);
  if (box->currentItem () == -1)
    box->setCurrentItem (0);
}

#include "dlgvartriglist.moc"
