/***************************************************************************
                          kmuddy.h  -  main class that handles interface
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : P Jun 14 12:37:51 CEST 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@kmuddy.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMUDDY_H
#define KMUDDY_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qiconset.h>
#include <qwidget.h>
#include <kmainwindow.h>
#include <qevent.h>

class QPopupMenu;
class QTimer;
class QWidgetStack;
class KAction;
class KHelpMenu;
class KMenuBar;
class KToggleAction;
class KToolBar;

class dlgConnect;
class dlgQuickConnect;
class dlgEditProfile;
class dlgGrabKey;
class dlgGroups;
class dlgRunningList;
class dlgStatistics;

class cSession;
class cSoundPlayer;
class cTabBar;

//there can be no more than 50 sessions
#define MAXSESSIONS 50
//flash time shall be 500ms
#define FLASHTIME 500

/** KMuddy is the base class of the project */
class KMuddy : public KMainWindow
{
  Q_OBJECT 
public:
  /** constructor */
  KMuddy(QWidget* parent=0, const char *name=0);
  /** destructor */
  ~KMuddy();

  //these functions enable/disable menu/toolbar functions
  //they are called by cConnection
  
  /** disables options valid only when we're connected */
  void disableConnectedOptions ();
  /** enables all such options, advanced options are still disabled */
  void enableConnectedOptions ();
  /** disables advanced options - all that require a standard (not quick) connection,
  as they need to store settings and so on; this includes aliases, macros, triggers,
  auto-mapper, ... */
  void disableAdvancedOptions ();
  /** enables all advanced options */
  void enableAdvancedOptions ();
  /** enable Close tab option */
  void setClosedConn (bool canCT);
  /** disable Close tab option */
  void unsetClosedConn ();
  void prepareObjects ();

  KToolBar *atool () { return atoolbar; };

  /** session count */
  int sessCount ();
  /** one session */
  cSession *getSession (int which);
  int getSessionNumber (cSession *sess);
  cSession *addSession ();
  bool removeSession (int which, bool dontClose = false);
  void setSession (int which);
  int sessionId (QWidget *widget);
  int activeSession ();
  void setSessionName (int which, QString name);

  /** should the tab bar be displayed even if there is only one connection? */
  bool alwaysTabBar () { return wantTabBar; };
  void setAlwaysTabBar (bool value);

  /** flashing of window caption */
  void setAllowGlobalNotify (bool notify);
  /** flashing of tabbar text */
  void setAllowLocalNotify (bool notify);
  /** flashing always or triggers only? */
  void setAlwaysNotify (bool notify);
  /** auto-connect? And where? */
  void setAutoConnect (const QString & ac);
  
  /** some text has been displayed - we may need to start notification */
  void displayedSomeText (cSession *sess);
  /** some trigger requested notification */
  void requestNotify (cSession *sess);

  /** called by cConnection::updateMenus() when switching tabs */
  void setMenuAliasesEnabled (bool val);
  /** called by cConnection::updateMenus() when switching tabs */
  void setMenuTriggersEnabled (bool val);
  /** called by cConnection::updateMenus() when switching tabs */
  void setMenuTimersEnabled (bool val);
  /** called by cConnection::updateMenus() when switching tabs */
  void setMenuShortcutsEnabled (bool val);

  cSoundPlayer *soundPlayer () { return splayer; };
  cSoundPlayer *midiPlayer () { return mplayer; };

  void setGrabDialog (dlgGrabKey *dlg) { grabdlg = dlg; };
public slots:
  void showAToolbar (bool val);
  void showAliasGroups (bool val);
  void showTriggerGroups (bool val);
  void showRunningScripts (bool val);
  void showStatistics (bool val);
protected slots:

  /** close current tab - only valid if that connection has been closed */
  void closeTab ();
  void showAndHandleConnectDialog ();
  void showAndHandleQuickConnectDialog ();
  /** called when user clicks Connect in dlgQuickConnect */
  void doQuickConnect ();
  /** called when user clicks Connect in dlgConnect */
  void doConnect ();
  /** called by doConnect and when auto-connecting on start-up */
  void doProfileConnect (const QString &name, bool sendNothing);
  void doDisconnect ();
  void doReconnect ();
  /** called when we use File->Quit */
  void quit ();
  
  void sendCommandToActiveSession (const QString &command);
  
  //these are ideal candidates for moving to dlgConnect
  
  /** called when user clicks Add in dlgConnect */
  void addPressed ();
  /** called when user clicks Modify in dlgConnect */
  void modifyPressed ();
  /** called when user clicks Delete in dlgConnect */
  void deletePressed ();
  /** called when user clicks Duplicate in dlgConnect */
  void duplicatePressed ();
  /** called when user clicks Ok in dlgEditProfile - Add profile */
  void doAdd ();
  /** called when user clicks Ok in dlgEditProfile - Modify profile */
  void doModify ();

  void changeSession (int id);
  void uncheckMultiLine ();
  void aboutApp ();

  /** lots of slots that handle menus; were handled by slots in objects,
  now we have cSession, so we have to handle them here */
  void addSelectionToClipboard ();
  void setParsing (bool value);
  void clearCommandQueue ();
  void handleAliasesDialog ();
  void handleTriggersDialog ();
  void handleActionsDialog ();
  void handleScriptsDialog ();
  void handleTimersDialog ();
  void handleShortcutsDialog ();
  void handleVariablesDialog ();
  void handleVarTrigsDialog ();
  void handleWindowsDialog ();
  void switchAliases (bool value);
  void switchTriggers (bool value);
  void switchTimers (bool value);
  void switchShortcuts (bool value);
  void showConnPrefsDialog ();
  void configureTranscript ();
  void dumpBuffer ();
  void makeDecision ();
  void importProfile ();
  void exportProfile ();
  void lineUp ();
  void lineDown ();
  void pageUp ();
  void pageDown ();
  void aconUp ();
  void aconDown ();

  void aliasGroupsFinished ();
  void triggerGroupsFinished ();
  void runningScriptsFinished ();
  void statisticsFinished ();

  void handleFlashing ();

  void prevTab ();
  void nextTab ();
  void switchTab1 () { switchTab(0); };
  void switchTab2 () { switchTab(1); };
  void switchTab3 () { switchTab(2); };
  void switchTab4 () { switchTab(3); };
  void switchTab5 () { switchTab(4); };
  void switchTab6 () { switchTab(5); };
  void switchTab7 () { switchTab(6); };
  void switchTab8 () { switchTab(7); };
  void switchTab9 () { switchTab(8); };
  void switchTab10 () { switchTab(9); };
protected:
  void killObjects ();
  /** called when we try to close the window (Alt+F4 or the Close icon) */
  virtual bool queryClose ();
  
  /** event filter, used to grab macro keys */
  bool eventFilter (QObject *o, QEvent *e);

  /** save session properties */
  void saveProperties (KConfig *config);
  /** read session properties */
  void readProperties (KConfig *config);

  /** sound/music players */
  cSoundPlayer *splayer, *mplayer;

  dlgGrabKey *grabdlg;
  
  /** Event here. Used to disable flashing when a window gets activated. */
  virtual bool event (QEvent *e);

  /** This function does the real work when showing/hiding the toolbar */
  void doShowAToolbar (bool val);

  /** switch to tab <which> */
  void switchTab (int index);
  
  /** True when the app is about to terminate */
  static bool goingDown;
  
  /**menus*/
  QPopupMenu *connectionMenu, *editMenu, *viewMenu, *profileMenu;
  QPopupMenu *toolsMenu, *settingsMenu, *popup;
  KHelpMenu *helpMenu;
  
  /**action toolbar*/
  KToolBar *atoolbar;

  /** do we want the toolbar? Used to reactivate the toolbar when switching
  from non-profiled connection to a profile-based one (if we want it) */
  bool wantToolbar;
  
  /** should the tab bar be displayed even if there is only one connection? */
  bool wantTabBar;
  
  /** tab bar */
  cTabBar *tabbar;

  /** icons for the tab bar */
  QIconSet iconOk, iconNo, iconFlag;

  //notification
  bool globalnotify, localnotify, alwaysnotify;
  bool globalFlashingNOW;
  QTimer *notifyTimer;
  bool flashVisible; //updated on every flash

  //auto-connect
  QString autoconnect;
  
  //widget stack and sessions
  QWidgetStack *stack;
  int count;
  cSession *session[MAXSESSIONS];
  int tabID[MAXSESSIONS];
  int activesess;

  //actions
  KAction *stdconnect, *qconnect, *disconnect, *reconnect;
  KAction *closetab;
  KAction *clipcopy;
  KToggleAction *parsing;
  KAction *cancelpending;
  KToggleAction *showmultiline, *showatoolbar;
  KToggleAction *showaliasgroups, *showtriggergroups;
  KToggleAction *showrunningscripts;
  KToggleAction *showstatistics;
  KAction *aliases, *triggers ,*actions, *scripts, *timers, *shortcuts;
  KAction *variables, *vartrigs, *windows;
  KToggleAction *switchaliases, *switchtriggers, *switchtimers;
  KToggleAction *switchshortcuts;
  KAction *connprefs;
  KAction *appconfig;
  KAction *sesstranscript, *dumpbuffer, *decide;
  KAction *importprofile, *exportprofile;

  //actions that are not in menus
  KAction *lineup, *linedown, *pageup, *pagedown;
  KAction *aconup, *acondown;
  KAction *prevtab, *nexttab, *tabnum[10];
  
  //some dialog boxes
  dlgConnect *cdlg;
  dlgQuickConnect *qdlg;
  dlgEditProfile *mdlg;
  dlgGroups *agroups, *tgroups;
  dlgRunningList *rdlg;
  dlgStatistics *statdlg;
};

#endif
