/***************************************************************************
                          main.cpp  -  main file for KMuddy
    This file is a part of KMuddy distribution.
                             -------------------
    begin                : P Jun 14 12:37:51 CEST 2002
    copyright            : (C) 2002 by Tomas Mecir
    email                : kmuddy@host.sk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <config.h>

#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <klocale.h>
#include <kapp.h>

#include <signal.h>

#ifdef HAVE_SDL
#include <SDL/SDL.h>  //SDL reimplements main() so this is needed here
#endif  //HAVE_SDL

#include "kmuddy.h"
#include "cglobalsettings.h"

KMuddy *kmuddy;

extern cGlobalSettings *globalsettings;

static const char *description =
  I18N_NOOP("KMuddy is a MUD (Multi-User Dungeon) client for KDE.\n\n\
It already contains various interesting features, more advanced features \
are being added constantly. \n\nMaybe one day KMuddy will be the best client out there \
(for KDE at least...).");


static KCmdLineOptions options[] =
{
  { 0, 0, 0 }
  // currently there are no command line options; I'll probably include
  // options that allow you to directly connect somewhere...
};

int main(int argc, char *argv[])
{
  KAboutData aboutData ("kmuddy", I18N_NOOP("KMuddy"),
    VERSION, description, KAboutData::License_GPL,
    "(c) 2002-2003, Tomas Mecir", 0, "http://www.kmuddy.org/", "kmuddy@kmuddy.org");
  aboutData.addAuthor("Tomas Mecir",
      I18N_NOOP("Main developer and maintainer"), "kmuddy@kmuddy.org");
  aboutData.addAuthor("Vladimir Lazarenko",
      I18N_NOOP("Developer"), "vlad@lazarenko.net");
  aboutData.addCredit("Alex Bache",
      I18N_NOOP("Improvements in output view and aliases/triggers, bugfixes"), "alex.bache@ntlworld.com");
  aboutData.addCredit("Alsherok staff",
      I18N_NOOP("for releasing AFKMud codebase, used to test MCCP/MSP"));
  aboutData.addCredit("Andrea Cascio",
      I18N_NOOP("SuSE RPMs, speed-walking improvements"), "hacksaw@triangolo.it");
  aboutData.addCredit("Orion Elder",
      I18N_NOOP("KMuddy forum, many ideas"), "orion@mudplanet.org");
  aboutData.addCredit("Scott Monachello",
      I18N_NOOP("Improvements in scripting, bugfixes"), "smonachello@yahoo.com");
  aboutData.addCredit("Tyler Montbriand",
      I18N_NOOP("Help with SDL support"), "tsm@accesscomm.ca");

  KCmdLineArgs::init( argc, argv, &aboutData );
  KCmdLineArgs::addCmdLineOptions( options );

  //alarm signal is sometimes causing KMuddy to terminate incorrectly
  //when switching tabs using Alt+number - ignoring the signal, hoping
  //for the best...
  signal (SIGALRM, SIG_IGN);
  
  KApplication a;

/*
  if (a.isRestored ())  //session management
    RESTORE(KMuddy); //DOES NOT WORK - we need kmuddy object in the program
        //and I couldn't find a way to get to it after RESTORE (mainWidget
        //didn't work (?))
  else
  {
*/
    //normal startup
    kmuddy = new KMuddy();
    a.setMainWidget(kmuddy);
/*
  }
*/
  //event handler for macro keys
  a.installEventFilter (kmuddy);
  
  //prepare objects - must be done even if restoring session
  kmuddy->prepareObjects ();
  kmuddy->show();

  int retCode = a.exec ();

  //Looks like KApplication deletes the widget using deleteLater, resulting
  //in double deleting and problems. We therefore keep the object alive,
  //hoping for the best

  //delete kmuddy;
  
  return retCode;
}
